/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: AdminService.java 233 2011-08-08 10:08:06Z otterley $
 *
 */

package org.ow2.sirocco.cloudmanager.admin;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import org.ow2.sirocco.cloudmanager.admin.event.NewEventEvent;
import org.ow2.sirocco.cloudmanager.admin.exception.CloudAdminException;
import org.ow2.sirocco.cloudmanager.admin.exception.DuplicateNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidHostIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidPerfMetricIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidPowerStateException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidSessionException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidUsernameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVMImageException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVMSizeException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVirtualMachineIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVolumeIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.NoRelevantResourcesPoolException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.exception.ProjectInUseException;
import org.ow2.sirocco.cloudmanager.admin.exception.ResourceQuotaExceededException;
import org.ow2.sirocco.cloudmanager.admin.exception.UserAlreadyExistsException;
import org.ow2.sirocco.cloudmanager.admin.exception.UserAlreadyInProjectException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMImageInUseException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMInUseException;
import org.ow2.sirocco.cloudmanager.admin.exception.VolumeInUseException;
import org.ow2.sirocco.cloudmanager.admin.vo.DbParametersVO;
import org.ow2.sirocco.cloudmanager.admin.vo.EventVO;
import org.ow2.sirocco.cloudmanager.admin.vo.ProjectVO;
import org.ow2.sirocco.cloudmanager.admin.vo.ResourceInfoVO;
import org.ow2.sirocco.cloudmanager.admin.vo.ResourceQuotaVO;
import org.ow2.sirocco.cloudmanager.admin.vo.UserVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VMImageVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VMSizeVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineExtendedInfo;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineSpec;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VolumeVO;
import org.ow2.sirocco.cloudmanager.business.Event;
import org.ow2.sirocco.cloudmanager.business.VMImage;
import org.ow2.sirocco.cloudmanager.clustermanager.api.Domain;
import org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricBase;
import org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricInfo;
import org.ow2.sirocco.cloudmanager.clustermanager.api.VolumeSpec;
import org.ow2.sirocco.cloudmanager.common.DbParameters;
import org.ow2.sirocco.cloudmanager.common.jndilocator.ServiceLocator;
import org.ow2.sirocco.cloudmanager.service.IRemoteCloudAdmin;
import org.ow2.sirocco.cloudmanager.service.IRemoteClusterManagerProxy;
import org.ow2.sirocco.cloudmanager.service.IRemoteEventManager;
import org.ow2.sirocco.cloudmanager.service.IRemoteProjectManager;
import org.ow2.sirocco.cloudmanager.service.IRemoteUserManager;
import org.ow2.sirocco.cloudmanager.service.IRemoteVolumeManager;
import org.ow2.sirocco.cloudmanager.service.reservation.api.ReservationException;
import org.ow2.sirocco.cloudmanager.utils.VirtUtils;
import org.ow2.sirocco.vmm.api.VMMException;

/**
 * POJO object that acts as the access point of Sirocco Cloud Manager
 * administration interface
 */
public class AdminService implements Serializable {

    private static final long serialVersionUID = 7488453566710948678L;

    private static Logger logger = Logger.getLogger(AdminService.class.getName());

    private transient IRemoteCloudAdmin cloudAdmin;

    private transient IRemoteEventManager eventManager;

    private transient IRemoteUserManager userManager;

    private transient IRemoteProjectManager projectManager;

    private transient IRemoteVolumeManager volumeManager;

    private transient IRemoteClusterManagerProxy clusterManagerProxy;

    // A session token Id is associated with a username at logging time and
    // removed when logging out
    private static Map<String, String> tokenMap = new HashMap<String, String>();

    /**
     * Constructs a AdminService and binds it to the business tier
     */
    public AdminService() {
        this.initialize();
    }

    private void initialize() {
        try {
            this.cloudAdmin = (IRemoteCloudAdmin) ServiceLocator.getInstance().getRemoteObject("CloudAdminBean");
            this.eventManager = (IRemoteEventManager) ServiceLocator.getInstance().getRemoteObject("EventManagerBean");
            this.userManager = (IRemoteUserManager) ServiceLocator.getInstance().getRemoteObject("UserManagerBean");
            this.projectManager = (IRemoteProjectManager) ServiceLocator.getInstance().getRemoteObject("ProjectManagerBean");
            this.clusterManagerProxy = (IRemoteClusterManagerProxy) ServiceLocator.getInstance().getRemoteObject(
                "ClusterManagerProxyBean");
            this.volumeManager = (IRemoteVolumeManager) ServiceLocator.getInstance().getRemoteObject("VolumeManagerBean");
        } catch (Exception ex) {
            AdminService.logger.severe(ex.getMessage());
        }
        AdminService.logger.info("AdminService initialized");
    }

    private String sessionTokenIdToUsername(final String sessionTokenId) throws InvalidSessionException {
        String userName = AdminService.tokenMap.get(sessionTokenId);
        if (userName == null) {
            throw new InvalidSessionException("invalid token");
        }
        return userName;
    }

    public synchronized String authenticateUser(final String username, final String password) throws InvalidUsernameException {
        Boolean isAuth = this.userManager.authenticateUser(username, password);
        if (isAuth) {
            // Generate a token
            String tokenId = VirtUtils.CreateUniqueKey();
            AdminService.tokenMap.put(tokenId, username);
            return tokenId;
        }
        return null;
    }

    public Boolean userLogout(final String tokenId) {
        AdminService.tokenMap.remove(tokenId);
        return true;
    }

    public String createVirtualMachine(final String projectId, final String username, final VirtualMachineSpec spec,
        final String sessionTokenId) throws CloudAdminException, InvalidUsernameException, InvalidNameException,
        InvalidVMImageException, ResourceQuotaExceededException, DuplicateNameException, InvalidVMSizeException,
        InvalidProjectIdException, PermissionDeniedException, ReservationException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        boolean startVM = true;
        return this.cloudAdmin.createAndStartVirtualMachine(projectId, username, spec, startVM);
    }

    public void createVMImage(final String projectId, final VMImage.Visibility visibility, final String vmId,
        final String name, final String description, final String sessionTokenId) throws CloudAdminException,
        InvalidVirtualMachineIdException, InvalidNameException, DuplicateNameException, InvalidProjectIdException {
        String fromUsername = this.sessionTokenIdToUsername(sessionTokenId);
        this.cloudAdmin.createVMImage(projectId, visibility, vmId, name, description, fromUsername);
    }

    public void destroyVirtualMachine(final String vmId, final String sessionTokenId) throws InvalidVirtualMachineIdException,
        InvalidPowerStateException, PermissionDeniedException, InvalidArgumentException, ReservationException,
        NoRelevantResourcesPoolException, InvalidSessionException {
        String fromUser = this.sessionTokenIdToUsername(sessionTokenId);
        this.cloudAdmin.destroyVirtualMachine(vmId, fromUser);
    }

    public void destroyVMImage(final Integer imageId, final String sessionTokenId) throws InvalidVMImageException,
        VMImageInUseException, PermissionDeniedException, InvalidSessionException {
        String fromUser = this.sessionTokenIdToUsername(sessionTokenId);
        this.cloudAdmin.destroyVMImage(imageId, fromUser);
    }

    public List<UserVO> getAllUsersVo(final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.userManager.getAllUsersVo();
    }

    public List<ProjectVO> getAllProjectVo(final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.projectManager.getAllProjectsVo();
    }

    public List<ProjectVO> getProjectsVoByUsername(final String username, final String sessionTokenId)
        throws InvalidUsernameException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.projectManager.getProjectsVoByUsername(username);
    }

    public void destroyProject(final String projectId, final String sessionTokenId) throws InvalidProjectIdException,
        VMInUseException, VMImageInUseException, InvalidSessionException, VolumeInUseException {
        this.sessionTokenIdToUsername(sessionTokenId);
        this.projectManager.deleteProject(projectId);
    }

    public void addUserToProject(final String username, final String projectId, final String rights, final String sessionTokenId)
        throws UserAlreadyInProjectException, InvalidSessionException {
        String fromUsername = this.sessionTokenIdToUsername(sessionTokenId);
        this.projectManager.addUserToProject(username, projectId, rights, fromUsername);
    }

    public void delUserFromProject(final String username, final String projectId, final String sessionTokenId)
        throws InvalidProjectIdException, VMInUseException, VMImageInUseException, InvalidSessionException {
        String fromUsername = this.sessionTokenIdToUsername(sessionTokenId);
        this.projectManager.delUserFromProject(username, projectId, fromUsername);
    }

    public void createProject(final String projectName, final String desc, final String owner, final String sessionTokenId)
        throws UserAlreadyInProjectException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        this.projectManager.createProject(null, projectName, desc, owner);
    }

    public void createUser(final String username, final String password, final String userMail, final String userFirstName,
        final String userLastName, final String sessionTokenId) throws UserAlreadyExistsException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        this.userManager.createUser(username, password, userMail, userFirstName, userLastName);
    }

    public List<VMImageVO> getAllVMImagesVo(final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.cloudAdmin.getAllVMImagesVo();
    }

    public List<VirtualMachineVO> getAllVirtualMachinesVo(final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.cloudAdmin.getAllVirtualMachinesVo();
    }

    public List<String> getDataCenterLocations(final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.clusterManagerProxy.getDataCenterLocations();
    }

    public UserVO getUserVoByUsername(final String username, final String sessionTokenId) throws InvalidUsernameException,
        InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.userManager.getUserVoByUsername(username);
    }

    public VirtualMachineVO getVirtualMachineVoByVmId(final String vmId, final String sessionTokenId)
        throws InvalidVirtualMachineIdException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.cloudAdmin.getVirtualMachineVoByVmId(vmId);
    }

    public List<VMImageVO> getVMImagesVoByProjectId(final String projectId, final String sessionTokenId)
        throws InvalidProjectIdException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.projectManager.getVMImagesVoByProjectId(projectId);
    }

    public List<VirtualMachineVO> getVirtualMachinesVoByUsername(final String username, final String sessionTokenId)
        throws InvalidUsernameException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.userManager.getVirtualMachinesVoByUsername(username, false);
    }

    public List<VirtualMachineVO> getVirtualMachinesVoByProjectId(final String projectId, final String sessionTokenId)
        throws InvalidProjectIdException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.projectManager.getVirtualMachinesVoByProjectId(projectId);
    }

    public String getVMSizeName(final String vmId, final String sessionTokenId) throws InvalidVirtualMachineIdException,
        InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.cloudAdmin.getVMSizeName(vmId);
    }

    public List<VMSizeVO> getVMSizesVo(final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.cloudAdmin.getVMSizesVo();
    }

    public void rebootVirtualMachine(final String vmId, final String sessionTokenId) throws InvalidVirtualMachineIdException,
        InvalidPowerStateException, PermissionDeniedException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        String fromUsername = AdminService.tokenMap.get(sessionTokenId);
        this.cloudAdmin.rebootVirtualMachine(vmId, fromUsername);
    }

    public void updateVirtualMachineExpirationDate(final String vmId, final Date expirationDate, final String sessionTokenId)
        throws InvalidVirtualMachineIdException, PermissionDeniedException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        String fromUsername = AdminService.tokenMap.get(sessionTokenId);
        this.cloudAdmin.updateVirtualMachineExpirationDate(vmId, expirationDate, fromUsername);
    }

    public void startVirtualMachine(final String vmId, final String sessionTokenId) throws InvalidVirtualMachineIdException,
        InvalidPowerStateException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        this.cloudAdmin.startVirtualMachine(vmId);
    }

    public String getVirtualMachineConsole(final String vmId, final String sessionTokenId)
        throws InvalidVirtualMachineIdException, CloudAdminException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.cloudAdmin.getVirtualMachineConsole(vmId);
    }

    public void stopVirtualMachine(final String vmId, final String sessionTokenId) throws InvalidVirtualMachineIdException,
        InvalidPowerStateException, PermissionDeniedException, InvalidSessionException {
        String fromUsername = this.sessionTokenIdToUsername(sessionTokenId);
        this.cloudAdmin.stopVirtualMachine(vmId, fromUsername);
    }

    public void pauseVirtualMachine(final String vmId, final String sessionTokenId) throws InvalidVirtualMachineIdException,
        InvalidPowerStateException, PermissionDeniedException, InvalidSessionException {
        String fromUsername = this.sessionTokenIdToUsername(sessionTokenId);
        this.cloudAdmin.pauseVirtualMachine(vmId, fromUsername);
    }

    public void unpauseVirtualMachine(final String vmId, final String sessionTokenId) throws InvalidVirtualMachineIdException,
        InvalidPowerStateException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        this.cloudAdmin.unpauseVirtualMachine(vmId);
    }

    public void migrateVirtualMachine(final String vmId, final String destinationHostId, final String sessionTokenId)
        throws InvalidVirtualMachineIdException, InvalidHostIdException, CloudAdminException {
        this.sessionTokenIdToUsername(sessionTokenId);
        this.cloudAdmin.migrateVirtualMachine(vmId, destinationHostId);
    }

    public void updateUserSSHKey(final String username, final String sshKey, final String sessionTokenId)
        throws InvalidUsernameException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        this.userManager.updateUserSSHKey(username, sshKey);
    }

    public void updateVMImage(final VMImageVO image, final String sessionTokenId) throws InvalidVMImageException,
        InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        this.cloudAdmin.updateVMImage(image);
    }

    public ResourceQuotaVO getProjectResourceQuotaVo(final String projectId, final String sessionTokenId)
        throws InvalidProjectIdException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.projectManager.getProjectResourceQuotaVo(projectId);
    }

    public void updateProjectResourceQuota(final String projectId, final ResourceQuotaVO quota, final String sessionTokenId)
        throws InvalidProjectIdException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        this.projectManager.updateProjectResourceQuota(projectId, quota);
    }

    public void updateUserPasswords(final String username, final String webPassword, final String apiPassword,
        final String sessionTokenId) throws InvalidUsernameException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        this.userManager.updateUserPasswords(username, webPassword, apiPassword);
    }

    public Domain getDataCenterResourceTree(final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.clusterManagerProxy.getDataCenterResourceTree();
    }

    public PerfMetricInfo[] getHostPerfMetricInfos(final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.clusterManagerProxy.getHostPerfMetricInfos();
    }

    public PerfMetricBase[] getHostPerfMetrics(final String hostId, final String metricId, final Date startTime,
        final Date endTime, final String sessionTokenId) throws InvalidHostIdException, InvalidPerfMetricIdException,
        VMMException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.clusterManagerProxy.getHostPerfMetrics(hostId, metricId, startTime, endTime);
    }

    public float[] getVirtualMachinesCPULoad(final String[] vmIds, final String sessionTokenId)
        throws InvalidVirtualMachineIdException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.cloudAdmin.getVirtualMachinesCPULoad(vmIds);
    }

    public VirtualMachineExtendedInfo getVirtualMachineExtendedInfo(final String vmId, final String sessionTokenId)
        throws InvalidVirtualMachineIdException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.cloudAdmin.getVirtualMachineExtendedInfo(vmId);
    }

    public long[] getVirtualMachinesMemoryUsedMB(final String[] vmIds, final String sessionTokenId)
        throws InvalidVirtualMachineIdException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.cloudAdmin.getVirtualMachinesMemoryUsedMB(vmIds);
    }

    public PerfMetricInfo[] getVirtualMachinePerfMetricInfos(final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.clusterManagerProxy.getVirtualMachinePerfMetricInfos();
    }

    public PerfMetricBase[] getVirtualMachinePerfMetrics(final String vmId, final String metricId, final Date startTime,
        final Date endTime, final String sessionTokenId) throws InvalidVirtualMachineIdException, InvalidPerfMetricIdException,
        VMMException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.clusterManagerProxy.getVirtualMachinePerfMetrics(vmId, metricId, startTime, endTime);
    }

    public List<VirtualMachineVO> getVirtualMachinesVoByHostId(final String hostId, final String sessionTokenId)
        throws InvalidHostIdException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.cloudAdmin.getVirtualMachinesVoByHostId(hostId);
    }

    public void deleteUser(final String username, final String sessionTokenId) throws InvalidUsernameException,
        VMInUseException, VMImageInUseException, InvalidProjectIdException, ProjectInUseException, PermissionDeniedException,
        InvalidSessionException, VolumeInUseException {
        String fromUsername = this.sessionTokenIdToUsername(sessionTokenId);
        this.userManager.deleteUser(username, fromUsername);
    }

    public List<EventVO> findEventsVo(final Date startTime, final Date endTime, final int pageNumber, final int pageSize,
        final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.eventManager.findEventsVo(startTime, endTime, pageNumber, pageSize);
    }

    public List<EventVO> findEventsVoByProject(final String projectId, final Date startTime, final Date endTime,
        final int pageNumber, final int pageSize, final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.eventManager.findEventsVoByProject(projectId, startTime, endTime, pageNumber, pageSize);
    }

    public int countEvents(final Date startTime, final Date endTime, final String sessionTokenId)
        throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.eventManager.countEvents(startTime, endTime);
    }

    public int countEventsByProject(final String projectId, final Date startTime, final Date endTime,
        final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.eventManager.countEventsByProject(projectId, startTime, endTime);
    }

    public void newEvent(final Event.Level level, final String description, final String projectId, final String objectId,
        final String type, final String detail, final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        Event event = this.eventManager.newEvent(level, description, projectId, objectId, type, detail);
        this.cloudAdmin.emitTopicMessage(new NewEventEvent(EventVO.from(event)), "projectId", projectId);
    }

    public Integer countCPUsByProject(final String projectId, final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.projectManager.countCPUsByProject(projectId);
    }

    public Integer countDiskByProject(final String projectId, final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.projectManager.countDiskByProject(projectId);
    }

    public Integer countRAMByProject(final String projectId, final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.projectManager.countRAMByProject(projectId);
    }

    public Integer countVMByProject(final String projectId, final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.projectManager.countVMByProject(projectId);
    }

    public ResourceInfoVO getResourceInfoVoByProjectId(final String projectId, final String sessionTokenId)
        throws InvalidProjectIdException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.projectManager.getResourceInfoVoByProjectId(projectId);
    }

    public UserVO getUserVoLdapInfoByUsername(final String username, final String sessionTokenId)
        throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.userManager.getUserVoLdapInfoByUsername(username);
    }

    public DbParametersVO getDbParametersInstance(final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return DbParametersVO.from(DbParameters.getInstance());
    }

    public void createVolume(final String projectId, final String username, final VolumeSpec volumeSpec,
        final String sessionTokenId) throws InvalidUsernameException, InvalidNameException, ResourceQuotaExceededException,
        DuplicateNameException, InvalidProjectIdException, PermissionDeniedException, CloudAdminException,
        InvalidVMSizeException, InvalidProjectIdException, PermissionDeniedException, ReservationException,
        InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        this.volumeManager.createVolume(projectId, username, volumeSpec);
    }

    public void destroyVolume(final String volumeId, final String sessionTokenId) throws InvalidUsernameException,
        InvalidProjectIdException, InvalidVolumeIdException, VolumeInUseException, PermissionDeniedException,
        CloudAdminException, InvalidSessionException {
        String fromUser = this.sessionTokenIdToUsername(sessionTokenId);
        this.volumeManager.destroyVolume(fromUser, volumeId);
    }

    public void attachVolumeToVirtualMachine(final String projectId, final String volumeId, final String vmId,
        final String sessionTokenId) throws InvalidUsernameException, InvalidProjectIdException, InvalidVolumeIdException,
        InvalidVirtualMachineIdException, VolumeInUseException, PermissionDeniedException, CloudAdminException {
        String fromUser = this.sessionTokenIdToUsername(sessionTokenId);
        this.volumeManager.attachVolumeToVirtualMachine(projectId, fromUser, volumeId, vmId);
    }

    public void detachVolumeFromVirtualMachine(final String projectId, final String volumeId, final String vmId,
        final String sessionTokenId) throws InvalidUsernameException, InvalidProjectIdException, InvalidVolumeIdException,
        InvalidVirtualMachineIdException, VolumeInUseException, PermissionDeniedException, CloudAdminException {
        String fromUser = this.sessionTokenIdToUsername(sessionTokenId);
        this.volumeManager.detachVolumeFromVirtualMachine(projectId, fromUser, volumeId, vmId);
    }

    public List<VolumeVO> getAllVolumeVo(final String sessionTokenId) throws InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.cloudAdmin.getAllVolumeVo();
    }

    public List<VolumeVO> getVolumesVoByProjectId(final String projectId, final String sessionTokenId)
        throws InvalidProjectIdException, InvalidSessionException {
        this.sessionTokenIdToUsername(sessionTokenId);
        return this.projectManager.getVolumeVoByProjectId(projectId);
    }

    private void readObject(final ObjectInputStream in) throws IOException, ClassNotFoundException {
        try {
            // reconnect EJB3 if needed
            if (this.cloudAdmin == null) {
                this.initialize();
            }
        } catch (Throwable t) {
            // Java 5 doesn't have new IOException(message, cause)
            IOException ioe = new IOException("Cannot read object");
            ioe.initCause(t);
            throw ioe;
        }

        // deserialize everything except for the transient fields
        in.defaultReadObject();
    }
}
