/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: VMImage.java 358 2011-09-30 08:57:27Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.business;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;

@SuppressWarnings("serial")
@Entity
public class VMImage implements Serializable {

    public static enum Status {
        IN_CONSTRUCTION, AVAILABLE, ERROR, DESTROYED
    }

    public static enum Visibility {
        USER, PROJECT, PUBLIC
    }

    private Integer id;

    private String name;

    private String osType;

    private String description;

    private User user;

    private String providerId;

    private String hypervisor;

    private String architecture;

    private Integer diskSizeMB;

    private Integer imageSizeMB;

    private Project project;

    private Status status;

    private Visibility visibility;

    /**
     * Image's format (a part for .iso, it defines the hypervisor on which the
     * image can be "instanciated").
     */
    private String vMImageFormat;

    /**
     * Indicates if the image is bootable (or not).
     */
    private Boolean isBootable;

    /**
     * Indicates the URL from which the VMImage has been uploaded.
     */
    private String uploadedFromURL;

    private Collection<VolumeTemplate> volumeTemplates;

    private Collection<CloudProviderAccount> cloudProviderAccounts;

    public VMImage() {
        // This contructor is required by JPA when using non-empty
        // contructor(s).

        // Let's initialize associations' attributs.
        this.volumeTemplates = new ArrayList<VolumeTemplate>();
        this.cloudProviderAccounts = new ArrayList<CloudProviderAccount>();
    }

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Integer getId() {
        return this.id;
    }

    @ManyToOne
    public User getUser() {
        return this.user;
    }

    @ManyToOne
    public Project getProject() {
        return this.project;
    }

    @Enumerated(EnumType.STRING)
    public Status getStatus() {
        return this.status;
    }

    @Enumerated(EnumType.STRING)
    public Visibility getVisibility() {
        return this.visibility;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    public String getOsType() {
        return this.osType;
    }

    public void setOsType(final String osType) {
        this.osType = osType;
    }

    public String getDescription() {
        return this.description;
    }

    public void setDescription(final String description) {
        this.description = description;
    }

    public void setUser(final User user) {
        this.user = user;
    }

    public void setHypervisor(final String hypervisor) {
        this.hypervisor = hypervisor;
    }

    public String getHypervisor() {
        return this.hypervisor;
    }

    /**
     * @return the id of the VmImage that has been attributed either by VMM, or
     *         by the CloudProvider.
     */
    public String getProviderId() {
        return this.providerId;
    }

    public void setProviderId(final String providerId) {
        this.providerId = providerId;
    }

    public String getArchitecture() {
        return this.architecture;
    }

    public void setArchitecture(final String architecture) {
        this.architecture = architecture;
    }

    public Integer getDiskSizeMB() {
        return this.diskSizeMB;
    }

    public void setDiskSizeMB(final Integer diskSizeMB) {
        this.diskSizeMB = diskSizeMB;
    }

    public Integer getImageSizeMB() {
        return this.imageSizeMB;
    }

    public void setImageSizeMB(final Integer imageSizeMB) {
        this.imageSizeMB = imageSizeMB;
    }

    public void setProject(final Project project) {
        this.project = project;
    }

    public String getName() {
        return this.name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public void setStatus(final Status status) {
        this.status = status;
    }

    public void setVisibility(final Visibility visibility) {
        this.visibility = visibility;
    }

    public String getVMImageFormat() {
        return this.vMImageFormat;
    }

    public void setVMImageFormat(final String vMImageFormat) {
        this.vMImageFormat = vMImageFormat;
    }

    public Boolean getIsBootable() {
        return this.isBootable;
    }

    public void setIsBootable(final Boolean isBootable) {
        this.isBootable = isBootable;
    }

    public String getUploadedFromURL() {
        return this.uploadedFromURL;
    }

    public void setUploadedFromURL(final String uploadedFromURL) {
        this.uploadedFromURL = uploadedFromURL;
    }

    @OneToMany(mappedBy = "virtualMachineImage", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public Collection<VolumeTemplate> getVolumeTemplates() {
        return this.volumeTemplates;
    }

    public void setVolumeTemplates(final Collection<VolumeTemplate> volumeTemplates) {
        this.volumeTemplates = volumeTemplates;
    }

    @ManyToMany(mappedBy = "vmImages")
    public Collection<CloudProviderAccount> getCloudProviderAccounts() {
        return this.cloudProviderAccounts;
    }

    public void setCloudProviderAccounts(final Collection<CloudProviderAccount> cloudProviderAccounts) {
        this.cloudProviderAccounts = cloudProviderAccounts;
    }

    @Override
    public String toString() {
        String s = this.getClass().getSimpleName() + "=[id=" + this.getId() + ", name=" + this.getName() + ", osType="
            + this.getOsType() + ", description=" + this.getDescription() + ", user=" + this.getUser() + ", providerId="
            + this.getProviderId() + ", hypervisor=" + this.getHypervisor() + ", architecture=" + this.getArchitecture()
            + ", diskSizeMB=" + this.getDiskSizeMB() + ", imageSizeMB=" + this.getImageSizeMB() + ", project="
            + this.getProject() + ", status=" + this.getStatus() + ", visibility=" + this.getVisibility() + ", vMImageFormat="
            + this.getVMImageFormat() + ", isBootable=" + this.getIsBootable() + ", uploadedFromURL="
            + this.getUploadedFromURL() + "]";
        return s;
    }

}
