/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: VirtualMachine.java 319 2011-09-09 14:27:34Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.business;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Map;
import java.util.logging.Logger;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.annotations.CollectionOfElements;
import org.ow2.sirocco.cloudmanager.service.monitoring.impl.PerfMetricInfo;

@SuppressWarnings("serial")
@Entity
public class VirtualMachine implements Serializable {

    public static enum Status {
        IN_CONFIGURATION, RUNNING, STOPPED, STARTING, SUSPENDED, PAUSED, STOPPING, BEING_TEMPLATED, DESTROYED, ERROR
    }

    Logger log = Logger.getLogger(VirtualMachine.class.getName());

    private Integer id;

    private String name;

    private String providerId;

    private User user;

    private Integer ramSizeMB;

    private Integer diskSizeMB;

    private Integer nbCpus;

    private String ip;

    private String dnsName;

    private String mac;

    private String machine;

    private VMImage vmImage;

    private Status status;

    private Date createDate;

    private Date expirationDate;

    private Date deleteDate;

    private String location;

    private Project project;

    private Map<String, String> additionalProperties;

    /**
     * If the VM is created using a reservation, it contains the reservationId,
     * otherwise it contains null.
     */
    private Integer reservationId;

    private SystemInstance systemInstance;

    private VirtualMachineTemplate virtualMachineTemplate;

    private Collection<Volume> volumes;

    private Collection<PerfMetricInfo> perfMetricInfos;

    private CloudProviderAccount cloudProviderAccount;

    public VirtualMachine() {
        // This contructor is required by JPA when using non-empty
        // contructor(s).

        // Let's initialize associations' attributs.
        this.volumes = new ArrayList<Volume>();
        this.perfMetricInfos = new ArrayList<PerfMetricInfo>();
    }

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Integer getId() {
        return this.id;
    }

    @Enumerated(EnumType.STRING)
    public Status getStatus() {
        return this.status;
    }

    @ManyToOne
    public User getUser() {
        return this.user;
    }

    @ManyToOne
    public Project getProject() {
        return this.project;
    }

    @OneToOne(cascade = CascadeType.PERSIST, fetch = FetchType.EAGER)
    public VMImage getVmImage() {
        return this.vmImage;
    }

    @Temporal(TemporalType.TIMESTAMP)
    public Date getCreateDate() {
        return this.createDate;
    }

    public String getIp() {
        return this.ip;
    }

    public String getMac() {
        return this.mac;
    }

    public String getName() {
        return this.name;
    }

    public Integer getRamSizeMB() {
        if (this.getVirtualMachineTemplate() == null) {
            return this.ramSizeMB;
        } else {
            return this.getVirtualMachineTemplate().getRamSizeMB();
        }
    }

    /**
     * @return the id of the Vm that has been attributed by the CloudProvider.
     */
    @Column(unique = true)
    public String getProviderId() {
        return this.providerId;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    public void setIp(final String ip) {
        this.ip = ip;
    }

    public void setMac(final String mac) {
        this.mac = mac;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public void setStatus(final Status status) {
        this.status = status;
    }

    public void setDiskSizeMB(final Integer diskSizeMB) {
        this.diskSizeMB = diskSizeMB;
    }

    public void setRamSizeMB(final Integer ramSizeMB) {
        this.ramSizeMB = ramSizeMB;
    }

    public void setUser(final User user) {
        this.user = user;
    }

    public void setProviderId(final String providerId) {
        this.providerId = providerId;
    }

    public void setVmImage(final VMImage vmImage) {
        this.vmImage = vmImage;
    }

    public Integer getNbCpus() {
        if (this.getVirtualMachineTemplate() == null) {
            return this.nbCpus;
        } else {
            return this.getVirtualMachineTemplate().getNbCpus();
        }
    }

    public void setNbCpus(final Integer nbCpus) {
        this.nbCpus = nbCpus;
    }

    public String getLocation() {
        return this.location;
    }

    public void setLocation(final String location) {
        this.location = location;
    }

    public void setCreateDate(final Date createDate) {
        this.createDate = createDate;
    }

    public String getDnsName() {
        return this.dnsName;
    }

    public void setDnsName(final String dnsName) {
        this.dnsName = dnsName;
    }

    public String getMachine() {
        return this.machine;
    }

    public void setMachine(final String machine) {
        this.machine = machine;
    }

    public Integer getDiskSizeMB() {
        return this.diskSizeMB;
    }

    public void setProject(final Project project) {
        this.project = project;
    }

    public void setExpirationDate(final Date expirationDate) {
        this.expirationDate = expirationDate;
    }

    public Date getExpirationDate() {
        return this.expirationDate;
    }

    public void setAdditionalProperties(final Map<String, String> additionalProperties) {
        this.additionalProperties = additionalProperties;
    }

    @CollectionOfElements
    public Map<String, String> getAdditionalProperties() {
        return this.additionalProperties;
    }

    public void setReservationId(final Integer reservationId) {
        this.reservationId = reservationId;
    }

    public Integer getReservationId() {
        return this.reservationId;
    }

    public void setDeleteDate(final Date deleteDate) {
        this.deleteDate = deleteDate;
    }

    @Temporal(TemporalType.TIMESTAMP)
    public Date getDeleteDate() {
        return this.deleteDate;
    }

    @ManyToOne
    @JoinColumn(name = "SystemInstance_id")
    public SystemInstance getSystemInstance() {
        return this.systemInstance;
    }

    public void setSystemInstance(final SystemInstance systemInstance) {
        this.systemInstance = systemInstance;
    }

    @ManyToOne
    @JoinColumn(name = "VirtualMachineTemplate_id")
    public VirtualMachineTemplate getVirtualMachineTemplate() {
        return this.virtualMachineTemplate;
    }

    public void setVirtualMachineTemplate(final VirtualMachineTemplate virtualMachineTemplate) {
        this.virtualMachineTemplate = virtualMachineTemplate;
    }

    @ManyToMany
    @JoinTable(name = "VIRTUALMACHINE_VOLUME")
    /**
     * VirtualMachine est le OWNER, il faut passer par lui pour associer et persister. (VirtualMachine is the OWNER, we must use it (instead of Volume) in order to associate, and persist).
     */
    public Collection<Volume> getVolumes() {
        return this.volumes;
    }

    public void setVolumes(final Collection<Volume> volumes) {
        this.volumes = volumes;
    }

    @OneToMany(cascade = CascadeType.ALL)
    @JoinTable(name = "VM_PERFMETRICINFOS")
    public Collection<PerfMetricInfo> getPerfMetricInfos() {
        return this.perfMetricInfos;
    }

    public void setPerfMetricInfos(final Collection<PerfMetricInfo> perfMetricInfos) {
        this.perfMetricInfos = perfMetricInfos;
    }

    @ManyToOne
    // @JoinColumn(name = "CloudProviderAccount_id", nullable = false)
    @JoinColumn(name = "CloudProviderAccount_id")
    public CloudProviderAccount getCloudProviderAccount() {
        return this.cloudProviderAccount;
    }

    public void setCloudProviderAccount(final CloudProviderAccount cloudProviderAccount) {
        this.cloudProviderAccount = cloudProviderAccount;
    }

    @Override
    public String toString() {
        String s = this.getClass().getSimpleName() + "=[id=" + this.getId() + ", status=" + this.getStatus() + ", name="
            + this.getName() + ", providerId=" + this.getProviderId() + "]";
        return s;
    }

}
