/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: Volume.java 289 2011-08-31 10:01:02Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.business;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

@SuppressWarnings("serial")
@Entity
public class Volume implements Serializable {
    public static enum Status {
        PENDING, AVAILABLE, DESTROYED
    }

    public static enum Visibility {
        USER, PROJECT, PUBLIC
    }

    /**
     * This is the volumeId
     */
    private Integer id;

    private Status status;

    private User user;

    private Project project;

    private String providerId;

    private String name;

    private String description;

    private Date createDate;

    private Date deleteDate;

    /**
     * Storage size in MB/Mo. There is a wanted duplication of the attribut
     * capacityInMB: it appears both in VolumeTemplate, and in Volume.
     */
    private Long capacityInMB;

    /**
     * True defines that it is possible to boot on this volume (it requires a
     * bootable VMImage).
     */
    private Boolean bootableVolume;

    // XXX We must define the volume on which a virtualMachine will
    // boot, especially if it has several volumes, i.e. which volume is the
    // master/primary one ?

    private VolumeTemplate volumeTemplate;

    private Collection<SystemInstance> systemInstances;

    private Collection<VirtualMachine> virtualMachines;

    private Visibility visibility;

    private CloudProviderAccount cloudProviderAccount;

    public Volume() {
        // This contructor is required by JPA when using non-empty
        // contructor(s).

        // Let's initialize associations' attributs.
        this.systemInstances = new ArrayList<SystemInstance>();
        this.virtualMachines = new ArrayList<VirtualMachine>();
    }

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Integer getId() {
        return this.id;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    @Enumerated(EnumType.STRING)
    public Status getStatus() {
        return this.status;
    }

    public void setStatus(final Status status) {
        this.status = status;
    }

    public String getName() {
        return this.name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public String getDescription() {
        return this.description;
    }

    public void setDescription(final String description) {
        this.description = description;
    }

    public Long getCapacityInMB() {
        if (this.getVolumeTemplate() == null) {
            return this.capacityInMB;
        } else {
            return this.getVolumeTemplate().getCapacityInMB();
        }
    }

    public void setCapacityInMB(final Long capacityInMB) {
        this.capacityInMB = capacityInMB;
    }

    public Boolean getBootableVolume() {
        return this.bootableVolume;
    }

    public void setBootableVolume(final Boolean bootableVolume) {
        this.bootableVolume = bootableVolume;
    }

    @ManyToOne
    @JoinColumn(name = "VolumeTemplate_id")
    public VolumeTemplate getVolumeTemplate() {
        return this.volumeTemplate;
    }

    public void setVolumeTemplate(final VolumeTemplate volumeTemplate) {
        this.volumeTemplate = volumeTemplate;
    }

    @ManyToMany(mappedBy = "volumes")
    public Collection<SystemInstance> getSystemInstances() {
        return this.systemInstances;
    }

    public void setSystemInstances(final Collection<SystemInstance> systemInstances) {
        this.systemInstances = systemInstances;
    }

    @ManyToMany(mappedBy = "volumes")
    public Collection<VirtualMachine> getVirtualMachines() {
        return this.virtualMachines;
    }

    public void setVirtualMachines(final Collection<VirtualMachine> virtualMachines) {
        this.virtualMachines = virtualMachines;
    }

    @ManyToOne
    public User getUser() {
        return this.user;
    }

    public void setUser(final User user) {
        this.user = user;
    }

    @ManyToOne
    public Project getProject() {
        return this.project;
    }

    public void setProject(final Project project) {
        this.project = project;
    }

    @Column(unique = true)
    public String getProviderId() {
        return this.providerId;
    }

    public void setProviderId(final String providerId) {
        this.providerId = providerId;
    }

    @Enumerated(EnumType.STRING)
    public Visibility getVisibility() {
        return this.visibility;
    }

    public void setVisibility(final Visibility visibility) {
        this.visibility = visibility;
    }

    @Temporal(TemporalType.TIMESTAMP)
    public Date getCreateDate() {
        return this.createDate;
    }

    public void setCreateDate(final Date createDate) {
        this.createDate = createDate;
    }

    @Temporal(TemporalType.TIMESTAMP)
    public Date getDeleteDate() {
        return this.deleteDate;
    }

    public void setDeleteDate(final Date deleteDate) {
        this.deleteDate = deleteDate;
    }

    @ManyToOne
    // @JoinColumn(name = "CloudProviderAccount_id", nullable = false)
    @JoinColumn(name = "CloudProviderAccount_id")
    public CloudProviderAccount getCloudProviderAccount() {
        return this.cloudProviderAccount;
    }

    public void setCloudProviderAccount(final CloudProviderAccount cloudProviderAccount) {
        this.cloudProviderAccount = cloudProviderAccount;
    }

    @Override
    public String toString() {
        String s = this.getClass().getSimpleName() + "=[id=" + this.getId() + ", name=" + this.getName() + ", description="
            + this.getDescription() + ", capacityInMB=" + this.getCapacityInMB() + ", bootableVolume="
            + this.getBootableVolume() + ", volumeTemplate=" + this.getVolumeTemplate() + "]";
        return s;
    }
}
