/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: ClusterManagerMDB.java 406 2011-10-03 15:50:18Z otterley $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.ejb.ActivationConfigProperty;
import javax.ejb.EJB;
import javax.ejb.MessageDriven;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.Query;

import org.ow2.easybeans.osgi.annotation.OSGiResource;
import org.ow2.sirocco.cloudmanager.admin.event.DataCenterResourceUsageEvent;
import org.ow2.sirocco.cloudmanager.admin.event.VMImageUpdateEvent;
import org.ow2.sirocco.cloudmanager.admin.event.VMMigrationEvent;
import org.ow2.sirocco.cloudmanager.admin.event.VirtualMachineStatusChangeEvent;
import org.ow2.sirocco.cloudmanager.admin.vo.VMImageVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineVO;
import org.ow2.sirocco.cloudmanager.business.Event;
import org.ow2.sirocco.cloudmanager.business.VMImage;
import org.ow2.sirocco.cloudmanager.business.VMImage.Status;
import org.ow2.sirocco.cloudmanager.business.VirtualMachine;
import org.ow2.sirocco.cloudmanager.clustermanager.api.Domain;
import org.ow2.sirocco.cloudmanager.clustermanager.api.IClusterManager;
import org.ow2.sirocco.cloudmanager.clustermanager.api.event.NewPerfMetricsEvent;
import org.ow2.sirocco.cloudmanager.clustermanager.api.event.ResourceUsageEvent;
import org.ow2.sirocco.cloudmanager.clustermanager.api.event.VMImageStatusChangeEvent;
import org.ow2.sirocco.cloudmanager.clustermanager.api.event.VMImageUploadCompletionEvent;
import org.ow2.sirocco.cloudmanager.clustermanager.api.event.VirtualMachineCreatedEvent;
import org.ow2.sirocco.cloudmanager.clustermanager.api.event.VirtualMachineFailureEvent;
import org.ow2.sirocco.cloudmanager.clustermanager.api.event.VirtualMachineGuestOSReadyEvent;
import org.ow2.sirocco.cloudmanager.clustermanager.api.event.VirtualMachineMigrationEvent;
import org.ow2.sirocco.cloudmanager.clustermanager.api.event.VirtualMachinePowerStateChangeEvent;
import org.ow2.sirocco.cloudmanager.clustermanager.api.event.VolumeReadyEvent;
import org.ow2.sirocco.cloudmanager.common.DbParameters;
import org.ow2.sirocco.cloudmanager.service.monitoring.impl.PerfMetricInfo;
import org.ow2.sirocco.cloudmanager.utils.Result;
import org.ow2.sirocco.cloudmanager.utils.SshCommand;
import org.ow2.sirocco.cloudmanager.utils.VirtUtils;
import org.ow2.sirocco.vmm.api.VMMException;
import org.ow2.sirocco.vmm.api.VirtualMachineMXBean;
import org.ow2.sirocco.vmm.api.monitoring.PerfMetric;

import com.jcraft.jsch.JSchException;

@MessageDriven(activationConfig = {
    @ActivationConfigProperty(propertyName = "destinationType", propertyValue = "javax.jms.Topic"),
    @ActivationConfigProperty(propertyName = "destination", propertyValue = "ClusterManager")})
public class ClusterManagerMDB implements MessageListener {
    private Logger log = Logger.getLogger(ClusterManagerMDB.class.getName());

    @EJB(mappedName = "CloudAdminBean")
    ICloudAdmin cloudAdmin;

    @EJB(mappedName = "EventManagerBean")
    private IEventManager eventManager;

    @OSGiResource
    private IClusterManager clusterManager;

    @PersistenceContext(unitName = "persistence-unit/main", type = PersistenceContextType.TRANSACTION)
    protected EntityManager em;

    @TransactionAttribute(TransactionAttributeType.REQUIRED)
    public void onMessage(final Message msg) {
        if (msg instanceof ObjectMessage) {
            ObjectMessage objectMessage = (ObjectMessage) msg;
            Object payload;
            try {
                payload = objectMessage.getObject();
                this.log.info("ClusterManager TOPIC RECEIVED " + payload);
            } catch (JMSException ex) {
                this.log.log(Level.SEVERE, "Failed to extract from JMS message", ex);
                return;
            }
            if (payload instanceof VirtualMachinePowerStateChangeEvent) {
                VirtualMachinePowerStateChangeEvent event = (VirtualMachinePowerStateChangeEvent) payload;
                this.updateVirtualMachineStatusByProviderId(event.getVmProviderId(), event.getState(), false);
            } else if (payload instanceof VirtualMachineMigrationEvent) {
                VirtualMachineMigrationEvent event = (VirtualMachineMigrationEvent) payload;
                this.moveVirtualMachineToHost(event.getVmProviderId(), event.getFromHostId(), event.getToHostId());
            } else if (payload instanceof ResourceUsageEvent) {
                ResourceUsageEvent event = (ResourceUsageEvent) payload;
                this.updateDataCenterResourceUsage(event.getDomain());
            } else if (payload instanceof VirtualMachineGuestOSReadyEvent) {
                VirtualMachineGuestOSReadyEvent event = (VirtualMachineGuestOSReadyEvent) payload;
                this.onVirtualMachineGuestOSReady(event.getVmProviderId(), event.getVmName(), event.getInfos());
            } else if (payload instanceof VirtualMachineFailureEvent) {
                VirtualMachineFailureEvent event = (VirtualMachineFailureEvent) payload;
                this.onVirtualMachineFailure(event.getVmProviderId(), event.getVmName(), event.getFailureType(),
                    event.getFailureDetail());
            } else if (payload instanceof VMImageStatusChangeEvent) {
                VMImageStatusChangeEvent event = (VMImageStatusChangeEvent) payload;
                this.onVMImageStatusChange(event.getImageId(), event.getVmProviderId(), event.getImageProviderId(),
                    event.getImageStatus(), event.getErrorDetail());
            } else if (payload instanceof VMImageUploadCompletionEvent) {
                VMImageUploadCompletionEvent event = (VMImageUploadCompletionEvent) payload;
                this.onVMImageUploadCompletionEvent(event.getVmImageProviderId(), event.getVmImageUrl());
            } else if (payload instanceof VolumeReadyEvent) {
                VolumeReadyEvent event = (VolumeReadyEvent) payload;
                this.onVolumeReady(event.getName(), event.getProviderId(), event.getHandback());
            } else if (payload instanceof VirtualMachineCreatedEvent) {
                VirtualMachineCreatedEvent event = (VirtualMachineCreatedEvent) payload;
                this.onVirtualMachineCreated(event.getVmProviderId(), event.getVmName());
            } else if (payload instanceof NewPerfMetricsEvent) {
                NewPerfMetricsEvent event = (NewPerfMetricsEvent) payload;
                this.onNewPerfMetricsEvent(event.getMetrics());
                // this.log.info("ClusterManagerMDB:Recoit NewPMEvent:" +
                // event.getMetrics().size());
            } else {
                this.log.warning("This event has been received, and ignored: " + payload.getClass().getSimpleName() + ", "
                    + payload.toString() + ".");
            }
        }
    }

    // HashMap <providerId, HashMap<MetricName, PerfMetric[]>>
    private void onNewPerfMetricsEvent(
        final HashMap<String, HashMap<org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricInfo, PerfMetric[]>> metrics) {

        if (metrics != null) {
            // for all VM
            for (String providerId : metrics.keySet()) {
                VirtualMachine v = this.cloudAdmin.getVirtualMachineByProviderId(providerId);
                // this.log.info("\n\n\n Get PMI for provider id:" +
                // v.getProviderId());
                if (v != null) {
                    HashMap<org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricInfo, PerfMetric[]> metrics2 = metrics
                        .get(v.getProviderId());
                    // for each PerfMetricInfo

                    if (metrics2 != null && metrics2.keySet() != null) {
                        // this.log.info("ClusterManagerMDB:vm " + v.getName() +
                        // " sizePMI:" + metrics2.size());
                        for (org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricInfo info : metrics2.keySet()) {
                            // this.log.info("\n\n\n Get after PMI for provider id:"
                            // +
                            // metrics.get(v.getProviderId()));

                            Collection<PerfMetricInfo> list = v.getPerfMetricInfos();

                            for (PerfMetricInfo infob : list) {
                                if (infob.getName().equals(info.getName())
                                    && infob.getIntervalSample().equals(info.getInterval())
                                    && infob.getTypeSample().equals(info.getType())) {
                                    // this.log.info("\n\n\n Get PMI (" +
                                    // info.getName()
                                    // +
                                    // ":" + metrics2.get(info.getName()));
                                    // we save all Perfmetrics
                                    PerfMetric[] pmstmp = metrics2.get(info);
                                    if (pmstmp != null) {
                                        // this.log.info("\n\n\n Save PMs");
                                        for (PerfMetric pmtmp : pmstmp) {
                                            // this.log.info(v.getName() +
                                            // ": Save PM : " + pmtmp.getName()
                                            // + " in "
                                            // + infob.getName());

                                            org.ow2.sirocco.cloudmanager.service.monitoring.impl.PerfMetric pm = new org.ow2.sirocco.cloudmanager.service.monitoring.impl.PerfMetric();
                                            pm.setTimeStamp(new Date(pmtmp.getTime()));
                                            pm.setInfo(infob);
                                            if (Float.isNaN(pmtmp.getValue())) {
                                                pm.setValue(0);
                                            } else {
                                                pm.setValue(pmtmp.getValue());
                                            }

                                            infob.addPerfMetric(pm);
                                            this.em.persist(infob);
                                        }
                                    }
                                    break;
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    private void onVMImageUploadCompletionEvent(final String vMImageProviderId, final String vMImageUrl) {
        VMImage vMImage_temp = null;

        // Check if the VMImage corresponding to the given
        // optionalAssociatedVolumeTemplateName has already been persisted in BD
        // (and loop if needed).
        while (vMImage_temp == null) {
            // XXX Ideally, we should/must use a listener on the DB instead of
            // the following lines.
            long currentTimeInMilliSeconds = System.currentTimeMillis();
            long timePlus1000MilliSeconds = currentTimeInMilliSeconds + 1000;
            while (currentTimeInMilliSeconds < timePlus1000MilliSeconds) {
                currentTimeInMilliSeconds = System.currentTimeMillis();
            }

            Query vMImage_temp_list_query = this.em.createQuery(
                "FROM " + VMImage.class.getSimpleName() + " v WHERE v.uploadedFromURL=:uploadedFromURL").setParameter(
                "uploadedFromURL", vMImageUrl);
            if (vMImage_temp_list_query == null) {
                // nothing to do.
            } else {
                if (vMImage_temp_list_query.getResultList().isEmpty()) {
                    // nothing to do.
                } else {
                    vMImage_temp = (VMImage) vMImage_temp_list_query.getResultList().iterator().next();
                }
            }
        }

        vMImage_temp.setProviderId(vMImageProviderId);
        vMImage_temp.setStatus(Status.AVAILABLE);

        this.em.persist(vMImage_temp);
        this.em.flush();
    }

    private void onVMImageStatusChange(final Integer vmiId, final String providerId, final String imageProviderId,
        final String vmiStatus, final String vmiError) {
        VirtualMachine vs = null;
        try {
            VMImage image = this.em.find(VMImage.class, vmiId);
            if (image == null) {
                this.log.warning("Received VMImage status change event for unknown image " + vmiId);
                return;
            }
            if (vmiStatus.equals("AVAILABLE")) {
                image.setStatus(VMImage.Status.AVAILABLE);
                image.setProviderId(imageProviderId);
            } else if (vmiStatus.equals("ERROR")) {
                image.setStatus(VMImage.Status.ERROR);
            }
            this.em.persist(image);
            this.em.flush();
            vs = this.cloudAdmin.getVirtualMachineByProviderId(providerId);
            this.cloudAdmin.emitTopicMessage(new VMImageUpdateEvent(VMImageVO.from(image)), "projectId", vs.getProject()
                .getProjectId());
        } finally {
            VirtualMachineMXBean vm = this.clusterManager.getVirtualMachineMBean(providerId);
            try {
                this.updateVirtualMachineStatusByProviderId(providerId, vm.getState(), true);
            } catch (VMMException e) {
            }
        }
    }

    private void onVolumeReady(final String volumeName, final String volumeProviderId, final Object handback) {
        Integer volumeId = (Integer) handback;
        this.log.info("The volume named " + volumeName + " is ready (volumeId=" + volumeId + ", volumeProviderId="
            + volumeProviderId + ").");
    }

    private void onVirtualMachineFailure(final String vmProviderId, final String vmName, final String failureType,
        final String failureDetail) {
        String vmId = null;
        int i = vmName.lastIndexOf("-");
        if (i != -1) {
            vmId = vmName.substring(i + 1);
        }
        if (vmId == null) {
            this.log.log(Level.SEVERE, "Internal error: unable to extract id from vmName" + vmName);
            return;
        }
        VirtualMachine vs = this.cloudAdmin.getVirtualMachineByVmId(vmId);
        if (vs == null) {
            this.log.log(Level.SEVERE, "Internal error: received failure event for unknown VM " + vmId);
            return;
        }
        this.cloudAdmin.updateVirtualMachineStatusById(vs.getId(), VirtualMachine.Status.ERROR);
        this.cloudAdmin.emitTopicMessage(new VirtualMachineStatusChangeEvent(VirtualMachineVO.from(vs)), "projectId", vs
            .getProject().getProjectId());
        this.eventManager.newEvent(Event.Level.ERROR, "Virtual Machine " + vs.getName() + " creation failure", vs.getProject()
            .getProjectId(), vs.getId().toString(), Event.ITypes.VM_ERROR, failureDetail);
    }

    private void onVirtualMachineGuestOSReady(final String vmProviderId, final String vmName,
        final org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricInfo[] perfMetricInfos) {
        String vmId = null;
        int i = vmName.lastIndexOf("-");
        if (i != -1) {
            vmId = vmName.substring(i + 1);
        }
        if (vmId == null) {
            this.log.log(Level.SEVERE, "Internal error: unable to extract id from vmName " + vmName);
            return;
        }

        // Check if the VirtualMachine corresponding to the given vmId has
        // already been persisted in BD (and loop if needed).
        int time_out = 0;
        boolean found = false;
        while ((!found) && (time_out < 10)) {
            found = this.cloudAdmin.doesThisVmExist(vmId);
            if (found) {
                // nothing to do.
            } else {
                this.log.warning("Thread.sleep(1000)");
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
            time_out = time_out + 1;
        }

        if (!found) {
            this.log.log(Level.SEVERE, "Internal error: received VirtualMachineGuestOSReady event for unknown VM " + vmId);
            return;
        } else {
            VirtualMachineVO vmVo = this.cloudAdmin.onVirtualMachineGuestOSReadySecondPart(vmId, vmName, vmProviderId);

            this.cloudAdmin.setPerfMetricInfosToVM(vmId, perfMetricInfos);
            // post-creation
            this.postCreateVM(vmVo);

            // set status to RUNNING and fire event
            this.cloudAdmin.emitTopicMessage(new VirtualMachineStatusChangeEvent(vmVo), "projectId", vmVo.getProjectId());
            this.eventManager.newEvent(Event.Level.INFO, "Virtual Machine " + vmVo.getName() + " created", vmVo.getProjectId(),
                vmVo.getId(), Event.ITypes.VM_CREATION, null);
        }

    }

    private void onVirtualMachineCreated(final String vmProviderId, final String vmName) {
        String vmId = null;
        int i = vmName.lastIndexOf("-");
        if (i != -1) {
            vmId = vmName.substring(i + 1);
        }
        if (vmId == null) {
            this.log.log(Level.SEVERE, "Internal error: unable to extract vmId from vmName " + vmName);
            return;
        }

        // Check if the VirtualMachine corresponding to the given vmId has
        // already been persisted in BD (and loop if needed).
        int time_out = 0;
        boolean found = false;
        while ((!found) && (time_out < 10)) {
            found = this.cloudAdmin.doesThisVmExist(vmId);
            if (found) {
                // nothing to do.
            } else {
                this.log.warning("Thread.sleep(1000)");
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
            time_out = time_out + 1;
        }

        if (!found) {
            this.log.log(Level.SEVERE, "Internal error: received VirtualMachineCreated event for unknown VM " + vmId);
            return;
        } else {

            VirtualMachineVO vmVo = this.cloudAdmin.onVirtualMachineCreatedSecondPart(vmId, vmName, vmProviderId);

            // post-creation
            this.postCreateVM(vmVo);

            // set status to CREATED and fire event
            this.cloudAdmin.emitTopicMessage(new VirtualMachineStatusChangeEvent(vmVo), "projectId", vmVo.getProjectId());
            this.eventManager.newEvent(Event.Level.INFO, "Virtual Machine " + vmVo.getName() + " created", vmVo.getProjectId(),
                vmVo.getId(), Event.ITypes.VM_CREATION, null);
        }
    }

    private void updateVirtualMachineStatusByProviderId(final String providerId,
        final VirtualMachineMXBean.PowerState powerState, final boolean ignoreCurrentStatus) {
        VirtualMachine vs = this.cloudAdmin.getVirtualMachineByProviderId(providerId);
        if (vs == null) {
            this.log.warning("Receiving updated power state for unknown VM providerId=" + providerId);
        } else {
            if (!ignoreCurrentStatus
                && (vs.getStatus() == VirtualMachine.Status.DESTROYED
                    || vs.getStatus() == VirtualMachine.Status.BEING_TEMPLATED || vs.getStatus() == VirtualMachine.Status.IN_CONFIGURATION)) {
                return;
            }
            switch (powerState) {
            case HALTED:
                vs.setStatus(VirtualMachine.Status.STOPPED);
                break;
            case PAUSED:
                vs.setStatus(VirtualMachine.Status.PAUSED);
                break;
            case RUNNING:
                vs.setStatus(VirtualMachine.Status.RUNNING);
                break;
            case SUSPENDED:
                vs.setStatus(VirtualMachine.Status.SUSPENDED);
                break;
            case UNKNOWN:
                // XXX
                break;
            }
            this.em.persist(vs);
            this.em.flush();
            this.log.info("VM " + vs.getName() + " new status=" + vs.getStatus());
            this.cloudAdmin.emitTopicMessage(new VirtualMachineStatusChangeEvent(VirtualMachineVO.from(vs)), "projectId", vs
                .getProject().getProjectId());
        }
    }

    private void updateDataCenterResourceUsage(final Domain domain) {
        this.cloudAdmin.emitTopicMessage(new DataCenterResourceUsageEvent(domain), "admin", "true");
    }

    private void moveVirtualMachineToHost(final String providerId, final String fromHostId, final String toHostId) {
        @SuppressWarnings("unchecked")
        List<VirtualMachine> list = this.em.createQuery("FROM VirtualMachine v WHERE v.providerId=:providerId")
            .setParameter("providerId", providerId).getResultList();
        VirtualMachine vm = list.get(0);
        vm.setMachine(toHostId);
        this.em.persist(vm);
        this.em.flush();
        this.cloudAdmin.emitTopicMessage(new VMMigrationEvent(vm.getId().toString(), fromHostId, toHostId), "admin", "true");
    }

    private void postCreateVM(final VirtualMachineVO vmVo) {
        // Completing SSH information
        String pubKey = vmVo.getUserPublicKey();
        if (pubKey != null && !"".equals(pubKey) && !"windows".equals(vmVo.getImageOsType())) {
            this.log.info("Execute SSH command");
            @SuppressWarnings("unused")
            Result sshCommand = new Result();
            try {
                sshCommand = SshCommand.command(vmVo.getIp(), "root", DbParameters.getInstance().VM_ROOT_PWD,
                    "mkdir .ssh && echo '" + pubKey + "' >>  .ssh/authorized_keys2");
            } catch (JSchException e) {
                this.log.info("SSH command failed !");
            }
        }

        if (DbParameters.getInstance().SEND_EMAIL_UPON_VM_CREATION) {
            final String ADMIN_MAIL = DbParameters.getInstance().ADMIN_MAIL;
            String to = vmVo.getUserEMail();

            // used to set $ variables in DbParameters mail
            final Map<String, String> varMap = new HashMap<String, String>();
            if (null != vmVo.getIp()) {
                varMap.put("ip", vmVo.getIp());
            } else {
                varMap.put("ip", "null");
            }

            if (to != null && to.length() > 0 && ADMIN_MAIL != null && ADMIN_MAIL.length() > 0) {
                try {
                    VirtUtils.sendMail(ADMIN_MAIL, to, DbParameters.getInstance().SERVICE_NAME,
                        VirtUtils.replace(DbParameters.getInstance().VM_CREATED, varMap));
                } catch (Exception e) {
                    this.log.log(Level.WARNING, "ERROR while sending mail", e);
                }
            }
        }
    }
}