/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: ICloudAdmin.java 313 2011-09-08 14:11:53Z chou $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import org.ow2.sirocco.cloudmanager.admin.exception.CloudAdminException;
import org.ow2.sirocco.cloudmanager.admin.exception.DuplicateNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InsufficientResourceException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidHostIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidPowerStateException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidUsernameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVMImageException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVMSizeException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVirtualMachineIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.NoRelevantResourcesPoolException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.exception.ResourceQuotaExceededException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMImageInUseException;
import org.ow2.sirocco.cloudmanager.admin.vo.VMImageVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VMSizeVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineExtendedInfo;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineSpec;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VolumeVO;
import org.ow2.sirocco.cloudmanager.business.Project;
import org.ow2.sirocco.cloudmanager.business.User;
import org.ow2.sirocco.cloudmanager.business.VMImage;
import org.ow2.sirocco.cloudmanager.business.VMSize;
import org.ow2.sirocco.cloudmanager.business.VirtualMachine;
import org.ow2.sirocco.cloudmanager.service.reservation.api.ReservationException;

/**
 * The ICloudAdmin interface provides the public administration interface of the
 * the Sirocco Cloud Manager service. This interface is intended to be used by a
 * presentation tier (typically a rich client interface). It provides a POJO
 * facade encapsulating the business logic of the service.
 */
public interface ICloudAdmin {

    /**
     * Emit JMS Message
     */
    void emitTopicMessage(final Serializable obj, final String property, final String value);

    /**
     * Returns the VirtualMachine value object with the given identifier
     * 
     * @param vmId a VM identifier
     * @return a VirtualMachine value object
     * @throws InvalidVirtualMachineIdException if the provided id is invalid
     */
    VirtualMachineVO getVirtualMachineVoByVmId(String vmId) throws InvalidVirtualMachineIdException;

    /**
     * Returns the remote console of the VM (if any)
     * 
     * @param vmId the id of the VM
     * @return an HTTP URL provided Web access to the VM console or null if no
     *         console is available
     * @throws InvalidVirtualMachineIdException if the provided VM id is invalid
     * @throws CloudAdminException if a runtime exception occurs while
     *         attempting to retrieve the console
     */
    String getVirtualMachineConsole(String vmId) throws InvalidVirtualMachineIdException, CloudAdminException;

    /**
     * Returns all virtual machines currently managed by the cloud service
     * 
     * @return a list of VirtualMachine value objects
     */
    List<VirtualMachineVO> getAllVirtualMachinesVo();

    /**
     * Starts the virtual machine identified with the provided id
     * 
     * @param vmId the VM id of the VM to start
     * @throws InvalidVirtualMachineIdException if the provided id is invalid
     * @throws InvalidPowerStateException if the VM is running
     */
    void startVirtualMachine(String vmId) throws InvalidVirtualMachineIdException, InvalidPowerStateException;

    /**
     * Stops the virtual machine identified with the provided id
     * 
     * @param vmId the VM id of the VM to stop
     * @param fromUsername TODO
     * @throws InvalidVirtualMachineIdException if the provided id is invalid
     * @throws InvalidPowerStateException if the VM is not running
     * @throws PermissionDeniedException
     */
    void stopVirtualMachine(String vmId, String fromUsername) throws InvalidVirtualMachineIdException,
        InvalidPowerStateException, PermissionDeniedException;

    /**
     * Pauses the virtual machine identified with the provided id
     * 
     * @param vmId the VM id of the VM to pause
     * @param fromUsername TODO
     * @throws InvalidVirtualMachineIdException if the provided id is invalid
     * @throws InvalidPowerStateException if the VM is not running
     * @throws PermissionDeniedException
     */
    void pauseVirtualMachine(String vmId, String fromUsername) throws InvalidVirtualMachineIdException,
        InvalidPowerStateException, PermissionDeniedException;

    /**
     * Unpauses the virtual machine identified with the provided id
     * 
     * @param vmId the VM id of the VM to unpause
     * @throws InvalidVirtualMachineIdException if the provided id is invalid
     * @throws InvalidPowerStateException if the VM is not paused
     */
    void unpauseVirtualMachine(String vmId) throws InvalidVirtualMachineIdException, InvalidPowerStateException;

    /**
     * Reboots the virtual machine identified with the provided id
     * 
     * @param vmId the VM id of the VM to reboot
     * @param fromUsername TODO
     * @throws InvalidVirtualMachineIdException if the provided id is invalid
     * @throws InvalidPowerStateException if the VM is not running
     * @throws PermissionDeniedException
     */
    void rebootVirtualMachine(String vmId, String fromUsername) throws InvalidVirtualMachineIdException,
        InvalidPowerStateException, PermissionDeniedException;

    /**
     * Live migrates a virtual machine to a new destination host
     * 
     * @param vmId the VM id of the VM to migrate
     * @param destinationHostId the id of the destination host
     * @throws InvalidVirtualMachineIdException if the provided VM id is invalid
     * @throws InvalidHostIdException if the provided host id is invalid. The
     *         destination host must exist and belong to the same server pool as
     *         the originating host of the VM
     * @throws CloudAdminException if the migration fails
     */
    void migrateVirtualMachine(String vmId, String destinationHostId) throws InvalidVirtualMachineIdException,
        InvalidHostIdException, CloudAdminException;

    /**
     * Destroys the virtual machine identified with the provided id
     * 
     * @param vmId the VM id of the VM to destroy
     * @param fromUsername
     * @throws InvalidVirtualMachineIdException if the provided id is invalid
     * @throws InvalidPowerStateException if the operation is not allowed
     * @throws PermissionDeniedException
     * @throws InvalidArgumentException
     * @throws ReservationException
     * @throws NoRelevantResourcesPoolException
     */
    void destroyVirtualMachine(final String vmId, final String fromUsername) throws InvalidVirtualMachineIdException,
        InvalidPowerStateException, PermissionDeniedException, InvalidArgumentException, ReservationException,
        NoRelevantResourcesPoolException;

    /**
     * Creates, and starts a new virtual machine (if spec.getReservationId is
     * null, it will be an on-demand allocation, otherwise, VM creation will be
     * on-reservation). Start action requires that the parameter startVM ==
     * true.
     * 
     * @param projectId the projectName of the project for the new VM
     * @param user the username of the user the new VM should belong to
     * @param spec the properties of the virtual machine to create
     * @param startVM set to true for starting the VM
     * @return a VirtualMachine value object. Note that the actual creation of
     *         the VM is asynchronous and the status of the VirtualMachine
     *         returned is IN_CONFIGURATION.
     * @throws InvalidUsernameException if the provided username is invalid
     * @throws InvalidProjectNameException if the provided project is invalid
     * @throws InvalidNameException if the name of the virtual machine is
     *         invalid
     * @throws InvalidVMImageException if the VM image ID provided is invalid
     * @throws ResourceQuotaExceededException if the resource quota of the user
     *         is exceeded
     * @throws DuplicateNameException if the name of the virtual machine is
     *         already used
     * @throws InvalidVMSizeException if the VM size provided is invalid
     * @throws InvalidProjectIdException
     * @throws PermissionDeniedException
     * @throws ReservationException
     * @throws InvalidArgumentException
     * @throws InsufficientResourceException
     * @throws NoRelevantResourcesPoolException
     * @throws InvalidVirtualMachineIdException
     */
    String createAndStartVirtualMachine(String projectId, String user, VirtualMachineSpec spec, boolean startVM)
        throws InvalidUsernameException, InvalidNameException, InvalidVMImageException, ResourceQuotaExceededException,
        DuplicateNameException, InvalidVMSizeException, InvalidProjectIdException, PermissionDeniedException,
        ReservationException, InvalidProjectNameException, InvalidArgumentException, InsufficientResourceException,
        NoRelevantResourcesPoolException, InvalidVirtualMachineIdException;

    /**
     * Returns the VM size symbolic name of a virtual machine
     * 
     * @param vmId the identifier of the VM
     * @return the name of the VM size or the value "custom" if the resource
     *         capacities (CPU, disk, ram) of the VM do not match a predefined
     *         VM size.
     * @throws InvalidVirtualMachineIdException if the provided id is invalid
     */
    String getVMSizeName(String vmId) throws InvalidVirtualMachineIdException;

    /**
     * Returns the available VM sizes.
     * 
     * @return a list of VM size value objects
     */
    List<VMSizeVO> getVMSizesVo();

    /**
     * Creates a new VM image out of an existing VM by taking a snapshot of its
     * disk
     * 
     * @param vmId the identifier of the VM to snapshot
     * @param name the user-friendly name of the new VM image
     * @param description a description of the VM image
     * @throws InvalidVirtualMachineIdException if the provided VM id is invalid
     * @throws InvalidNameException if the name of the VM image is invalid
     * @throws DuplicateNameException if the name of the VM image is already
     *         used
     * @throws CloudAdminException if any type of runtime exception not covered
     *         by the previous exception
     */
    void createVMImage(final String projectId, final VMImage.Visibility visibility, final String vmId, final String name,
        final String description, final String fromUsername) throws CloudAdminException, InvalidVirtualMachineIdException,
        InvalidNameException, DuplicateNameException, InvalidProjectIdException;

    /**
     * Returns all VM images registered by the Cloud service
     * 
     * @return a list of VM image value objects
     */
    List<VMImageVO> getAllVMImagesVo();

    /**
     * Returns all volumes registered by the Cloud service
     * 
     * @return a list of Volume value objects
     */
    List<VolumeVO> getAllVolumeVo();

    /**
     * Destroys a VM image
     * 
     * @param imageId the identifier of the VM image
     * @param fromUsername
     * @throws InvalidVMImageException if the provided VM image id is invalid
     * @throws PermissionDeniedException
     */
    void destroyVMImage(Integer imageId, String fromUsername) throws InvalidVMImageException, VMImageInUseException,
        PermissionDeniedException;

    /**
     * Updates a VM image properties (name, description, hypervisor)
     * 
     * @param image a VM image value object with updated properties. Note that
     *        only the name, description and hypervisor field of the value
     *        objects are taken into account
     * @throws InvalidVMImageException if the VM image id is invalid
     */
    void updateVMImage(VMImageVO image) throws InvalidVMImageException;

    /**
     * Returns the virtual machines located on a given host
     * 
     * @param hostId the identifier of the host
     * @return a list of virtual machines located on the host
     * @throws InvalidHostIdException if the provided host identifier is invalid
     */
    List<VirtualMachineVO> getVirtualMachinesVoByHostId(final String hostId) throws InvalidHostIdException;

    /**
     * Returns extended information on a virtual machine
     * 
     * @param vmId the identifier of the virtual machine
     * @return a VirtualMachineExtendedInfo object
     * @throws InvalidVirtualMachineIdException if the provided VM identifier is
     *         invalid
     */
    VirtualMachineExtendedInfo getVirtualMachineExtendedInfo(final String vmId) throws InvalidVirtualMachineIdException;

    /**
     * Returns the CPU loads of several virtal machines
     * 
     * @param vmIds an array of VM identifiers
     * @return CPU loads of the virtual machines. Each load is a floating value
     *         between 0 and 1 representing a percentage
     * @throws InvalidVirtualMachineIdException if one of the provided VM
     *         identifier is invalid
     */
    float[] getVirtualMachinesCPULoad(final String vmIds[]) throws InvalidVirtualMachineIdException;

    /**
     * Returns the used memory of several virtual machines
     * 
     * @param vmIds an array of VM identifiers
     * @return an array of integers representing the memory used in megabytes by
     *         each virtual machine
     * @throws InvalidVirtualMachineIdException if one of the provided VM
     *         identifier is invalid
     */
    long[] getVirtualMachinesMemoryUsedMB(final String vmIds[]) throws InvalidVirtualMachineIdException;

    int getVirtualMachineVNCConsoleLocalPort(final String vmId) throws InvalidVirtualMachineIdException;

    VMImage getVMImageByImageId(final Integer vmiId);

    VirtualMachine getVirtualMachineByProviderId(final String providerId);

    VirtualMachine getVirtualMachineByVmId(final String vmId);

    VirtualMachine getVirtualMachineById(final Integer vmId);

    void updateVirtualMachineStatusById(final int vmId, final VirtualMachine.Status status);

    void updateVirtualMachineExpirationDate(String vmId, Date expirationDate, String fromUsername)
        throws InvalidVirtualMachineIdException, PermissionDeniedException;

    /**
     * Creates a VM with given volumes. Note that the VM is not started.
     * 
     * @param projectId
     * @param userName
     * @param vmSpec
     * @param volumesId
     * @return the id of the just created VirtualMachine (i.e., the id in DB).
     * @throws InvalidUsernameException
     * @throws InvalidNameException
     * @throws InvalidVMImageException
     * @throws ResourceQuotaExceededException
     * @throws DuplicateNameException
     * @throws InvalidVMSizeException
     * @throws InvalidProjectIdException
     * @throws PermissionDeniedException
     * @throws ReservationException
     * @throws CloudAdminException
     */
    String createVirtualMachineWithVolumes(final String projectId, final String userName, final VirtualMachineSpec vmSpec,
        final List<Integer> volumesId) throws InvalidUsernameException, InvalidNameException, InvalidVMImageException,
        ResourceQuotaExceededException, DuplicateNameException, InvalidVMSizeException, InvalidProjectIdException,
        PermissionDeniedException, ReservationException, CloudAdminException;

    int createVMImageRecord(VMImage image);

    /**
     * @param project
     * @param user
     * @param spec
     * @return Vm's Id.
     * @throws InvalidVMImageException
     */
    int newVirtualMachine(final Project project, final User user, final VirtualMachineSpec spec) throws InvalidVMImageException;

    VirtualMachineVO destroyVirtualMachineRecord(final String vmId, final String fromUsername)
        throws InvalidVirtualMachineIdException, PermissionDeniedException, InvalidPowerStateException,
        InvalidArgumentException, NoRelevantResourcesPoolException, ReservationException;

    boolean doesThisVmExist(final String vmId);

    VirtualMachineVO onVirtualMachineCreatedSecondPart(final String vmId, final String vmName, final String vmProviderId);

    VirtualMachineVO onVirtualMachineGuestOSReadySecondPart(final String vmId, final String vmName, final String vmProviderId);

    boolean checkIfVmNameIsCorrect(final String vmLabel);

    boolean checkIfVMNameExistForProjectId(final String vmName, final String projectId);

    VMSize lookUpVMSizeByName(final String name);

    boolean validateVirtualMachineCreation(final Project project, final int numCPUs, final int memorySizeMB,
        final int diskSizeMB);

    void setPerfMetricInfosToVM(String vmId, org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricInfo[] perfMetricInfos);

}
