/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: IClusterManagerProxy.java 36 2011-06-20 14:38:41Z dangtran $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.util.Date;
import java.util.List;

import org.ow2.sirocco.cloudmanager.admin.exception.InvalidHostIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidPerfMetricIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVirtualMachineIdException;
import org.ow2.sirocco.cloudmanager.clustermanager.api.Domain;
import org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricBase;
import org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricInfo;

public interface IClusterManagerProxy {

    /**
     * Returns the available datacenter locations
     * 
     * @return a list of datacenter locations
     */
    public List<String> getDataCenterLocations();

    /**
     * Returns the object tree of the datacenter as a hierarchical structure of
     * domains, server pools and hosts
     * 
     * @return the top level domain of the datacenter
     */
    public Domain getDataCenterResourceTree();

    /**
     * Returns information about the supported performance metrics which can be
     * retrieved from a host
     * 
     * @return an array of PerffMetricInfo objects describing the supported
     *         metrics
     */
    public PerfMetricInfo[] getHostPerfMetricInfos();

    /**
     * Returns information about the supported performance metrics which can be
     * retrieved from a virtual machine
     * 
     * @return an array of PerffMetricInfo objects describing the supported
     *         metrics
     */
    public PerfMetricInfo[] getVirtualMachinePerfMetricInfos();

    /**
     * Returns a series of performance metric values for a host in a specific
     * time interval
     * 
     * @param hostId the identifier of the host
     * @param metricId the identifier of the metric. This metric Id must be
     *        supported by the host.
     * @param startTime the time from which metric values are to be retrieved.
     *        When startTime is null, the returned metrics start from the first
     *        available metric in the system.
     * @param endTime the time up to which metric values are retrieved. When
     *        endTime is null, the returned result includes up to the most
     *        recent metric value.
     * @return an array of PerfMetricBase objects. If the metric is composite,
     *         these objects are PerfSimpleMetric instances otherwise they are
     *         PerfCompositeMetric instances.
     * @throws InvalidHostIdException if the provided host identifier is invalid
     * @throws InvalidPerfMetricIdException if the provided metric identifier is
     *         invalid (either because it is unknown by the system or
     *         unsupported for a host)
     */
    public PerfMetricBase[] getHostPerfMetrics(final String hostId, final String metricId, final Date startTime,
        final Date endTime) throws InvalidHostIdException, InvalidPerfMetricIdException;

    /**
     * Returns a series of performance metric values for a VM in a specific time
     * interval
     * 
     * @param vmId the identifier of the VM
     * @param metricId the identifier of the metric. This metric Id must be
     *        supported by the VM.
     * @param startTime the time from which metric values are to be retrieved.
     *        When startTime is null, the returned metrics start from the first
     *        available metric in the system.
     * @param endTime the time up to which metric values are retrieved. When
     *        endTime is null, the returned result includes up to the most
     *        recent metric value.
     * @return an array of PerfMetricBase objects. If the metric is composite,
     *         these objects are PerfSimpleMetric instances otherwise they are
     *         PerfCompositeMetric instances.
     * @throws InvalidVirtualMachineIdException if the provided VM identifier is
     *         invalid
     * @throws InvalidPerfMetricIdException if the provided metric identifier is
     *         invalid (either because it is unknown by the system or
     *         unsupported for a virtual machine)
     */
    public PerfMetricBase[] getVirtualMachinePerfMetrics(final String vmId, final String metricId, final Date startTime,
        final Date endTime) throws InvalidVirtualMachineIdException, InvalidPerfMetricIdException;

}
