/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: ISystemInstanceManager.java 228 2011-08-04 10:26:12Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.util.List;
import java.util.Properties;

import org.ow2.sirocco.cloudmanager.admin.exception.CloudAdminException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidSystemInstanceStatusException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidUsernameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVolumeIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.exception.ResourceQuotaExceededException;
import org.ow2.sirocco.cloudmanager.admin.exception.VolumeInUseException;
import org.ow2.sirocco.cloudmanager.admin.vo.SystemInstanceVO;

/**
 * This interface provides public methods for managing System Instances (i.e.,
 * instances of SystemTemplates).
 */
public interface ISystemInstanceManager {

    /**
     * @return
     */
    List<SystemInstanceVO> listSystemInstances();

    /**
     * Creates a SystemInstance starting from a SystemTemplate (specified via
     * its systemTemplateId). This method will create a SystemInstance object,
     * and it will, at least, create the related VirtualMachines via the use of
     * the CloudProvider's API (XXX how can we choose the cloudProvider ? Is it
     * defined in the systemTemplate ? Is it given as an input parameter ?)
     * 
     * @param userName
     * @param projectId
     * @param systemTemplateName
     * @param properties
     * @return The system instance's Id.
     * @throws ResourceQuotaExceededException
     * @throws PermissionDeniedException
     * @throws CloudAdminException
     */
    Integer createSystemInstance(final String userName, final String projectId, final String systemTemplateName,
        final Properties properties) throws ResourceQuotaExceededException, PermissionDeniedException, CloudAdminException;

    /**
     * Starts, in the specfied order, all the VirtualMachines belonging to the
     * SystemInstance specified by the given systemInstanceId.
     * 
     * @param userName
     * @param systemInstanceId
     * @throws InvalidArgumentException
     * @throws InvalidSystemInstanceStatusException
     * @throws PermissionDeniedException
     * @throws CloudAdminException
     */
    void startSystemInstance(final String userName, final Integer systemInstanceId) throws InvalidArgumentException,
        InvalidSystemInstanceStatusException, PermissionDeniedException, CloudAdminException;

    /**
     * Stops, in the specified order, all the VirtualMachines belonging to the
     * SystemInstance specified by the given systemInstanceId.
     * 
     * @param userName
     * @param systemInstanceId
     * @throws InvalidArgumentException
     * @throws InvalidSystemInstanceStatusException
     * @throws PermissionDeniedException
     * @throws CloudAdminException
     */
    void stopSystemInstance(final String userName, final Integer systemInstanceId) throws InvalidArgumentException,
        InvalidSystemInstanceStatusException, PermissionDeniedException, CloudAdminException;

    /**
     * Destroys the SystemInstance specified by the given systemInstanceId. This
     * method destroys the related VirtualMachine(s), and the SystemInstance
     * object itself (i.e. move object(s) to DESTROYED state).
     * 
     * @param userName
     * @param systemInstanceId
     * @throws InvalidUsernameException
     * @throws InvalidArgumentException
     * @throws PermissionDeniedException
     * @throws InvalidSystemInstanceStatusException
     * @throws CloudAdminException
     */
    void destroySystemInstance(final String userName, final Integer systemInstanceId) throws InvalidUsernameException,
        InvalidArgumentException, PermissionDeniedException, InvalidSystemInstanceStatusException, CloudAdminException;

    /**
     * Remove SystemInstances in the Destroyed state from the DB (remove also
     * SystemInstances' destroyed NetworkInstances, VirtualMachines, Volumes).
     */
    void cleanUpDestroyedSystemInstances();

    public boolean destroyVolume(final String userName, final List<Integer> volumesIdToBeDeleted)
        throws InvalidUsernameException, InvalidProjectIdException, InvalidVolumeIdException, VolumeInUseException,
        PermissionDeniedException, CloudAdminException;

    public Integer moveStatusOfSiToDESTROYING(final Integer siId);

}
