/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: IUserManager.java 233 2011-08-08 10:08:06Z otterley $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.util.List;

import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidUsernameException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.exception.ProjectInUseException;
import org.ow2.sirocco.cloudmanager.admin.exception.UserAlreadyExistsException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMImageInUseException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMInUseException;
import org.ow2.sirocco.cloudmanager.admin.exception.VolumeInUseException;
import org.ow2.sirocco.cloudmanager.admin.vo.UserVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VolumeVO;
import org.ow2.sirocco.cloudmanager.business.User;

public interface IUserManager {

    public abstract User createUser(final String username, String password, final String userMail, final String userFirstName,
        final String userLastName) throws UserAlreadyExistsException;

    /**
     * Deletes the account associated with the provided username
     * 
     * @param username name of the account
     * @param fromUser TODO
     * @throws InvalidUsernameException if the provided username is invalid
     * @throws VMInUseException if the provided account has still VM in used
     * @throws VMImageInUseException
     * @throws InvalidProjectIdException
     * @throws ProjectInUseException
     * @throws PermissionDeniedException
     * @throws VolumeInUseException
     */
    public abstract void deleteUser(String username, String fromUser) throws InvalidUsernameException, VMInUseException,
        VMImageInUseException, InvalidProjectIdException, ProjectInUseException, PermissionDeniedException,
        VolumeInUseException;

    public abstract User getUserByUsername(final String username);

    public abstract List<User> getUsersByProjectId(final String projectId);

    public abstract String getApiPasswordForUser(final String username);

    /**
     * Authenticates a user on the basis of his/her login name (username),
     * password and administrator status.
     * 
     * @param username the username (login) of the user
     * @param password the password of the user
     * @return true if the given user can be authenticated
     * @throws InvalidUsernameException if the provided username is unknown
     */
    public abstract boolean authenticateUser(String username, String password) throws InvalidUsernameException;

    /**
     * Returns all user accounts registered by the cloud service
     * 
     * @return a list of user account value objects
     */
    public abstract List<UserVO> getAllUsersVo();

    /**
     * Returns a list of the virtual machines belonging to a specific user
     * 
     * @param username the username of the user
     * @return a list of VirtualMachine value objects
     * @throws InvalidUsernameException if the provided username is invalid
     */
    public abstract List<VirtualMachineVO> getVirtualMachinesVoByUsername(String username, boolean includeDestroyedVMs)
        throws InvalidUsernameException;

    /**
     * Returns a list of the volumes belonging to a specific user
     * 
     * @param username user's login
     * @return a list of Volume value objects
     * @throws InvalidUsernameException if the provided username is invalid
     */
    List<VolumeVO> getVolumeVoByUser(String username) throws InvalidUsernameException;

    /**
     * Returns the user account information for a specific user
     * 
     * @param username the username of the user
     * @return a user account value object
     * @throws InvalidUsernameException if the provided username is invalid
     */
    public abstract UserVO getUserVoByUsername(String username) throws InvalidUsernameException;

    /**
     * Updates the SSH public key of a user
     * 
     * @param username the username of the user
     * @param sshKey the new SSH key
     * @throws InvalidUsernameException if the provided username is invalid
     */
    public abstract void updateUserSSHKey(String username, String sshKey) throws InvalidUsernameException;

    /**
     * Updates username passwords (web portal, api)
     * 
     * @param username the username of the user
     * @param webPassword new Web portal password
     * @param apiPassword new API password
     */
    public abstract void updateUserPasswords(String username, String webPassword, String apiPassword)
        throws InvalidUsernameException;

    public abstract List<User> getAllAdmins();

    public abstract UserVO getUserVoLdapInfoByUsername(final String username);

}
