/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: OVFImporterBean.java 372 2011-09-30 14:49:32Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.math.BigInteger;
import java.net.InetSocketAddress;
import java.net.Proxy;
import java.net.SocketAddress;
import java.net.URL;
import java.net.URLConnection;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.logging.Logger;

import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.Query;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.ow2.sirocco.cloudmanager.admin.exception.DuplicateNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.OVFImporterException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMImageWithTheGivenURLAlreadyExistsException;
import org.ow2.sirocco.cloudmanager.business.NetworkInterfaceTemplate;
import org.ow2.sirocco.cloudmanager.business.NetworkTemplate;
import org.ow2.sirocco.cloudmanager.business.SystemTemplate;
import org.ow2.sirocco.cloudmanager.business.SystemTemplate.SystemTemplateStatus;
import org.ow2.sirocco.cloudmanager.business.User;
import org.ow2.sirocco.cloudmanager.business.VMImage;
import org.ow2.sirocco.cloudmanager.business.VirtualMachineTemplate;
import org.ow2.sirocco.cloudmanager.business.VolumeTemplate;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.AnnotationSectionType;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.CimString;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.ContentType;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.DiskSectionType;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.EnvelopeType;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.FileType;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.NetworkSectionType;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.NetworkSectionType.Network;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.OperatingSystemSectionType;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.ProductSectionType;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.RASDType;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.ReferencesType;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.SectionType;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.StartupSectionType;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.StartupSectionType.Item;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.VirtualDiskDescType;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.VirtualHardwareSectionType;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.VirtualSystemCollectionType;
import org.ow2.sirocco.cloudmanager.service.ovf.datamodel.VirtualSystemType;
import org.ow2.sirocco.cloudmanager.utils.PermissionChecker;
import org.ow2.sirocco.vmm.api.VMMException;

@Stateless(name = "OVFImporterBean", mappedName = "OVFImporterBean")
@Remote(IRemoteOVFImporter.class)
@Local(IOVFImporter.class)
public class OVFImporterBean implements IOVFImporter {

    private static Logger log = Logger.getLogger(OVFImporterBean.class.getName());

    @PersistenceContext(unitName = "persistence-unit/main", type = PersistenceContextType.TRANSACTION)
    protected EntityManager em;

    @EJB(mappedName = "VMImageManagerBean")
    private IVMImageManager vMImageManager;

    @Override
    public String oVFApplianceFileImportation(final String userName, final String oVFApplianceLocation)
        throws OVFImporterException, IOException, InvalidArgumentException, PermissionDeniedException {
        String result = null;

        // Check if a user corresponding to userName exists.
        User user = null;
        Query user_temp_list_query = this.em
            .createQuery("FROM " + User.class.getSimpleName() + " u WHERE u.username=:userName").setParameter("userName",
                userName);
        if (user_temp_list_query == null) {
            throw new InvalidArgumentException("The given userName: " + userName + " is unknown.");
        } else {
            if (user_temp_list_query.getResultList().isEmpty()) {
                throw new InvalidArgumentException("The given userName: " + userName + " is unknown.");
            } else {
                user = (User) user_temp_list_query.getResultList().iterator().next();
            }
        }

        // Check user's authorization.
        if (new PermissionChecker().canImportOVF(user)) {
            // nothing to do.
        } else {
            throw new PermissionDeniedException("Permission denied.");
        }

        // // Create a temp folder.
        // String tempFolderName = "temp";
        // File tempFolder = new File(tempFolderName);
        // if (tempFolder.exists()) {
        // // nothing to do.
        // } else {
        // boolean doesTempFolderCreationSucceed = tempFolder.mkdir();
        // if (doesTempFolderCreationSucceed) {
        // // nothing to do.
        // } else {
        // throw new OVFImporterException(
        // "ERROR: temp folder creation was not possible.");
        // }
        // }

        // oVFApplianceLocation =
        // "http://.../ovf/springooReformatted.ovf";

        // oVFApplianceLocation =
        // "file:/C:/jzbv7415/Check_out_orange_open_cloud/OrangeOpenCloud/OrangeOpenCloud-ejb-jar/src/test/resources/springooReformatted.ovf";

        // oVFApplianceLocation =
        // "C:/jzbv7415/Check_out_orange_open_cloud/OrangeOpenCloud/OrangeOpenCloud-ejb-jar/src/test/resources/springooReformatted.ovf";

        //

        // oVFApplianceLocation = "src/test/resources/springooReformatted.ovf";

        // oVFApplianceLocation =
        // "src/test/resources/DSP2021_1.0.0/examples/lamp-multi-vm.ovf";

        File localCopy = null;

        if (oVFApplianceLocation.startsWith("http://")) {
            // URL
            URL url = new URL(oVFApplianceLocation);

            OVFImporterBean.log.finest(url.toString());
            OVFImporterBean.log.finest(url.getFile());

            // TODO Should we manage proxy configuration (?).
            SocketAddress sA = new InetSocketAddress("p-goodway", 3128);
            Proxy proxy = new Proxy(Proxy.Type.HTTP, sA);
            URLConnection uRLConnection = url.openConnection(proxy);

            InputStream iS = uRLConnection.getInputStream();
            // BufferedReader br = new BufferedReader(new
            // InputStreamReader(iS));
            // String line;
            localCopy = File.createTempFile("temp_" + new Date().getTime() + "_", ".ovf");
            localCopy.createNewFile();
            // localCopy = File.createTempFile("temp_" + new Date().getTime()
            // + "_", ".ovf");
            // localCopy.deleteOnExit();
            OVFImporterBean.log.finest(localCopy.getAbsolutePath());
            // while ((line = br.readLine()) != null) {
            // System.out.println(line);
            // }

            this.writeInputStreamToOutputStream(iS, new FileOutputStream(localCopy));

            iS.close();
        } else if (oVFApplianceLocation.startsWith("file:/")) {
            String oVFApplianceLocation_temp = oVFApplianceLocation.replaceFirst("file:/", "");
            OVFImporterBean.log.finest(oVFApplianceLocation_temp);
            localCopy = new File(oVFApplianceLocation_temp);
        } else if (oVFApplianceLocation.startsWith("file:")) {
            String oVFApplianceLocation_temp = oVFApplianceLocation.replaceFirst("file:", "");
            OVFImporterBean.log.finest(oVFApplianceLocation_temp);
            localCopy = new File(oVFApplianceLocation_temp);
        } else {
            // Here, we assume that this is a local file.
            localCopy = new File(oVFApplianceLocation);
        }

        // Let's check if the given file (exists) and is a file.
        if (localCopy.isFile()) {
            result = localCopy.getAbsolutePath();
            return result;
        } else {
            throw new OVFImporterException("ERROR: The given file is NOT a file.");
        }
    }

    private void writeInputStreamToOutputStream(final InputStream in, final OutputStream out) {
        BufferedReader inReader = new BufferedReader(new InputStreamReader(in));
        OutputStreamWriter outWriter = new OutputStreamWriter(out);

        String line = null;
        try {
            while ((line = inReader.readLine()) != null) {
                outWriter.write(line + "\n");
            }
            outWriter.flush();
            in.close();
            outWriter.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void oVFApplianceParsingAndSystemTemplateCreation(final String userName,
        final String oVFApplianceLocationOnTheLocalFileSystem) throws OVFImporterException, FileNotFoundException,
        JAXBException, VMMException, VMImageWithTheGivenURLAlreadyExistsException, InvalidArgumentException,
        DuplicateNameException, PermissionDeniedException {

        // Check if a user corresponding to userName exists.
        User user = null;
        Query user_temp_list_query = this.em
            .createQuery("FROM " + User.class.getSimpleName() + " u WHERE u.username=:userName").setParameter("userName",
                userName);
        if (user_temp_list_query == null) {
            throw new InvalidArgumentException("The given userName: " + userName + " is unknown.");
        } else {
            if (user_temp_list_query.getResultList().isEmpty()) {
                throw new InvalidArgumentException("The given userName: " + userName + " is unknown.");
            } else {
                user = (User) user_temp_list_query.getResultList().iterator().next();
            }
        }

        // Check user's authorization.
        if (new PermissionChecker().canImportOVF(user)) {
            // nothing to do.
        } else {
            throw new PermissionDeniedException("Permission denied.");
        }

        File oVFApplianceFile = new File(oVFApplianceLocationOnTheLocalFileSystem);

        // Let's check if the "file" (exists) and is a file.
        if (oVFApplianceFile.isFile()) {
            JAXBContext jc = JAXBContext.newInstance(EnvelopeType.class);
            Unmarshaller unm = jc.createUnmarshaller();

            JAXBElement<?> jAXBElement = ((JAXBElement<?>) unm.unmarshal(new FileInputStream(oVFApplianceFile)));

            OVFImporterBean.log.finest("DeclaredType of the object returned by the unmarshaller: "
                + jAXBElement.getDeclaredType());

            EnvelopeType eT = (EnvelopeType) jAXBElement.getValue();

            OVFImporterBean.log.finest(eT.toString());

            SystemTemplate sT = this.fromEnvelopeTypeToSystemTemplate(eT);

            this.em.persist(sT);
            this.em.flush();
        } else {
            throw new OVFImporterException("ERROR: The given path does NOT refer to a file: "
                + oVFApplianceLocationOnTheLocalFileSystem + ".");
        }
    }

    private SystemTemplate fromEnvelopeTypeToSystemTemplate(final EnvelopeType envelopeType) throws OVFImporterException,
        VMMException, VMImageWithTheGivenURLAlreadyExistsException, InvalidArgumentException, DuplicateNameException {
        if (envelopeType == null) {
            throw new OVFImporterException("The given envelopeType is " + envelopeType + ".");
        } else {
            // Warning, XML tags can occur twice or more, il y a (au moins) deux
            // fois le tag XML <ProductSection ...

            ReferencesType referencesType = envelopeType.getReferences();

            LinkedHashMap<String, String> fileType_oVFId_and_oVFHRef_lHM = new LinkedHashMap<String, String>();
            LinkedHashMap<String, Integer> fileType_oVFId_and_oVFSizeInMB_lHM = new LinkedHashMap<String, Integer>();
            LinkedHashMap<String, String> virtualDiskDescType_oVFDiskId_and_oVFFileRef_lHM = new LinkedHashMap<String, String>();
            if (referencesType == null) {
                throw new OVFImporterException("The given referencesType is " + referencesType + ".");
            } else {
                // For the moment, we create a LinkedHashMap in order to store
                // the reference files temporarily, and to set fileOVFHRef on
                // the VolumeTemplate.

                Iterator<FileType> fileType_temp_ITERATOR = referencesType.getFile().iterator();
                while (fileType_temp_ITERATOR.hasNext()) {
                    FileType fileType_temp = fileType_temp_ITERATOR.next();
                    fileType_oVFId_and_oVFHRef_lHM.put(fileType_temp.getId(), fileType_temp.getHref());

                    // OVF specification defines that storage/disk
                    // capacity must be expressed in byte. The optional
                    // ovf:capacityAllocationUnits XML tag can also be
                    // used to define the unit (based on byte). DMTF
                    // allows: Bytes, KiloBytes, MegaBytes, GigaBytes,
                    // Words, DoubleWords, QuadWords.
                    // In practice, "byte", "byte * 2^20" are used.

                    // Here, we assume that fileType_temp.getSize() has been
                    // expressed using the default byte unit (see OVF
                    // specification).
                    Integer oVFSizeInMB = (fileType_temp.getSize().divide(new BigInteger(new Long((long) Math.pow(2, 20))
                        .toString()))).intValue();
                    fileType_oVFId_and_oVFSizeInMB_lHM.put(fileType_temp.getId(), oVFSizeInMB);
                }

            }

            List<JAXBElement<? extends SectionType>> sectionTypesList = envelopeType.getSection();

            if (sectionTypesList == null || sectionTypesList.isEmpty()) {
                throw new OVFImporterException("The given sectionTypesList is either null or empty: " + sectionTypesList + ".");
            } else {
                // Let's see the VirtualSystemCollection.
                VirtualSystemCollectionType virtualSystemCollectionType = (VirtualSystemCollectionType) envelopeType
                    .getContent().getValue();

                SystemTemplate systemTemplate = null;

                // Check if the SystemTemplate's name isn't taken by an already
                // persisted SystemTemplate.
                Query systemTemplate_temp_list_query = this.em.createQuery(
                    "FROM " + SystemTemplate.class.getSimpleName() + " s WHERE s.name=:name").setParameter("name",
                    virtualSystemCollectionType.getId());
                if (systemTemplate_temp_list_query == null) {
                    // nothing to do.
                } else {
                    if (systemTemplate_temp_list_query.getResultList().isEmpty()) {
                        // nothing to do.
                    } else {
                        systemTemplate = (SystemTemplate) systemTemplate_temp_list_query.getResultList().iterator().next();
                        throw new DuplicateNameException(
                            "The specified SystemTemplate's name (i.e., the VirtualSystemCollection's ovf:id) "
                                + virtualSystemCollectionType.getId()
                                + " is already taken by another persisted SystemTemplate.");
                    }
                }

                // SystemTemplate creation.
                systemTemplate = new SystemTemplate();
                systemTemplate.setName(virtualSystemCollectionType.getId());
                systemTemplate.setStatus(SystemTemplateStatus.IN_CONFIGURATION);

                this.em.persist(systemTemplate);
                this.em.flush();

                // Let's see the sections, e.g., DiskSection, NetworkSection
                Iterator<JAXBElement<? extends SectionType>> sectionType_ITERATOR = sectionTypesList.iterator();
                while (sectionType_ITERATOR.hasNext()) {
                    JAXBElement<? extends SectionType> sectionType_temp = sectionType_ITERATOR.next();

                    if (DiskSectionType.class.getCanonicalName().equals(sectionType_temp.getDeclaredType().getCanonicalName())) {

                        DiskSectionType diskSectionType = (DiskSectionType) sectionType_temp.getValue();

                        List<VirtualDiskDescType> virtualDiskDescType_list = diskSectionType.getDisk();
                        Iterator<VirtualDiskDescType> virtualDiskDescType_temp_ITERATOR = virtualDiskDescType_list.iterator();
                        while (virtualDiskDescType_temp_ITERATOR.hasNext()) {
                            VirtualDiskDescType virtualDiskDescType_temp = virtualDiskDescType_temp_ITERATOR.next();

                            OVFImporterBean.log.finest(virtualDiskDescType_temp.getCapacity());
                            OVFImporterBean.log.finest(virtualDiskDescType_temp.getCapacityAllocationUnits());
                            OVFImporterBean.log.finest(virtualDiskDescType_temp.getFormat());
                            OVFImporterBean.log.finest("" + virtualDiskDescType_temp.getPopulatedSize());

                            // VolumeTemplate creation.
                            VolumeTemplate volumeTemplate = new VolumeTemplate();
                            volumeTemplate.setName(virtualDiskDescType_temp.getDiskId());
                            volumeTemplate.setVolumeFormat(virtualDiskDescType_temp.getFormat());

                            virtualDiskDescType_oVFDiskId_and_oVFFileRef_lHM.put(virtualDiskDescType_temp.getDiskId(),
                                virtualDiskDescType_temp.getFileRef());

                            // OVF specification defines that storage/disk
                            // capacity must be expressed in byte. The optional
                            // ovf:capacityAllocationUnits XML tag can also be
                            // used to define the unit (based on byte). DMTF
                            // allows: Bytes, KiloBytes, MegaBytes, GigaBytes,
                            // Words, DoubleWords, QuadWords.
                            // In practice, "byte", "byte * 2^20" are used.
                            String capacityAllocationUnits = virtualDiskDescType_temp.getCapacityAllocationUnits();
                            Long virtualDiskCapacity = new Long(virtualDiskDescType_temp.getCapacity());
                            Long virtualDiskCapacityInMB = null;
                            if (capacityAllocationUnits == null) {
                                // We assume that
                                // virtualDiskDescType_temp.getCapacity() has
                                // been expressed using the default byte unit
                                // (see OVF specification).
                                virtualDiskCapacityInMB = virtualDiskCapacity / new Float(Math.pow(2, 20)).longValue();
                            } else if ("byte".equalsIgnoreCase(capacityAllocationUnits)) {
                                virtualDiskCapacityInMB = virtualDiskCapacity / new Float(Math.pow(2, 20)).longValue();
                            } else if (capacityAllocationUnits.startsWith("byte * 2^")
                                && capacityAllocationUnits.length() == 11 && capacityAllocationUnits.endsWith("0")) {
                                int i = new Integer(capacityAllocationUnits.substring(9, 10)).intValue();
                                if (i >= 0 && i <= 2) {
                                    virtualDiskCapacityInMB = virtualDiskCapacity
                                        / new Float(Math.pow(2, 20 - i * 10)).longValue();
                                } else {
                                    // Here, i can be 3, 4, 5, 6, 7, 8, or 9.
                                    virtualDiskCapacityInMB = virtualDiskCapacity
                                        * new Float(Math.pow(2, (i - 2) * 10)).longValue();
                                }
                            } else {
                                throw new OVFImporterException("ERROR: The given CapacityAllocationUnits ("
                                    + capacityAllocationUnits + ") of the VirtualSystem identified by: "
                                    + virtualDiskDescType_temp.getDiskId()
                                    + " is unknown. \"byte\" or \"byte * 2^20\" may be used.");
                            }
                            volumeTemplate.setCapacityInMB(virtualDiskCapacityInMB);
                            volumeTemplate.setSystemTemplate(systemTemplate);
                            systemTemplate.getVolumeTemplates().add(volumeTemplate);
                        }

                        this.em.persist(systemTemplate);
                        this.em.flush();

                    } else if (NetworkSectionType.class.getCanonicalName().equals(
                        sectionType_temp.getDeclaredType().getCanonicalName())) {

                        NetworkSectionType networkSectionType = (NetworkSectionType) sectionType_temp.getValue();

                        List<Network> network_list = networkSectionType.getNetwork();
                        Iterator<Network> network_temp_ITERATOR = network_list.iterator();
                        while (network_temp_ITERATOR.hasNext()) {
                            Network network_temp = network_temp_ITERATOR.next();

                            OVFImporterBean.log.finest(network_temp.getName());

                            // NetworkTemplate creation.
                            NetworkTemplate networkTemplate = new NetworkTemplate();
                            networkTemplate.setName(network_temp.getName());

                            networkTemplate.setSystemTemplate(systemTemplate);
                            systemTemplate.getNetworkTemplates().add(networkTemplate);
                        }

                        this.em.persist(systemTemplate);
                        this.em.flush();

                    } else {
                        OVFImporterBean.log.warning("WARNING: This section ("
                            + sectionType_temp.getDeclaredType().getCanonicalName() + ") is unknown: it is ignored.");
                        // throw new OVFImporterException(
                        // "This section is unknown: "
                        // + sectionType_temp.getDeclaredType()
                        // .getCanonicalName() + ".");
                    }
                }

                // Let's see the VirtualSystem(s).
                OVFImporterBean.log.finest("" + virtualSystemCollectionType.getContent());
                Iterator<JAXBElement<? extends ContentType>> virtualSystemType_temp_ITERATOR = virtualSystemCollectionType
                    .getContent().iterator();
                while (virtualSystemType_temp_ITERATOR.hasNext()) {
                    VirtualSystemType virtualSystemType_temp = (VirtualSystemType) virtualSystemType_temp_ITERATOR.next()
                        .getValue();
                    OVFImporterBean.log.finest(virtualSystemType_temp.getId());
                    OVFImporterBean.log.finest("" + virtualSystemType_temp.getSection());

                    VirtualMachineTemplate virtualMachineTemplate = new VirtualMachineTemplate();
                    virtualMachineTemplate.setName(virtualSystemType_temp.getId());

                    virtualMachineTemplate.setSystemTemplate(systemTemplate);
                    systemTemplate.getVirtualMachineTemplates().add(virtualMachineTemplate);

                    Iterator<JAXBElement<? extends SectionType>> virtualSystemSection_temp_ITERATOR = virtualSystemType_temp
                        .getSection().iterator();
                    while (virtualSystemSection_temp_ITERATOR.hasNext()) {
                        JAXBElement<? extends SectionType> virtualSystemSection_temp = virtualSystemSection_temp_ITERATOR
                            .next();
                        OVFImporterBean.log.finest(virtualSystemSection_temp.getDeclaredType().getCanonicalName());
                        if (ProductSectionType.class.getCanonicalName().equals(
                            virtualSystemSection_temp.getDeclaredType().getCanonicalName())) {
                            ProductSectionType productSectionType = (ProductSectionType) virtualSystemSection_temp.getValue();
                            OVFImporterBean.log.finest(productSectionType.getInfo().getValue());
                            // TODO ?
                        } else if (AnnotationSectionType.class.getCanonicalName().equals(
                            virtualSystemSection_temp.getDeclaredType().getCanonicalName())) {
                            AnnotationSectionType annotationSectionType = (AnnotationSectionType) virtualSystemSection_temp
                                .getValue();
                            OVFImporterBean.log.finest(annotationSectionType.getInfo().getValue());
                            // TODO ?
                        } else if (OperatingSystemSectionType.class.getCanonicalName().equals(
                            virtualSystemSection_temp.getDeclaredType().getCanonicalName())) {
                            OperatingSystemSectionType operatingSystemSectionType = (OperatingSystemSectionType) virtualSystemSection_temp
                                .getValue();
                            OVFImporterBean.log.finest(operatingSystemSectionType.getInfo().getValue());
                            // TODO ?
                        } else if (VirtualHardwareSectionType.class.getCanonicalName().equals(
                            virtualSystemSection_temp.getDeclaredType().getCanonicalName())) {
                            VirtualHardwareSectionType virtualHardwareSectionType = (VirtualHardwareSectionType) virtualSystemSection_temp
                                .getValue();
                            OVFImporterBean.log.finest(virtualHardwareSectionType.getInfo().getValue());
                            Iterator<RASDType> resourceAllocationSettingData_temp_ITERATOR = virtualHardwareSectionType
                                .getItem().iterator();
                            while (resourceAllocationSettingData_temp_ITERATOR.hasNext()) {
                                RASDType resourceAllocationSettingData_temp = resourceAllocationSettingData_temp_ITERATOR
                                    .next();

                                // resourceAllocationSettingData_temp
                                // .getResourceType().getValue() can be:
                                // Kind ResourceType
                                // Other 0
                                // Processor 3
                                // Memory 4
                                // IDE Controller 5
                                // SCSI Controller 6
                                // Ethernet Adapter 10
                                // Floppy Drive 14
                                // CD/DVD Drive 15/16
                                // Disk Drive 17
                                // USB Controller 23
                                //
                                // see:
                                // http://blogs.vmware.com/vapp/2009/11/virtual-hardware-in-ovf-part-1.html

                                if ("0".equals(resourceAllocationSettingData_temp.getResourceType().getValue())) {
                                    // TODO ?
                                    OVFImporterBean.log.warning("WARNING: This resource allocation setting data ("
                                        + resourceAllocationSettingData_temp.getResourceType().getValue()
                                        + ") is not handle yet: it is ignored.");
                                } else if ("3".equals(resourceAllocationSettingData_temp.getResourceType().getValue())) {
                                    // Processor
                                    // Here, we fill virtualMachineTemplate's
                                    // nbCpus attribut.
                                    virtualMachineTemplate.setNbCpus(new Integer(resourceAllocationSettingData_temp
                                        .getVirtualQuantity().getValue().toString()));

                                } else if ("4".equals(resourceAllocationSettingData_temp.getResourceType().getValue())) {
                                    // Memory
                                    // Here, we fill virtualMachineTemplate's
                                    // ramSizeMB attribut.
                                    virtualMachineTemplate.setRamSizeMB(new Integer(resourceAllocationSettingData_temp
                                        .getVirtualQuantity().getValue().toString()));
                                } else if ("5".equals(resourceAllocationSettingData_temp.getResourceType().getValue())) {
                                    // TODO ?
                                    OVFImporterBean.log.warning("WARNING: This resource allocation setting data ("
                                        + resourceAllocationSettingData_temp.getResourceType().getValue()
                                        + ") is not handle yet: it is ignored.");
                                } else if ("6".equals(resourceAllocationSettingData_temp.getResourceType().getValue())) {
                                    // TODO ?
                                    OVFImporterBean.log.warning("WARNING: This resource allocation setting data ("
                                        + resourceAllocationSettingData_temp.getResourceType().getValue()
                                        + ") is not handle yet: it is ignored.");
                                } else if ("10".equals(resourceAllocationSettingData_temp.getResourceType().getValue())) {
                                    // TODO ?
                                    OVFImporterBean.log.warning("WARNING: This resource allocation setting data ("
                                        + resourceAllocationSettingData_temp.getResourceType().getValue()
                                        + ") is not handle yet: it is ignored.");
                                } else if ("14".equals(resourceAllocationSettingData_temp.getResourceType().getValue())) {
                                    // TODO ?
                                    OVFImporterBean.log.warning("WARNING: This resource allocation setting data ("
                                        + resourceAllocationSettingData_temp.getResourceType().getValue()
                                        + ") is not handle yet: it is ignored.");
                                } else if (("15".equals(resourceAllocationSettingData_temp.getResourceType().getValue()))
                                    || ("16".equals(resourceAllocationSettingData_temp.getResourceType().getValue()))) {
                                    // TODO ?
                                    OVFImporterBean.log.warning("WARNING: This resource allocation setting data ("
                                        + resourceAllocationSettingData_temp.getResourceType().getValue()
                                        + ") is not handle yet: it is ignored.");
                                } else if ("17".equals(resourceAllocationSettingData_temp.getResourceType().getValue())) {
                                    // Hardware disks are handled below, via the
                                    // use of
                                    // resourceAllocationSettingData_temp.getHostResource()
                                    // XXX harddisks without VMImage aren't
                                    // kept for the moment.
                                    OVFImporterBean.log.warning("WARNING: This resource allocation setting data ("
                                        + resourceAllocationSettingData_temp.getResourceType().getValue()
                                        + ") is not handle yet: it is ignored.");
                                } else if ("23".equals(resourceAllocationSettingData_temp.getResourceType().getValue())) {
                                    // TODO ?
                                    OVFImporterBean.log.warning("WARNING: This resource allocation setting data ("
                                        + resourceAllocationSettingData_temp.getResourceType().getValue()
                                        + ") is not handle yet: it is ignored.");
                                } else {
                                    OVFImporterBean.log.warning("WARNING: This resource allocation setting data ("
                                        + resourceAllocationSettingData_temp.getResourceType().getValue()
                                        + ") is unknown: it is ignored.");
                                    // throw new OVFImporterException(
                                    // "ERROR: Unknown resourceType (coming from the OVF file): "
                                    // + resourceAllocationSettingData_temp
                                    // .getResourceType()
                                    // .getValue() + ".");
                                }

                                Iterator<CimString> hostResourceAsString_temp_ITERATOR = resourceAllocationSettingData_temp
                                    .getHostResource().iterator();
                                while (hostResourceAsString_temp_ITERATOR.hasNext()) {
                                    String hostResourceAsString_temp = hostResourceAsString_temp_ITERATOR.next().getValue();
                                    OVFImporterBean.log.finest(hostResourceAsString_temp);

                                    Iterator<VolumeTemplate> vT_temp_ITERATOR = systemTemplate.getVolumeTemplates().iterator();
                                    while (vT_temp_ITERATOR.hasNext()) {
                                        VolumeTemplate vT_temp = vT_temp_ITERATOR.next();

                                        if (hostResourceAsString_temp.endsWith(vT_temp.getName())) {
                                            vT_temp.getVirtualMachineTemplates().add(virtualMachineTemplate);
                                            virtualMachineTemplate.getVolumeTemplates().add(vT_temp);
                                        } else {
                                            // nothing to do.
                                        }
                                    }
                                }

                                OVFImporterBean.log.finest(resourceAllocationSettingData_temp.getElementName().getValue());

                                // Iterator<CimString>
                                // connectionAsString_temp_ITERATOR =
                                // resourceAllocationSettingData_temp
                                // .getConnection().iterator();
                                Iterator<CimString> connectionAsString_temp_ITERATOR = resourceAllocationSettingData_temp
                                    .getConnection().iterator();
                                while (connectionAsString_temp_ITERATOR.hasNext()) {
                                    String connectionAsString_temp = connectionAsString_temp_ITERATOR.next().getValue();

                                    OVFImporterBean.log.finest(connectionAsString_temp);

                                    NetworkInterfaceTemplate networkInterfaceTemplate = new NetworkInterfaceTemplate();
                                    networkInterfaceTemplate.setName(resourceAllocationSettingData_temp.getElementName()
                                        .getValue());
                                    networkInterfaceTemplate.setVirtualMachineTemplate(virtualMachineTemplate);
                                    virtualMachineTemplate.getNetworkInterfaceTemplates().add(networkInterfaceTemplate);

                                    OVFImporterBean.log.finest("" + networkInterfaceTemplate);

                                    // Let's link networkInterfaceTemplate, and
                                    // networkTemplate.
                                    Iterator<NetworkTemplate> networkTemplate_temp_ITERATOR = systemTemplate
                                        .getNetworkTemplates().iterator();
                                    while (networkTemplate_temp_ITERATOR.hasNext()) {
                                        NetworkTemplate networkTemplate_temp = networkTemplate_temp_ITERATOR.next();
                                        if (connectionAsString_temp.equals(networkTemplate_temp.getName())) {
                                            networkInterfaceTemplate.setNetworkTemplate(networkTemplate_temp);
                                            networkTemplate_temp.getNetworkInterfaceTemplates().add(networkInterfaceTemplate);
                                        } else {
                                            // nothing to do.
                                        }
                                    }
                                }

                            }
                        } else {
                            OVFImporterBean.log.warning("WARNING: This section ("
                                + virtualSystemSection_temp.getDeclaredType().getCanonicalName()
                                + ") is unknown: it is ignored.");
                            // throw new OVFImporterException(
                            // "The given (virtual system) section is unknown: "
                            // + virtualSystemSection_temp
                            // .getDeclaredType()
                            // .getCanonicalName() + ".");
                        }
                    }
                }

                this.em.persist(systemTemplate);
                this.em.flush();

                // Let's see the StartupSection, AnnotationSection, and
                // ProductSection
                OVFImporterBean.log.finest("" + virtualSystemCollectionType.getSection());

                Iterator<JAXBElement<? extends SectionType>> sectionOfVirtualSystemCollection_temp_ITERATOR = virtualSystemCollectionType
                    .getSection().iterator();
                while (sectionOfVirtualSystemCollection_temp_ITERATOR.hasNext()) {
                    JAXBElement<? extends SectionType> sectionOfVirtualSystemCollection_temp = sectionOfVirtualSystemCollection_temp_ITERATOR
                        .next();
                    if (StartupSectionType.class.getCanonicalName().equals(
                        sectionOfVirtualSystemCollection_temp.getDeclaredType().getCanonicalName())) {
                        StartupSectionType startupSectionType = (StartupSectionType) sectionOfVirtualSystemCollection_temp
                            .getValue();

                        Iterator<Item> itemOfStartupSectionType_temp_ITERATOR = startupSectionType.getItem().iterator();
                        while (itemOfStartupSectionType_temp_ITERATOR.hasNext()) {
                            Item itemOfStartupSectionType_temp = itemOfStartupSectionType_temp_ITERATOR.next();

                            String oVF_vMT_id = itemOfStartupSectionType_temp.getId();
                            Integer oVF_order = itemOfStartupSectionType_temp.getOrder();
                            Integer oVF_startDelay = itemOfStartupSectionType_temp.getStartDelay();
                            Integer oVF_stopDelay = itemOfStartupSectionType_temp.getStopDelay();

                            Iterator<VirtualMachineTemplate> virtualMachineTemplate_temp_ITERATOR = systemTemplate
                                .getVirtualMachineTemplates().iterator();
                            while (virtualMachineTemplate_temp_ITERATOR.hasNext()) {
                                VirtualMachineTemplate virtualMachineTemplate_temp = virtualMachineTemplate_temp_ITERATOR
                                    .next();

                                if (virtualMachineTemplate_temp.getName().equals(oVF_vMT_id)) {
                                    virtualMachineTemplate_temp.setoVF_order(oVF_order);
                                    virtualMachineTemplate_temp.setoVF_startDelay(oVF_startDelay);
                                    virtualMachineTemplate_temp.setoVF_stopDelay(oVF_stopDelay);
                                } else {
                                    // nothing to do.
                                }
                            }

                        }

                    } else if (AnnotationSectionType.class.getCanonicalName().equals(
                        sectionOfVirtualSystemCollection_temp.getDeclaredType().getCanonicalName())) {
                        // TODO ?
                        // AnnotationSectionType annotationSectionType =
                        // (AnnotationSectionType)
                        // sectionOfVirtualSystemCollection_temp
                        // .getValue();
                        OVFImporterBean.log.warning("WARNING: This section ("
                            + sectionOfVirtualSystemCollection_temp.getDeclaredType().getCanonicalName()
                            + ") is not handle yet: it is ignored.");
                    } else if (ProductSectionType.class.getCanonicalName().equals(
                        sectionOfVirtualSystemCollection_temp.getDeclaredType().getCanonicalName())) {
                        // TODO ?
                        // ProductSectionType productSectionType =
                        // (ProductSectionType)
                        // sectionOfVirtualSystemCollection_temp
                        // .getValue();
                        OVFImporterBean.log.warning("WARNING: This section ("
                            + sectionOfVirtualSystemCollection_temp.getDeclaredType().getCanonicalName()
                            + ") is not handle yet: it is ignored.");
                    } else {
                        OVFImporterBean.log.warning("WARNING: This section ("
                            + sectionOfVirtualSystemCollection_temp.getDeclaredType().getCanonicalName()
                            + ") is unknown: it is ignored.");
                        // throw new OVFImporterException(
                        // "The given (virtual system collection) section is unknown: "
                        // + sectionOfVirtualSystemCollection_temp
                        // .getDeclaredType()
                        // .getCanonicalName() + ".");
                    }
                }

                // Let's specify the VolumeTemplates' fileOVFHRef attribut.

                Iterator<VolumeTemplate> volumeTemplate_temp_ITERATOR = systemTemplate.getVolumeTemplates().iterator();
                while (volumeTemplate_temp_ITERATOR.hasNext()) {
                    VolumeTemplate volumeTemplate_temp = volumeTemplate_temp_ITERATOR.next();

                    Iterator<String> virtualDiskDescType_oVFDiskId_temp_ITERATOR = virtualDiskDescType_oVFDiskId_and_oVFFileRef_lHM
                        .keySet().iterator();
                    while (virtualDiskDescType_oVFDiskId_temp_ITERATOR.hasNext()) {
                        String virtualDiskDescType_oVFDiskId_temp = virtualDiskDescType_oVFDiskId_temp_ITERATOR.next();

                        if (volumeTemplate_temp.getName().equals(virtualDiskDescType_oVFDiskId_temp)) {
                            String virtualDiskDescType_oVFFileRef = virtualDiskDescType_oVFDiskId_and_oVFFileRef_lHM
                                .get(virtualDiskDescType_oVFDiskId_temp);
                            String fileType_oVFHRef = fileType_oVFId_and_oVFHRef_lHM.get(virtualDiskDescType_oVFFileRef);
                            volumeTemplate_temp.setFileOVFHRef(fileType_oVFHRef);
                            volumeTemplate_temp.setVirtualDiskDescType_oVFFileRef(virtualDiskDescType_oVFFileRef);

                            // We also launch the upload of the described
                            // vmimage file(s) via the VMImages manager module,
                            // it will then enable the real importation of the
                            // given OVF, and its associated vmimage file(s).
                            // Be careful when importing the vmimage files as
                            // VMImage Sirocco objects, their vmimage file
                            // format is given by the ovf (see DiskSection,
                            // virtualDiskDescType_temp.getFormat();).
                            // The above value can be retrieved via
                            // volumeTemplate_temp.
                            String virtualMachineImageFileURL = fileType_oVFHRef;
                            String format = volumeTemplate_temp.getVolumeFormat();
                            String name = virtualDiskDescType_oVFFileRef;
                            Integer oVFSizeInMB = fileType_oVFId_and_oVFSizeInMB_lHM.get(virtualDiskDescType_oVFFileRef);
                            // XXX description, checkSum, and cloudProviderId
                            // are ignored, i.e. set to null.
                            String description = null;
                            String checkSum = null;
                            String cloudProviderId = null;

                            // Creation of the VMImage.
                            VMImage vMImage = new VMImage();
                            vMImage.setUploadedFromURL(virtualMachineImageFileURL);
                            vMImage.setStatus(VMImage.Status.IN_CONSTRUCTION);
                            vMImage.setDescription(description);
                            vMImage.setVMImageFormat(format);
                            vMImage.setHypervisor(format);
                            vMImage.setName(name);
                            vMImage.setImageSizeMB(oVFSizeInMB);
                            // XXX setDiskSizeMB is used like in CloudAdminBean:
                            // createAndStartVirtualMachine.
                            vMImage.setDiskSizeMB(oVFSizeInMB);

                            // XXX vMImage_temp.setArchitecture(
                            // vs.getVmImage().getArchitecture());
                            // vMImage_temp.setOsType(vs.getVmImage().getOsType());
                            // vMImage_temp.setVisibility(visibility);
                            // vMImage_temp.setIsBootable(isBootable);

                            // TODO vMImage_temp.setUser(this.
                            // userManager.getUserByUsername(fromUsername));
                            // vMImage_temp.setProject(proj);

                            // Associate the volumenTemplate, and its vMImage.
                            volumeTemplate_temp.setVirtualMachineImage(vMImage);
                            vMImage.getVolumeTemplates().add(volumeTemplate_temp);

                            this.em.persist(vMImage);
                            this.em.flush();

                            this.vMImageManager.uploadVirtualMachineImageFile(virtualMachineImageFileURL, format, name,
                                description, checkSum, oVFSizeInMB, cloudProviderId, Boolean.TRUE);
                        } else {
                            // nothing to do.
                        }
                    }
                }

                // XXX Ideally, the systemTemplate's status should be set to
                // AVAILABLE only when all its vmimages upload run correctly.
                systemTemplate.setStatus(SystemTemplateStatus.AVAILABLE);

                this.em.persist(systemTemplate);
                this.em.flush();

                //

                // Here, we display the systemTemplate in order to see if it
                // is well created.

                OVFImporterBean.log.info("\n\n\nSystemTemplate DISPLAY:\n\n");

                OVFImporterBean.log.info("" + systemTemplate);

                // Let's list the networkTemplate(s).
                Iterator<NetworkTemplate> nT_temp_ITERATOR = systemTemplate.getNetworkTemplates().iterator();
                while (nT_temp_ITERATOR.hasNext()) {
                    NetworkTemplate nT_temp = nT_temp_ITERATOR.next();
                    OVFImporterBean.log.info("" + nT_temp);

                    Iterator<NetworkInterfaceTemplate> nIT_temp_ITERATOR = nT_temp.getNetworkInterfaceTemplates().iterator();
                    while (nIT_temp_ITERATOR.hasNext()) {
                        NetworkInterfaceTemplate nIT_temp = nIT_temp_ITERATOR.next();
                        OVFImporterBean.log.info("" + nIT_temp);

                    }
                }

                // Let's list the virtualMachineTemplate(s).
                Iterator<VirtualMachineTemplate> vMT_temp_ITERATOR = systemTemplate.getVirtualMachineTemplates().iterator();
                while (vMT_temp_ITERATOR.hasNext()) {
                    VirtualMachineTemplate vMT_temp = vMT_temp_ITERATOR.next();
                    OVFImporterBean.log.info("" + vMT_temp);

                    // Let's list the volumeTemplate(s).
                    Iterator<VolumeTemplate> volumeTemplate_temp_ITERATOR_2 = vMT_temp.getVolumeTemplates().iterator();
                    while (volumeTemplate_temp_ITERATOR_2.hasNext()) {
                        VolumeTemplate volumeTemplate_temp = volumeTemplate_temp_ITERATOR_2.next();
                        OVFImporterBean.log.info("" + volumeTemplate_temp);
                    }

                    // Let's list the networkInterfaceTemplate(s).
                    Iterator<NetworkInterfaceTemplate> networkInterfaceTemplate_temp_ITERATOR = vMT_temp
                        .getNetworkInterfaceTemplates().iterator();
                    while (networkInterfaceTemplate_temp_ITERATOR.hasNext()) {
                        NetworkInterfaceTemplate networkInterfaceTemplate_temp = networkInterfaceTemplate_temp_ITERATOR.next();
                        OVFImporterBean.log.info("" + networkInterfaceTemplate_temp);
                    }
                }
                return systemTemplate;
            }
        }
    }
}
