package org.ow2.sirocco.cloudmanager.service;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Collection;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Stateless;
import javax.ejb.Timeout;
import javax.ejb.Timer;
import javax.ejb.TimerService;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.Query;

import org.ow2.easybeans.osgi.annotation.OSGiResource;
import org.ow2.sirocco.cloudmanager.admin.event.VirtualMachineStatusChangeEvent;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineVO;
import org.ow2.sirocco.cloudmanager.business.VirtualMachine;
import org.ow2.sirocco.cloudmanager.business.VirtualMachine.Status;
import org.ow2.sirocco.cloudmanager.clustermanager.api.IClusterManager;
import org.ow2.sirocco.cloudmanager.common.DbParameters;
import org.ow2.sirocco.vmm.api.GuestInfo;
import org.ow2.sirocco.vmm.api.VirtualMachineMXBean;
import org.ow2.sirocco.vmm.api.VirtualMachineMXBean.PowerState;

@Stateless(name = "RefreshVMStateTimerSessionBean", mappedName = "RefreshVMStateTimerSessionBean")
@Local(IRefreshVMStateTimer.class)
public class RefreshVMStateTimerSessionBean implements IRefreshVMStateTimer {
    private Logger log = Logger.getLogger(RefreshVMStateTimerSessionBean.class.getName());

    @Resource
    private TimerService timerService;

    @EJB(mappedName = "CloudAdminBean")
    IRemoteCloudAdmin cloudAdmin;

    @PersistenceContext(type = PersistenceContextType.TRANSACTION)
    protected EntityManager em;

    @OSGiResource
    private IClusterManager clusterManager;

    public void startTimer() {
        long periodInSeconds = DbParameters.getInstance().VM_REFRESH_PERIOD_SECONDS;
        this.timerService.createTimer(1000 * 60, periodInSeconds * 1000, null);
        this.log.info("RefreshVMStateTimerSessionBean started, period=" + periodInSeconds + " sec");
    }

    @Timeout
    public void handleTimeout(final Timer timer) {
        Query query = this.em.createQuery("FROM VirtualMachine where status NOT IN ('DESTROYED','ERROR')");

        @SuppressWarnings("unchecked")
        List<VirtualMachine> result = query.getResultList();
        if (result.size() > 0) {
            for (Object obj : result) {
                VirtualMachine vs = (VirtualMachine) obj;
                VirtualMachineMXBean vmMBean = null;
                try {
                    if (vs.getProviderId() == null) {
                        continue;
                    }
                    vmMBean = this.clusterManager.getVirtualMachineMBean(vs.getProviderId());
                    if (vmMBean != null) {
                        VirtualMachine.Status savedStatus = vs.getStatus();
                        VirtualMachine.Status newStatus = savedStatus;
                        PowerState powerState = vmMBean.getState();
                        switch (powerState) {
                        case HALTED:
                            newStatus = VirtualMachine.Status.STOPPED;
                            break;
                        case PAUSED:
                            newStatus = VirtualMachine.Status.PAUSED;
                            break;
                        case RUNNING:
                            newStatus = VirtualMachine.Status.RUNNING;
                            break;
                        case SUSPENDED:
                            newStatus = VirtualMachine.Status.SUSPENDED;
                            break;
                        case UNKNOWN:
                            // XXX
                            break;
                        }
                        boolean updateDB = false;
                        if (newStatus != savedStatus) {
                            vs.setStatus(newStatus);
                            updateDB = true;
                        }
                        if (newStatus == Status.RUNNING) {
                            GuestInfo guestInfo = vmMBean.getGuestInfo();
                            if (guestInfo != null && guestInfo.getIpAddresses() != null
                                && guestInfo.getIpAddresses().size() > 0) {
                                String vmIpAddress = guestInfo.getIpAddresses().get(0);
                                if (vs.getIp() == null || !vs.getIp().equals(vmIpAddress)) {
                                    vs.setIp(vmIpAddress);
                                    InetAddress addr = null;
                                    try {
                                        addr = InetAddress.getByName(vmIpAddress);
                                    } catch (UnknownHostException e) {
                                    }
                                    if (addr != null) {
                                        vs.setDnsName(addr.getHostName());
                                    }
                                    updateDB = true;
                                }
                            }
                        }
                        if (updateDB) {
                            this.log.info("Updating VM " + vs.getName() + " status=" + vs.getStatus() + " ip=" + vs.getIp());
                            this.em.merge(vs);
                            this.em.flush();
                            this.cloudAdmin.emitTopicMessage(new VirtualMachineStatusChangeEvent(VirtualMachineVO.from(vs)),
                                "projectId", vs.getProject().getProjectId());
                        }
                    } else {
                        this.log.severe("Cannot retrieve MBean for VM " + vs.getName());
                    }
                } catch (Exception ex) {
                    this.log.log(Level.WARNING, "Error while updating status and @IP of VM " + vs.getName(), ex);
                }
            }
        }
    }

    public void checkStatus() {
        @SuppressWarnings("unchecked")
        Collection<Timer> timers = this.timerService.getTimers();
        for (Timer timer : timers) {
            this.log.info("Timer will expire after " + timer.getTimeRemaining() + " milliseconds.\n");
        }
    }

    public void stopTimers() {
        @SuppressWarnings("unchecked")
        Collection<Timer> timers = this.timerService.getTimers();
        for (Timer timer : timers) {
            timer.cancel();
        }
        this.log.info("RefreshVMStateTimerSessionBean stopped");
    }
}
