/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: SystemTemplateManagerBean.java 229 2011-08-04 14:07:22Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.Query;

import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidSystemInstanceStatusException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidSystemTemplateStatusException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidUsernameException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.vo.FromNonVOToVO;
import org.ow2.sirocco.cloudmanager.admin.vo.SystemTemplateVO;
import org.ow2.sirocco.cloudmanager.business.CustomizationProperties;
import org.ow2.sirocco.cloudmanager.business.NetworkInterfaceTemplate;
import org.ow2.sirocco.cloudmanager.business.NetworkTemplate;
import org.ow2.sirocco.cloudmanager.business.SystemInstance;
import org.ow2.sirocco.cloudmanager.business.SystemInstance.SystemInstanceStatus;
import org.ow2.sirocco.cloudmanager.business.SystemTemplate;
import org.ow2.sirocco.cloudmanager.business.SystemTemplate.SystemTemplateStatus;
import org.ow2.sirocco.cloudmanager.business.User;
import org.ow2.sirocco.cloudmanager.business.VMImage;
import org.ow2.sirocco.cloudmanager.business.VirtualMachineTemplate;
import org.ow2.sirocco.cloudmanager.business.VolumeTemplate;
import org.ow2.sirocco.cloudmanager.utils.PermissionChecker;

@Stateless(name = "SystemTemplateManagerBean", mappedName = "SystemTemplateManagerBean")
@Remote(IRemoteSystemTemplateManager.class)
@Local(ISystemTemplateManager.class)
public class SystemTemplateManagerBean implements IRemoteSystemTemplateManager, ISystemTemplateManager {

    private static Logger log = Logger.getLogger(SystemTemplateManagerBean.class.getName());

    @PersistenceContext(unitName = "persistence-unit/main", type = PersistenceContextType.TRANSACTION)
    private EntityManager em = null;

    @Override
    public List<SystemTemplateVO> listSystemTemplates() {
        SystemTemplateManagerBean.log.info("Enters in: listSystemTemplates()");
        List<SystemTemplateVO> result;
        Query systemTemplateList = this.em.createQuery("FROM " + SystemTemplate.class.getSimpleName() + " s");
        if (systemTemplateList == null) {
            result = null;
        } else {
            result = new ArrayList<SystemTemplateVO>();
            @SuppressWarnings("unchecked")
            List<SystemTemplate> stList = systemTemplateList.getResultList();
            for (SystemTemplate st : stList) {
                result.add(FromNonVOToVO.fromSystemTemplateToSystemTemplateVO(st));
            }
        }
        SystemTemplateManagerBean.log.finest("Leaves: listSystemTemplates()");
        return result;
    }

    @Override
    public void destroySystemTemplate(final String userName, final Integer systemTemplateId) throws InvalidUsernameException,
        InvalidArgumentException, PermissionDeniedException, InvalidSystemInstanceStatusException,
        InvalidSystemTemplateStatusException {
        SystemTemplateManagerBean.log.info("Enters in: destroySystemTemplate(...)");

        // Check if a user corresponding to userName exists.
        User user = null;
        Query userList = this.em.createQuery("FROM " + User.class.getSimpleName() + " u WHERE u.username=:userName")
            .setParameter("userName", userName);
        if (userList == null) {
            throw new InvalidArgumentException("The given userName: " + userName + " is unknown.");
        } else {
            if (userList.getResultList().isEmpty()) {
                throw new InvalidArgumentException("The given userName: " + userName + " is unknown.");
            } else {
                user = (User) userList.getResultList().iterator().next();
            }
        }

        // Check user's permission.
        if (new PermissionChecker().canManageSystemTemplates(user)) {
            // nothing to do.
        } else {
            throw new PermissionDeniedException("Permission denied.");
        }

        // Check if a SystemTemplate corresponding to systemTemplateId exists.
        SystemTemplate st = null;
        Query systemTemplateList = this.em.createQuery("FROM " + SystemTemplate.class.getSimpleName() + " s WHERE s.id=:id")
            .setParameter("id", systemTemplateId);
        if (systemTemplateList == null) {
            throw new InvalidArgumentException("The given systemTemplate: " + systemTemplateId + " is unknown.");
        } else {
            if (systemTemplateList.getResultList().isEmpty()) {
                throw new InvalidArgumentException("The given systemTemplate: " + systemTemplateId + " is unknown.");
            } else {
                st = (SystemTemplate) systemTemplateList.getResultList().iterator().next();
            }
        }

        // Check that it is in the AVAILABLE state.
        if (SystemTemplateStatus.AVAILABLE.equals(st.getStatus())) {
            // nothing to do.
        } else {
            throw new InvalidSystemTemplateStatusException("The given systemTemplate: " + systemTemplateId + " is NOT in the "
                + SystemTemplateStatus.AVAILABLE + " state.");
        }

        // Check that st has no SystemInstances at all, or only destroyed ones.
        for (SystemInstance siTemp : st.getSystemInstances()) {
            if (SystemInstanceStatus.DESTROYED.equals(siTemp.getStatus())) {
                // nothing to do.
            } else {
                throw new InvalidSystemInstanceStatusException("The given systemTemplate: " + systemTemplateId
                    + " has, at least, one systemInstance: " + siTemp.getId() + " that is NOT in the "
                    + SystemInstanceStatus.DESTROYED + " state.");
            }
        }

        // "Destroy" the SystemTemplate's related objects, and the
        // SystemTemplate's itself :
        // - i.e., move SystemTemplate's status to DESTROYED
        // Move its state to DESTROYING.
        st.setStatus(SystemTemplateStatus.DESTROYED);
        this.em.merge(st);
        this.em.flush();

        SystemTemplateManagerBean.log.finest("Leaves: destroySystemTemplate(...)");
    }

    @Override
    public void cleanUpDestroyedSystemTemplates() throws InvalidSystemInstanceStatusException {
        SystemTemplateManagerBean.log.info("Enters in: cleanUpDestroyedSystemTemplates(...)");

        // Retrieve all the destroyed SystemTemplate(s).
        Query destroyedSystemTemplatesQuery = this.em.createQuery(
            "FROM " + SystemTemplate.class.getSimpleName() + " s WHERE s.status=:status").setParameter("status",
            SystemTemplateStatus.DESTROYED);
        if (destroyedSystemTemplatesQuery == null) {
            // nothing to do.
        } else {
            @SuppressWarnings("unchecked")
            List<SystemTemplate> destroyedSystemTemplatesList = destroyedSystemTemplatesQuery.getResultList();

            for (SystemTemplate stTemp : destroyedSystemTemplatesList) {
                // Check that stTemp has no SystemInstances at all (even
                // destroyed ones).
                if (stTemp.getSystemInstances().isEmpty()) {
                    // nothing to do.
                } else {
                    throw new InvalidSystemInstanceStatusException("The given systemTemplate: " + stTemp.getId()
                        + " has, at least, one remaining systemInstance: "
                        + stTemp.getSystemInstances().iterator().next().getId()
                        + " that should be cleaned up before requesting the clean up of the systemTemplate itself.");
                }
            }

            for (SystemTemplate stTemp : destroyedSystemTemplatesList) {
                // Clean up (i.e., delete) this SystemTemplate's
                // NetworkInterfaceTemplate(s), and NetworkTemplate(s)
                ArrayList<Integer> ntIdToDelete = new ArrayList<Integer>();
                for (NetworkTemplate nt : stTemp.getNetworkTemplates()) {
                    ntIdToDelete.add(nt.getId());
                }

                for (Integer ntId : ntIdToDelete) {
                    NetworkTemplate nt = null;
                    nt = this.em.find(NetworkTemplate.class, ntId);

                    // Clean up (i.e., delete) the NetworkInterfaceTemplate(s)
                    // it/themselves.
                    ArrayList<Integer> nitIdToDelete = new ArrayList<Integer>();
                    for (NetworkInterfaceTemplate nit : nt.getNetworkInterfaceTemplates()) {
                        nitIdToDelete.add(nit.getId());
                    }

                    for (Integer nitId : nitIdToDelete) {
                        NetworkInterfaceTemplate nit = null;
                        nit = this.em.find(NetworkInterfaceTemplate.class, nitId);

                        nt.getNetworkInterfaceTemplates().remove(nit);
                        nit.setNetworkTemplate(null);

                        VirtualMachineTemplate vmt = nit.getVirtualMachineTemplate();
                        vmt.getNetworkInterfaceTemplates().remove(nit);
                        nit.setVirtualMachineTemplate(null);

                        this.em.remove(nit);
                        this.em.flush();
                    }

                    stTemp.getNetworkTemplates().remove(nt);
                    nt.setSystemTemplate(null);

                    this.em.remove(nt);
                    this.em.flush();
                }

                // Clean up (i.e., delete) this SystemInstance's
                // VolumeTemplate(s)
                ArrayList<Integer> vtIdToDelete = new ArrayList<Integer>();
                for (VolumeTemplate vtTemp : stTemp.getVolumeTemplates()) {
                    vtIdToDelete.add(vtTemp.getId());
                }

                for (Integer vtIdTemp : vtIdToDelete) {
                    VolumeTemplate vt = null;
                    vt = this.em.find(VolumeTemplate.class, vtIdTemp);

                    VMImage vmImage = vt.getVirtualMachineImage();
                    if (vmImage == null) {
                        // nothing to do.
                    } else {
                        vmImage.getVolumeTemplates().remove(vt);
                        vt.setVirtualMachineImage(null);
                    }

                    ArrayList<Integer> associatedVmtId = new ArrayList<Integer>();
                    for (VirtualMachineTemplate vmtTemp : vt.getVirtualMachineTemplates()) {
                        associatedVmtId.add(vmtTemp.getId());
                    }

                    for (Integer vmtIdTemp : associatedVmtId) {
                        VirtualMachineTemplate vmt = null;
                        vmt = this.em.find(VirtualMachineTemplate.class, vmtIdTemp);
                        vmt.getVolumeTemplates().remove(vt);
                        vt.getVirtualMachineTemplates().remove(vmt);
                    }

                    stTemp.getVolumeTemplates().remove(vt);
                    vt.setSystemTemplate(stTemp);

                    this.em.remove(vt);
                    this.em.flush();
                }

                // Clean up (i.e., delete) this SystemInstance's
                // VirtualMachineTemplate(s) (and the
                // VirtualMachineTemplate(s)'s customization
                // properties)
                ArrayList<Integer> vmtIdToDelete = new ArrayList<Integer>();
                for (VirtualMachineTemplate vmt : stTemp.getVirtualMachineTemplates()) {
                    vmtIdToDelete.add(vmt.getId());
                }

                for (Integer vmtId : vmtIdToDelete) {
                    VirtualMachineTemplate vmt = null;
                    vmt = this.em.find(VirtualMachineTemplate.class, vmtId);

                    ArrayList<Integer> cpIdToDelete = new ArrayList<Integer>();
                    for (CustomizationProperties cpTemp : vmt.getCustomizationProperties()) {
                        cpIdToDelete.add(cpTemp.getId());
                    }

                    for (Integer cpIdTemp : cpIdToDelete) {
                        CustomizationProperties cp = null;
                        cp = this.em.find(CustomizationProperties.class, cpIdTemp);

                        vmt.getCustomizationProperties().remove(cp);
                        cp.setVirtualMachineTemplate(null);

                        if (cp.getSystemTemplate() == null) {
                            this.em.remove(cp);
                            this.em.flush();
                        } else {
                            this.em.merge(cp);
                            this.em.flush();
                        }
                    }

                    stTemp.getVirtualMachineTemplates().remove(vmt);
                    vmt.setSystemTemplate(null);

                    this.em.remove(vmt);
                    this.em.flush();
                }

                // Clean up (i.e., delete) this SystemTemplate's
                // CustomizationProperties, and this SystemTemplate itself.
                ArrayList<Integer> cpIdToDelete = new ArrayList<Integer>();
                for (CustomizationProperties cpTemp : stTemp.getCustomizationProperties()) {
                    cpIdToDelete.add(cpTemp.getId());
                }

                for (Integer cpIdTemp : cpIdToDelete) {
                    CustomizationProperties cp = null;
                    cp = this.em.find(CustomizationProperties.class, cpIdTemp);

                    stTemp.getCustomizationProperties().remove(cp);
                    cp.setSystemTemplate(null);

                    this.em.remove(cp);
                    this.em.flush();
                }

                this.em.remove(stTemp);
                this.em.flush();
            }
        }

        SystemTemplateManagerBean.log.finest("Leaves: cleanUpDestroyedSystemTemplates(...)");
    }
}