/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: UserManagerBean.java 233 2011-08-08 10:08:06Z otterley $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.Query;

import org.ow2.sirocco.cloudmanager.admin.event.DelUserEvent;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidUsernameException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.exception.ProjectInUseException;
import org.ow2.sirocco.cloudmanager.admin.exception.UserAlreadyExistsException;
import org.ow2.sirocco.cloudmanager.admin.exception.UserAlreadyInProjectException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMImageInUseException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMInUseException;
import org.ow2.sirocco.cloudmanager.admin.exception.VolumeInUseException;
import org.ow2.sirocco.cloudmanager.admin.vo.ProjectVO;
import org.ow2.sirocco.cloudmanager.admin.vo.UserVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VolumeVO;
import org.ow2.sirocco.cloudmanager.business.User;
import org.ow2.sirocco.cloudmanager.business.VirtualMachine;
import org.ow2.sirocco.cloudmanager.business.Volume;
import org.ow2.sirocco.cloudmanager.common.DbParameters;
import org.ow2.sirocco.cloudmanager.realm.LdapUtil;
import org.ow2.sirocco.cloudmanager.utils.PasswordGenerator;
import org.ow2.sirocco.cloudmanager.utils.PermissionChecker;

@Stateless(name = "UserManagerBean", mappedName = "UserManagerBean")
@Remote(IRemoteUserManager.class)
@Local(IUserManager.class)
public class UserManagerBean implements IUserManager {

    private static Logger log = Logger.getLogger(UserManagerBean.class.getName());

    @PersistenceContext(unitName = "persistence-unit/main", type = PersistenceContextType.TRANSACTION)
    protected EntityManager em;

    @EJB(mappedName = "CloudAdminBean")
    private IRemoteCloudAdmin cloudAdmin;

    @EJB(mappedName = "ProjectManagerBean")
    private IProjectManager projectManager;

    public User createUser(final String username, final String password, final String userMail, final String userFirstName,
        final String userLastName) throws UserAlreadyExistsException {
        Date todayDate = new Date();

        Query queryVm = this.em.createQuery("FROM User u WHERE u.username=:username").setParameter("username", username);
        if (queryVm.getResultList().size() > 0) {
            throw new UserAlreadyExistsException("User " + username + " already exists");
        }

        User user = new User();
        user.setUsername(username.toLowerCase());
        user.setEmail(userMail);
        user.setFirstName(userFirstName);
        user.setLastName(userLastName);
        user.setCreateDate(todayDate);
        if (password != null) {
            user.setPassword(password);
        }
        user.setApiPassword(PasswordGenerator.getNext());
        this.em.persist(user);

        // Default ProjectId and project Name is username
        try {
            this.projectManager.createProject(username, username, username + " default project", username);
        } catch (UserAlreadyInProjectException e) {
            UserManagerBean.log.log(Level.SEVERE, "User already in project!", e);
        }

        return user;
    }

    public User getUserByUsername(final String username) {
        User user = null;
        try {
            user = (User) this.em.createQuery("FROM User WHERE username = :name").setParameter("name", username)
                .getSingleResult();
        } catch (NoResultException noResult) {
            if (DbParameters.getInstance().USE_LDAP_SERVER_FOR_AUTHENTIFICATION
                && DbParameters.getInstance().AUTO_CREATE_ACCOUNT) {
                String ldapAccount = LdapUtil.getInstance().getName(username);
                if (null != ldapAccount) {
                    try {
                        user = this.createUser(username, null, LdapUtil.getInstance().getEmail(username), LdapUtil
                            .getInstance().getFirstName(username), LdapUtil.getInstance().getLastName(username));
                    } catch (UserAlreadyExistsException e) {
                        e.printStackTrace();
                    }
                }
            }
        } catch (Exception ex) {
            UserManagerBean.log.log(Level.SEVERE, "Get user query problem", ex);
        }
        return user;
    }

    public UserVO getUserVoLdapInfoByUsername(final String username) {
        UserVO userVo = new UserVO();
        String email = LdapUtil.getInstance().getEmail(username);
        if (email == null) {
            userVo.setEmail("Unknown user");
        } else {
            userVo.setEmail(email);
        }
        return userVo;
    }

    public List<User> getUsersByProjectId(final String projectId) {
        @SuppressWarnings("unchecked")
        List<User> list = this.em.createQuery("SELECT user FROM RoleAssignment r WHERE r.project.projectId=:projectId")
            .setParameter("projectId", projectId).getResultList();
        return list;
    }

    private String setApiPasswordForUser(final User user) {
        String pw = PasswordGenerator.getNext();
        user.setApiPassword(pw);
        this.em.merge(user);
        this.em.flush();
        return pw;
    }

    public String getApiPasswordForUser(final String username) {
        User u;
        try {
            u = (User) this.em.createQuery("FROM User WHERE username = :user").setParameter("user", username).getSingleResult();
        } catch (NoResultException e) {
            return null; // user does not exists
        }
        String passwd = null;
        if (u.getApiPassword() != null) {
            passwd = u.getApiPassword();
        } else {
            passwd = this.setApiPasswordForUser(u);
        }

        return passwd;
    }

    public boolean authenticateUser(final String username, final String password) throws InvalidUsernameException {
        User user = this.getUserByUsername(username);
        String pass = null;
        if (user == null) {
            throw new InvalidUsernameException("username " + username + " unknown");
        }
        try {
            pass = user.getPassword();
            if (pass.equals(password)) {
                return true;
            }
            return false;
        } catch (Exception e) {
            if (DbParameters.getInstance().USE_LDAP_SERVER_FOR_AUTHENTIFICATION) {
                if (LdapUtil.getInstance().validate(username.toLowerCase(), password)) {
                    return true;
                }
            }
        }
        return false;
    }

    @SuppressWarnings("unchecked")
    private Collection<User> getAllUsers() {
        Query query = this.em.createQuery("SELECT u FROM User u WHERE u.username.admin IS NOT true");
        return query.getResultList();
    }

    @SuppressWarnings("unchecked")
    public List<User> getAllAdmins() {
        Query query = this.em.createQuery("SELECT u FROM User u WHERE u.username.admin IS true");
        return query.getResultList();
    }

    public List<UserVO> getAllUsersVo() {
        Collection<User> list = this.getAllUsers();
        List<UserVO> result = new ArrayList<UserVO>();
        for (User user : list) {
            result.add(new UserVO().from(user, null, true));
        }
        return result;
    }

    public UserVO getUserVoByUsername(final String username) throws InvalidUsernameException {
        User user = this.getUserByUsername(username);

        if (user == null) {
            throw new InvalidUsernameException("username " + username + " unknown");
        }
        return new UserVO().from(user, null, true);
    }

    public List<VirtualMachineVO> getVirtualMachinesVoByUsername(final String username, final boolean includeDestroyedVMs)
        throws InvalidUsernameException {
        User user = this.getUserByUsername(username);
        if (user == null) {
            throw new InvalidUsernameException("username " + username + " unknown");
        }

        Collection<VirtualMachine> vmsList = user.getVirtualMachines();

        List<VirtualMachineVO> result = new ArrayList<VirtualMachineVO>();
        for (VirtualMachine vm : vmsList) {
            if (includeDestroyedVMs || vm.getStatus() != VirtualMachine.Status.DESTROYED) {
                result.add(VirtualMachineVO.from(vm));
            }
        }
        return result;
    }

    @Override
    public List<VolumeVO> getVolumeVoByUser(final String username) throws InvalidUsernameException {
        User user = this.getUserByUsername(username);
        if (user == null) {
            throw new InvalidUsernameException("username " + username + " unknown");
        }

        Collection<Volume> vmsList = user.getVolumes();

        List<VolumeVO> result = new ArrayList<VolumeVO>();
        for (Object obj : vmsList) {
            result.add(VolumeVO.from((Volume) obj));
        }
        return result;
    }

    public void updateUserSSHKey(final String username, final String sshKey) throws InvalidUsernameException {
        @SuppressWarnings("unchecked")
        List<User> users = this.em.createQuery("FROM User u WHERE u.username=:username").setParameter("username", username)
            .getResultList();
        if (users.size() > 0) {
            User user = users.get(0);
            user.setPublicKey(sshKey);
            this.em.merge(user);
            this.em.flush();
        } else {
            throw new InvalidUsernameException("username " + username + " unknown");
        }
    }

    public void updateUserPasswords(final String username, final String webPassword, final String apiPassword)
        throws InvalidUsernameException {
        User user = this.getUserByUsername(username);
        if (user == null) {
            throw new InvalidUsernameException("username " + username + " unknown");
        }
        user.setPassword(webPassword);
        user.setApiPassword(apiPassword);
        this.em.merge(user);
        this.em.flush();
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
    public void deleteUser(final String username, final String fromUsername) throws InvalidUsernameException, VMInUseException,
        VMImageInUseException, InvalidProjectIdException, ProjectInUseException, PermissionDeniedException,
        VolumeInUseException {

        User user = this.getUserByUsername(username);
        User fromUser = this.getUserByUsername(fromUsername);

        // check authorization
        if (!new PermissionChecker().canDestroyUser(fromUser)) {
            throw new PermissionDeniedException("Permission denied");
        }

        if (user == null || fromUser == null) {
            throw new InvalidUsernameException("User unknown");
        }

        Query queryVm = this.em.createQuery("FROM VirtualMachine v WHERE v.user.username=:username AND v.status<>'DESTROYED'")
            .setParameter("username", username);
        if (queryVm.getResultList().size() > 0) {
            throw new VMInUseException("User " + username + " has still VM in used");
        }

        Query queryIm = this.em.createQuery("FROM VMImage v WHERE v.user.username=:username AND v.status<>'DESTROYED'")
            .setParameter("username", username);
        if (queryIm.getResultList().size() > 0) {
            throw new VMImageInUseException("User " + username + " has still VM images in used");
        }

        Query queryVol = this.em.createQuery("FROM Volume v WHERE v.user.username=:username AND v.status<>'DESTROYED'")
            .setParameter("username", username);
        if (queryVol.getResultList().size() > 0) {
            throw new VolumeInUseException("User " + username + " has still Volumes in used");
        }

        List<ProjectVO> listProjVo = this.projectManager.getProjectsVoByUsername(username);

        for (ProjectVO projVo : listProjVo) {
            if (!projVo.getOwner().equals(username)) {
                this.projectManager.delUserFromProject(username, projVo.getProjectId(), null);
            } else if ((!projVo.getProjectId().equals(projVo.getOwner()) && (projVo.getOwner().equals(username)))) {
                List<User> users = this.getUsersByProjectId(projVo.getProjectId());
                if (users.size() > 1) {
                    throw new ProjectInUseException("User " + username + " has still users in project " + projVo.getName());
                } else {
                    this.projectManager.deleteProject(projVo.getProjectId());
                }
            }
        }

        listProjVo = this.projectManager.getProjectsVoByUsername(username);

        for (ProjectVO projVo : listProjVo) {
            if (projVo.getProjectId().equals(projVo.getOwner())) {
                List<User> users = this.getUsersByProjectId(projVo.getProjectId());
                if (users.size() > 1) {
                    throw new ProjectInUseException("User " + username + " has still users in project " + projVo.getName());
                } else {
                    this.projectManager.delUserFromProject(username, projVo.getProjectId(), null);
                    this.projectManager.deleteProject(projVo.getProjectId());
                    this.em.remove(user);
                }
            }
        }

        this.cloudAdmin.emitTopicMessage(new DelUserEvent(username), "admin", "true");
    }
}
