package org.ow2.sirocco.cloudmanager.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;

import org.ow2.easybeans.osgi.annotation.OSGiResource;
import org.ow2.sirocco.cloudmanager.admin.event.DelVolumeEvent;
import org.ow2.sirocco.cloudmanager.admin.event.NewVolumeEvent;
import org.ow2.sirocco.cloudmanager.admin.event.VolumeUpdateEvent;
import org.ow2.sirocco.cloudmanager.admin.exception.CloudAdminException;
import org.ow2.sirocco.cloudmanager.admin.exception.DuplicateNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidUsernameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVirtualMachineIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVolumeIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.exception.ResourceQuotaExceededException;
import org.ow2.sirocco.cloudmanager.admin.exception.VolumeInUseException;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VolumeVO;
import org.ow2.sirocco.cloudmanager.business.Project;
import org.ow2.sirocco.cloudmanager.business.User;
import org.ow2.sirocco.cloudmanager.business.VirtualMachine;
import org.ow2.sirocco.cloudmanager.business.Volume;
import org.ow2.sirocco.cloudmanager.clustermanager.api.IClusterManager;
import org.ow2.sirocco.cloudmanager.clustermanager.api.VolumeSpec;
import org.ow2.sirocco.cloudmanager.utils.PermissionChecker;
import org.ow2.sirocco.vmm.api.VMMException;

@Stateless(name = "VolumeManagerBean", mappedName = "VolumeManagerBean")
@Remote(IRemoteVolumeManager.class)
@Local(IVolumeManager.class)
public class VolumeManagerBean implements IVolumeManager {
    private static long MAX_VOLUME_SIZE_IN_MB_ALLOWED = 500;

    private static Logger log = Logger.getLogger(VolumeManagerBean.class.getName());

    @PersistenceContext(unitName = "persistence-unit/main", type = PersistenceContextType.TRANSACTION)
    protected EntityManager em;

    @EJB(mappedName = "CloudAdminBean")
    private ICloudAdmin cloudAdmin;

    @OSGiResource
    private IClusterManager clusterManager;

    @EJB(mappedName = "UserManagerBean")
    private IUserManager userManager;

    @EJB(mappedName = "ProjectManagerBean")
    private IProjectManager projectManager;

    @Override
    public VolumeVO createVolume(final String projectId, final String username, final VolumeSpec volumeSpec)
        throws InvalidUsernameException, InvalidNameException, ResourceQuotaExceededException, DuplicateNameException,
        InvalidProjectIdException, PermissionDeniedException, CloudAdminException {

        User user = this.userManager.getUserByUsername(username);
        if (user == null) {
            throw new InvalidUsernameException("Unknown user " + username);
        }

        Project project = this.projectManager.getProjectByProjectId(projectId);
        if (project == null) {
            throw new InvalidProjectIdException("Unknown project " + projectId);
        }

        // check project authorization
        if (!new PermissionChecker().canCreateVolume(user, project)) {
            throw new PermissionDeniedException("Permission denied for user " + username + " in the projectId " + projectId);
        }

        String volName = volumeSpec.getName();

        // check Volume name validity
        if (!this.volumeNameIsValid(volName)) {
            throw new InvalidNameException("Invalid Volume name " + volName);
        }

        // check if Volume name already taken
        if (this.checkIfVolumeNameExistForProjectId(volName, projectId)) {
            throw new DuplicateNameException("Volume name " + volName + " already exists in this project !");
        }

        long capacityInMB = volumeSpec.getCapacityInMB();

        // check if capacity if correct
        if (capacityInMB <= 0 && capacityInMB > VolumeManagerBean.MAX_VOLUME_SIZE_IN_MB_ALLOWED) {
            throw new CloudAdminException("Invalid capacity");
        }

        if (!this.checkResourceQuota(project, capacityInMB)) {
            throw new ResourceQuotaExceededException("storage quota exceeded");
        }

        VolumeManagerBean.log.info("Creating Volume " + volName + " size=" + capacityInMB + ".");
        Volume volume = new Volume();
        volume.setCapacityInMB(capacityInMB);
        volume.setBootableVolume(false); // XXX
        volume.setDescription(volumeSpec.getDescription());
        volume.setName(volName);
        volume.setProject(project);
        volume.setStatus(Volume.Status.PENDING);
        volume.setUser(user);
        volume.setVisibility(Volume.Visibility.USER);
        this.em.persist(volume);
        this.em.flush();

        String volumeProviderId = null;
        try {
            volumeProviderId = this.clusterManager.createVolume(volumeSpec, volume.getId());
        } catch (VMMException ex) {
            throw new CloudAdminException(ex);
        }

        volume.setProviderId(volumeProviderId);
        volume.setStatus(Volume.Status.AVAILABLE);
        volume.setCreateDate(new Date());
        this.em.merge(volume);
        this.em.flush();

        VolumeVO volumeVO = VolumeVO.from(volume);
        this.cloudAdmin.emitTopicMessage(new NewVolumeEvent(volumeVO), "projectId", projectId);
        return volumeVO;
    }

    private boolean volumeNameIsValid(final String volName) {
        if (volName == null || volName.equals("")) {
            return false;
        }
        Pattern pattern = Pattern.compile("^[a-zA-Z0-9_.-]*$");
        Matcher matcher = pattern.matcher(volName);
        return matcher.find();
    }

    private boolean checkResourceQuota(final Project project, final long capacityInMB) {
        long diskUsedInMB = this.projectManager.countDiskByProject(project.getProjectId()) + capacityInMB;

        if (project.getResourceQuota().getDiskQuotaInMB() != -1 && diskUsedInMB > project.getResourceQuota().getDiskQuotaInMB()) {
            return false;
        }
        return true;
    }

    private boolean checkIfVolumeNameExistForProjectId(final String volName, final String projectId) {
        @SuppressWarnings("unchecked")
        List<Volume> list = this.em
            .createQuery("FROM Volume v WHERE v.name=:name AND v.project.projectId=:projectId AND v.status<>'DESTROYED'")
            .setParameter("name", volName).setParameter("projectId", projectId).getResultList();
        if (null == list || list.size() == 0) {
            return false; // don't exist
        }
        return true; // already exists
    }

    public Volume getVolumeByName(final String name) {
        @SuppressWarnings("unchecked")
        List<Volume> list = this.em.createQuery("FROM Volume v WHERE v.name=:name").setParameter("name", name).getResultList();
        if (list.size() > 0) {
            return list.get(0);
        }
        return null;
    }

    public Volume getVolumeById(final int id) {
        return this.em.find(Volume.class, id);
    }

    @Override
    public void destroyVolume(final String username, final String volumeId) throws InvalidUsernameException,
        InvalidProjectIdException, InvalidVolumeIdException, VolumeInUseException, PermissionDeniedException,
        CloudAdminException {
        User user = this.userManager.getUserByUsername(username);
        if (user == null) {
            throw new InvalidUsernameException("Unknown user " + username);
        }

        @SuppressWarnings("unchecked")
        List<Volume> list = this.em.createQuery("FROM Volume v WHERE v.id=:volumeId")
            .setParameter("volumeId", Integer.parseInt(volumeId)).getResultList();

        Volume volume = null;
        if (list.size() > 0) {
            volume = list.get(0);
        } else {
            throw new InvalidVolumeIdException("volumeId " + volumeId + " unknown");
        }

        if (volume.getVirtualMachines().size() > 0) {
            throw new VolumeInUseException("volume " + volume.getName() + " is in use");
        }

        // check project authorization
        if (!new PermissionChecker().canDestroyVolume(user, volume)) {
            throw new PermissionDeniedException("Permission denied for user " + username);
        }

        try {
            this.clusterManager.destroyVolume(volume.getProviderId());
        } catch (VMMException e) {
            throw new CloudAdminException(e.getMessage());
        } finally {
            volume.setStatus(Volume.Status.DESTROYED);
            volume.setDeleteDate(new Date());
            this.em.persist(volume);
            this.em.flush();
            this.cloudAdmin.emitTopicMessage(new DelVolumeEvent(volume.getId()), "projectId", volume.getProject()
                .getProjectId());
        }

    }

    @Override
    public void attachVolumeToVirtualMachine(final String projectId, final String username, final String volumeId,
        final String vmId) throws InvalidUsernameException, InvalidProjectIdException, InvalidVolumeIdException,
        VolumeInUseException, InvalidVirtualMachineIdException, PermissionDeniedException, CloudAdminException {
        User user = this.userManager.getUserByUsername(username);
        if (user == null) {
            throw new InvalidUsernameException("Unknown user " + username);
        }

        Project project = this.projectManager.getProjectByProjectId(projectId);
        if (project == null) {
            throw new InvalidProjectIdException("Unknown project " + projectId);
        }

        VirtualMachine vm = this.cloudAdmin.getVirtualMachineById(Integer.parseInt(vmId));
        if (vm == null) {
            throw new InvalidVirtualMachineIdException("vmId " + vmId + " invalid");
        }

        Volume volume = this.getVolumeById(Integer.parseInt(volumeId));
        if (volume == null) {
            throw new InvalidVolumeIdException("Volume id " + volumeId + " invalid");
        }
        // check project authorization
        if (!new PermissionChecker().canCreateVM(user, project)) {
            throw new PermissionDeniedException("Permission denied for user " + username + " in the projectId " + projectId);
        }

        if (volume.getVirtualMachines().size() > 0) {
            throw new VolumeInUseException("volume is already attached");
        }

        try {
            this.clusterManager.attachVolumeToVM(vm.getProviderId(), volume.getProviderId());
        } catch (VMMException ex) {
            throw new CloudAdminException(ex.getMessage());
        }

        vm.getVolumes().add(volume);
        this.em.persist(vm);
        this.em.flush();

        volume.getVirtualMachines().add(vm);

        this.cloudAdmin.emitTopicMessage(new VolumeUpdateEvent(VolumeVO.from(volume)), "projectId", project.getProjectId());
    }

    @Override
    public void detachVolumeFromVirtualMachine(final String projectId, final String username, final String volumeId,
        final String vmId) throws InvalidUsernameException, InvalidProjectIdException, InvalidVolumeIdException,
        InvalidVirtualMachineIdException, PermissionDeniedException, CloudAdminException {
        User user = this.userManager.getUserByUsername(username);
        if (user == null) {
            throw new InvalidUsernameException("Unknown user " + username);
        }

        Project project = this.projectManager.getProjectByProjectId(projectId);
        if (project == null) {
            throw new InvalidProjectIdException("Unknown project " + projectId);
        }

        VirtualMachine vm = this.cloudAdmin.getVirtualMachineById(Integer.parseInt(vmId));
        if (vm == null) {
            throw new InvalidVirtualMachineIdException("vmId " + vmId + " invalid");
        }

        Volume volume = this.getVolumeById(Integer.parseInt(volumeId));
        if (volume == null) {
            throw new InvalidVolumeIdException("Volume id " + volumeId + " invalid");
        }

        boolean volumeIsAttached = false;
        for (VirtualMachine attachedVM : volume.getVirtualMachines()) {
            if (vm.getId() == attachedVM.getId()) {
                volumeIsAttached = true;
                break;
            }
        }
        if (!volumeIsAttached) {
            throw new InvalidVirtualMachineIdException("Volume " + volumeId + " is not attached to VM " + vmId);
        }

        // check project authorization
        if (!new PermissionChecker().canCreateVM(user, project)) {
            throw new PermissionDeniedException("Permission denied for user " + username + " in the projectId " + projectId);
        }

        try {
            this.clusterManager.detachVolumeFromVM(vm.getProviderId(), volume.getProviderId());
        } catch (VMMException ex) {
            throw new CloudAdminException(ex.getMessage());
        }

        vm.getVolumes().remove(volume);
        this.em.persist(vm);
        this.em.flush();

        volume.getVirtualMachines().remove(vm);
        this.cloudAdmin.emitTopicMessage(new VolumeUpdateEvent(VolumeVO.from(volume)), "projectId", project.getProjectId());
    }

    @Override
    public List<VirtualMachineVO> getVirtualMachineUsingVolume(final String volumeId) throws InvalidVolumeIdException {
        Volume volume = this.getVolumeById(Integer.parseInt(volumeId));
        if (volume == null) {
            throw new InvalidVolumeIdException("Volume id " + volumeId + " invalid");
        }
        List<VirtualMachineVO> result = new ArrayList<VirtualMachineVO>();
        for (VirtualMachine vm : volume.getVirtualMachines()) {
            result.add(VirtualMachineVO.from(vm));
        }
        return result;
    }

}