/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: IResourcesReservation.java 87 2011-06-30 15:07:34Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service.reservation.api;

import java.util.Calendar;
import java.util.List;

import org.ow2.sirocco.cloudmanager.admin.exception.DuplicateNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InsufficientResourceException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidPowerStateException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidUsernameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVMImageException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVMSizeException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVirtualMachineIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.NoRelevantResourcesPoolException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.exception.ResourceQuotaExceededException;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineSpec;
import org.ow2.sirocco.cloudmanager.service.reservation.impl.ResourcesReservationVO;

/**
 * Resources reservation module API.
 */
public interface IResourcesReservation {

    /**
     * @param projectId
     * @param startTime
     * @param endTime
     * @param numberOfCPUCore
     * @param memorySizeInMB
     * @param storageSizeInMB
     * @param hypervisorType
     * @param location
     * @return The id of the reservation if the reservation can be reserved or
     *         informative exceptions if the reservation can not be reserved.
     * @throws InvalidArgumentException
     * @throws InsufficientResourceException
     * @throws NoRelevantResourcesPoolException
     */
    public Integer reserveResources(final String projectId, final Calendar startTime, final Calendar endTime,
        final Integer numberOfCPUCore, final Integer memorySizeInMB, final Integer storageSizeInMB,
        final String hypervisorType, final String location) throws InvalidArgumentException, InsufficientResourceException,
        NoRelevantResourcesPoolException;

    /**
     * @return the list containing all the resources reservations known by the
     *         reservation module (apart the DELETED ones), or informative
     *         exceptions if this list can not be retrieved.
     * @throws InvalidArgumentException
     */
    public List<ResourcesReservationVO> listResourcesReservations() throws InvalidArgumentException,
        NoRelevantResourcesPoolException;

    /**
     * It is possible to created SEVERAL VM in the same set of reserved
     * resources. This method is called by CloudAdminBean.
     * 
     * @param reservationId
     * @param vmSpec
     * @return Boolean.TRUE if the VM creation has been launched while using the
     *         reserved resources, or Boolean.FALSE if it is not possible to
     *         create the specified VM on the given reservation, or informative
     *         exceptions if the launching of the VM creation failed (e.g. wrong
     *         parameters).
     * @throws InvalidUsernameException
     * @throws InvalidNameException
     * @throws InvalidVMImageException
     * @throws ResourceQuotaExceededException
     * @throws DuplicateNameException
     * @throws InvalidVMSizeException
     * @throws InvalidProjectNameException
     * @throws InvalidArgumentException
     * @throws InsufficientResourceException
     * @throws ReservationException
     */
    public Boolean createAVMUsingReservedResources(final Integer reservationId, final VirtualMachineSpec vmSpec)
        throws InvalidUsernameException, InvalidNameException, InvalidVMImageException, ResourceQuotaExceededException,
        DuplicateNameException, InvalidVMSizeException, InvalidProjectNameException, InvalidArgumentException,
        InsufficientResourceException, NoRelevantResourcesPoolException, ReservationException;

    /**
     * This method is called by CloudAdminBean.
     * 
     * @param vmId
     * @return Boolean.TRUE if the VM uses reserved resources, if it is possible
     *         to destroy the given VM, and if the destruction has been
     *         launched. Boolean.FALSE otherwise (e.g., if the given VM does not
     *         use reserved resources).
     * @throws InvalidVirtualMachineIdException
     * @throws InvalidPowerStateException
     * @throws PermissionDeniedException
     * @throws InvalidArgumentException
     * @throws ReservationException
     */
    public Boolean destroyAVMThatUsesReservedResources(final String vmId) throws InvalidVirtualMachineIdException,
        InvalidPowerStateException, PermissionDeniedException, InvalidArgumentException, NoRelevantResourcesPoolException,
        ReservationException;

    /**
     * Cancels a resources reservation (destroys the VM created on reserved
     * resources, and moves the reservation's state to
     * ResourcesReservationState.DELETED).
     * 
     * @param fromUsername : see CloudAdminBean.destroyVirtualMachine.
     * @param reservationId
     * @return Nothing, or informative exceptions if the cancellation failed
     *         (e.g. wrong parameters).
     * @throws InvalidArgumentException
     * @throws InvalidVirtualMachineIdException
     * @throws InvalidPowerStateException
     * @throws PermissionDeniedException
     * @throws ReservationException
     */
    public void cancelAResourcesReservation(final String fromUsername, final Integer reservationId)
        throws InvalidArgumentException, InvalidVirtualMachineIdException, InvalidPowerStateException,
        PermissionDeniedException, NoRelevantResourcesPoolException, ReservationException;

    /**
     * @param projectId
     * @param startTime
     * @param endTime
     * @param numberOfCPUCore
     * @param memorySizeInMB
     * @param storageSizeInMB
     * @param hypervisorType
     * @param location
     * @return how many time a reservation can be reserved, or informative
     *         exceptions.
     * @throws InvalidArgumentException
     * @throws NoRelevantResourcesPoolException
     */
    public Integer describeResourcesReservationAvailability(final String projectId, final Calendar startTime,
        final Calendar endTime, final Integer numberOfCPUCore, final Integer memorySizeInMB, final Integer storageSizeInMB,
        final String hypervisorType, final String location) throws InvalidArgumentException, NoRelevantResourcesPoolException;

    /**
     * Enables to configure the booking period.
     * 
     * @param bookingPeriodExpressedInTimeUnit
     * @return Nothing, or informative exceptions if the instantiation failed
     *         (e.g. wrong parameters).
     * @throws InvalidArgumentException
     * @throws NoRelevantResourcesPoolException
     */
    public void configureBookingPeriod(final Integer bookingPeriodExpressedInTimeUnit) throws InvalidArgumentException,
        NoRelevantResourcesPoolException;

    /**
     * @return the current TimeUnitGranularityType.
     * @throws InvalidArgumentException
     * @throws NoRelevantResourcesPoolException
     */
    public TimeUnitGranularityType getTimeUnitGranularityType() throws InvalidArgumentException,
        NoRelevantResourcesPoolException;

    /**
     * Needed for test purpose.
     * 
     * @return the current booking period (expressed using the defined time
     *         unit).
     * @throws InvalidArgumentException
     * @throws NoRelevantResourcesPoolException
     */
    public Integer getBookingPeriodExpressedInTimeUnit() throws InvalidArgumentException, NoRelevantResourcesPoolException;

    /**
     * Needed for test purpose.
     * 
     * @param reservationId
     * @return Boolean.TRUE the reservation has been deleted/erased from the
     *         database, BOOLEAN.FALSE otherwise.
     * @throws InvalidArgumentException
     * @throws NoRelevantResourcesPoolException
     */
    public Boolean deleteAReservationFromDataBase(final Integer reservationId) throws InvalidArgumentException,
        NoRelevantResourcesPoolException;

}
