/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: DestroyVMOfExpiredReservationTimerSessionBean.java 208 2011-07-27 10:17:19Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service.reservation.impl;

import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.logging.Logger;

import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Stateless;
import javax.ejb.Timeout;
import javax.ejb.Timer;
import javax.ejb.TimerService;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.Query;

import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidPowerStateException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVirtualMachineIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.NoRelevantResourcesPoolException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.service.IUserManager;
import org.ow2.sirocco.cloudmanager.service.reservation.api.IDestroyVMOfExpiredReservationTimer;
import org.ow2.sirocco.cloudmanager.service.reservation.api.IResourcesReservation;
import org.ow2.sirocco.cloudmanager.service.reservation.api.ReservationException;
import org.ow2.sirocco.cloudmanager.service.reservation.api.ResourcesReservationState;
import org.ow2.sirocco.cloudmanager.service.reservation.api.TimeUnitGranularityType;

@Stateless(name = "DestroyVMOfExpiredReservationTimerSessionBean", mappedName = "DestroyVMOfExpiredReservationTimerSessionBean")
@Local(IDestroyVMOfExpiredReservationTimer.class)
public class DestroyVMOfExpiredReservationTimerSessionBean implements IDestroyVMOfExpiredReservationTimer {

    @Resource
    private TimerService timerService;

    @PersistenceContext(type = PersistenceContextType.TRANSACTION)
    protected EntityManager em;

    @EJB(mappedName = "ResourcesReservationBean")
    private IResourcesReservation resourcesReservationManagerBean;

    @EJB(mappedName = "UserManagerBean")
    private IUserManager userManager;

    private Logger log = Logger.getLogger(this.getClass().getName());

    public void startTimer() {
        // Retrieving granularity.
        TimeUnitGranularityType timeUnitGranularityType;
        try {
            timeUnitGranularityType = this.resourcesReservationManagerBean.getTimeUnitGranularityType();
            long timeBetweenTwoHandleTimeouts;
            long timeBeforeTheFirstExecutionExpressedInMilliseconds;
            Calendar now = new GregorianCalendar();
            if (TimeUnitGranularityType.HOUR.equals(timeUnitGranularityType)) {
                timeBetweenTwoHandleTimeouts = 60 * 60 * 1000;
                int currentMinute = now.get(Calendar.MINUTE);
                // Trigger is configured to be launched between H + 1 minute,
                // and H + 2 minutes.
                timeBeforeTheFirstExecutionExpressedInMilliseconds = (60 - currentMinute) * 60 * 1000 + 60 * 1000;
            } else if (TimeUnitGranularityType.DAY.equals(timeUnitGranularityType)) {
                timeBetweenTwoHandleTimeouts = 24 * 60 * 60 * 1000;
                // Trigger is configured to be launched between 2 and 3 A.M.
                if (0 == now.get(Calendar.HOUR_OF_DAY)) {
                    // First execution in the next 2 hours.
                    timeBeforeTheFirstExecutionExpressedInMilliseconds = 2 * 60 * 60 * 1000;
                } else if (1 == now.get(Calendar.HOUR_OF_DAY)) {
                    // First execution in the next hour.
                    timeBeforeTheFirstExecutionExpressedInMilliseconds = 60 * 60 * 1000;
                } else if (2 == now.get(Calendar.HOUR_OF_DAY)) {
                    // Add 5 minutes, this way Jonas has some time to start
                    // itself.
                    timeBeforeTheFirstExecutionExpressedInMilliseconds = 5 * 60 * 1000;
                } else {
                    int i = 0;
                    while ((now.get(Calendar.HOUR_OF_DAY) - i) != 2) {
                        i = i + 1;
                    }
                    timeBeforeTheFirstExecutionExpressedInMilliseconds = (24 - i) * 60 * 60 * 1000;
                }
            } else if (TimeUnitGranularityType.WEEK.equals(timeUnitGranularityType)) {
                // Ideally, it would be: timeBetweenTwoHandleTimeouts = 7 *
                // 24 * 60 * 60 * 1000;
                // But, we define the same mechanism than the one defined for
                // TimeUnitGranularityType.DAY.
                timeBetweenTwoHandleTimeouts = 7 * 24 * 60 * 60 * 1000;
                // Trigger is configured to be launched between 2 and 3 A.M. The
                // first day of the week.
                if (0 == now.get(Calendar.HOUR_OF_DAY)) {
                    // First execution in the next 2 hours.
                    timeBeforeTheFirstExecutionExpressedInMilliseconds = 2 * 60 * 60 * 1000;
                } else if (1 == now.get(Calendar.HOUR_OF_DAY)) {
                    // First execution in the next hour.
                    timeBeforeTheFirstExecutionExpressedInMilliseconds = 60 * 60 * 1000;
                } else if (2 == now.get(Calendar.HOUR_OF_DAY)) {
                    // Add 5 minutes, this way Jonas has some time to start
                    // itself.
                    timeBeforeTheFirstExecutionExpressedInMilliseconds = 5 * 60 * 1000;
                } else {
                    int i = 0;
                    while ((now.get(Calendar.HOUR_OF_DAY) - i) != 2) {
                        i = i + 1;
                    }
                    timeBeforeTheFirstExecutionExpressedInMilliseconds = (24 - i) * 60 * 60 * 1000;
                }
            } else {
                String msg = "This TimeUnitGranularityType ( " + timeUnitGranularityType + " ) is unexpected.\n";
                this.log.severe(msg);
                throw new InvalidArgumentException(msg);
            }

            // In milliseconds.
            // Arguments are:
            // - sleeping time before the first timer execution,
            // - sleeping amongst two executions,
            // - null.

            this.timerService.createTimer(timeBeforeTheFirstExecutionExpressedInMilliseconds, timeBetweenTwoHandleTimeouts,
                null);

            this.log.finest(this.getClass().getName() + " has been set.");
        } catch (InvalidArgumentException e) {
            // e.printStackTrace();
            this.log.severe("Error while retrieving TimeUnitGranularityType from the reservation module.");
        } catch (NoRelevantResourcesPoolException e) {
            // e.printStackTrace();
            this.log.severe("Error while retrieving TimeUnitGranularityType from the reservation module.");
        }
    }

    @Timeout
    public void handleTimeout(final Timer timer) {
        // Retrieving of the current time.
        long currentDate = new Date().getTime();

        this.log.finest(this.getClass().getName() + " is checking the persisted reservations looking for expired one(s).");

        // Retrieving of the current granularity.
        TimeUnitGranularityType timeUnitGranularityType;
        try {
            timeUnitGranularityType = this.resourcesReservationManagerBean.getTimeUnitGranularityType();

            // Reservations retrieving.
            // query database
            Query persistedResourcesReservation_list_query = this.em.createQuery("FROM "
                + ResourcesReservation.class.getSimpleName() + " p");

            @SuppressWarnings("unchecked")
            List<ResourcesReservation> persistedResourcesReservation_list = persistedResourcesReservation_list_query
                .getResultList();
            if (persistedResourcesReservation_list.size() > 0) {
                // Conversion of currentDate according to
                // timeUnitGranularityType
                long currentDateExpressedInTimeUnitGranularityType = -1;
                if (TimeUnitGranularityType.HOUR.equals(timeUnitGranularityType)) {
                    currentDateExpressedInTimeUnitGranularityType = currentDate / (60 * 60 * 1000);
                } else if (TimeUnitGranularityType.DAY.equals(timeUnitGranularityType)) {
                    currentDateExpressedInTimeUnitGranularityType = currentDate / (24 * 60 * 60 * 1000);
                } else if (TimeUnitGranularityType.WEEK.equals(timeUnitGranularityType)) {
                    currentDateExpressedInTimeUnitGranularityType = currentDate / (7 * 24 * 60 * 60 * 1000);
                } else {
                    String msg = "This TimeUnitGranularityType ( " + timeUnitGranularityType + " ) is unexpected.\n";
                    this.log.severe(msg);
                    throw new InvalidArgumentException(msg);
                }

                for (ResourcesReservation rREB_temp : persistedResourcesReservation_list) {

                    if (ResourcesReservationState.DELETED.equals(rREB_temp.getResourcesReservationState())) {
                        // nothing to do, the reservation is either out of date,
                        // or cancelled.
                    } else {
                        long reservationStartTime_temp = rREB_temp.getStartTime().getTimeInMillis();
                        long reservationStartTimeExpressedInTimeUnitGranularityType = -1;
                        long reservationEndTime_temp = rREB_temp.getEndTime().getTimeInMillis();
                        long reservationEndTimeExpressedInTimeUnitGranularityType = -1;
                        if (TimeUnitGranularityType.HOUR.equals(timeUnitGranularityType)) {
                            reservationStartTimeExpressedInTimeUnitGranularityType = reservationStartTime_temp
                                / (60 * 60 * 1000);
                            reservationEndTimeExpressedInTimeUnitGranularityType = reservationEndTime_temp / (60 * 60 * 1000);
                        } else if (TimeUnitGranularityType.DAY.equals(timeUnitGranularityType)) {
                            reservationStartTimeExpressedInTimeUnitGranularityType = reservationStartTime_temp
                                / (24 * 60 * 60 * 1000);
                            reservationEndTimeExpressedInTimeUnitGranularityType = reservationEndTime_temp
                                / (24 * 60 * 60 * 1000);
                        } else if (TimeUnitGranularityType.WEEK.equals(timeUnitGranularityType)) {
                            reservationStartTimeExpressedInTimeUnitGranularityType = reservationStartTime_temp
                                / (7 * 24 * 60 * 60 * 1000);
                            reservationEndTimeExpressedInTimeUnitGranularityType = reservationEndTime_temp
                                / (7 * 24 * 60 * 60 * 1000);
                        } else {
                            String msg = "This TimeUnitGranularityType ( " + timeUnitGranularityType + " ) is unexpected.\n";
                            this.log.severe(msg);
                            throw new InvalidArgumentException(msg);
                        }

                        int reservationId_temp = rREB_temp.getId();

                        if (currentDateExpressedInTimeUnitGranularityType < reservationStartTimeExpressedInTimeUnitGranularityType) {
                            // Change the reservation's state
                            // (normally, it is already in the
                            // ResourcesReservationState.PENDING state).
                            rREB_temp.setResourcesReservationState(ResourcesReservationState.PENDING);
                        } else if ((currentDateExpressedInTimeUnitGranularityType >= reservationStartTimeExpressedInTimeUnitGranularityType)
                            && (currentDateExpressedInTimeUnitGranularityType <= reservationEndTimeExpressedInTimeUnitGranularityType)) {
                            // Change the reservation's state from
                            // ResourcesReservationState.PENDING to
                            // ResourcesReservationState.ACTIVE.
                            // (normally, it is already in the
                            // ResourcesReservationState.ACTIVE state).
                            rREB_temp.setResourcesReservationState(ResourcesReservationState.ACTIVE);
                        } else if (currentDateExpressedInTimeUnitGranularityType > reservationEndTimeExpressedInTimeUnitGranularityType) {
                            // The change of the reservation's state from
                            // ResourcesReservationState.ACTIVE to
                            // ResourcesReservationState.DELETE is done by the
                            // cancel method (this latter also deletes the VM
                            // that were created on out-of-date reserved
                            // resources).

                            // XXX Fix hardcoded fromusername : "adm"
                            try {
                                this.resourcesReservationManagerBean.cancelAResourcesReservation(this.userManager
                                    .getAllAdmins().get(0).getUsername(), reservationId_temp);
                            } catch (InvalidVirtualMachineIdException e) {
                                // e.printStackTrace();
                                this.log.severe("Error while removing/cancelling an expired reservation.");
                            } catch (InvalidPowerStateException e) {
                                // e.printStackTrace();
                                this.log.severe("Error while removing/cancelling an expired reservation.");
                            } catch (PermissionDeniedException e) {
                                // e.printStackTrace();
                                this.log.severe("Error while removing/cancelling an expired reservation.");
                            } catch (ReservationException e) {
                                // e.printStackTrace();
                                this.log.severe("Error while removing/cancelling an expired reservation.");
                            } catch (NoRelevantResourcesPoolException e) {
                                // e.printStackTrace();
                                this.log.severe("Error while removing/cancelling an expired reservation.");
                            }
                        } else {
                            // this case should never happen.
                            String msg = "There is a pb. with the reservation (" + reservationId_temp
                                + "): currentDateExpressedInTimeUnitGranularityType ("
                                + currentDateExpressedInTimeUnitGranularityType
                                + "), reservationStartTimeExpressedInTimeUnitGranularityType ("
                                + reservationStartTimeExpressedInTimeUnitGranularityType
                                + "), and reservationEndTimeExpressedInTimeUnitGranularityType("
                                + reservationEndTimeExpressedInTimeUnitGranularityType + ") are incoherent/inconsistent.\n";
                            throw new ReservationException(msg);
                        }
                    }
                }
            } else {
                // nothing to do. There is no reservation.
            }
        } catch (InvalidArgumentException e) {
            // e.printStackTrace();
            this.log.severe("Error while retrieving TimeUnitGranularityType from the reservation module.");
        } catch (ReservationException e) {
            // e.printStackTrace();
            this.log.severe("Error ReservationException(" + e.getMessage() + ").");
        } catch (NoRelevantResourcesPoolException e) {
            // e.printStackTrace();
            this.log.severe("Error NoRelevantResourcesPoolException(" + e.getMessage() + ").");
        }
    }

    public void checkStatus() {
        @SuppressWarnings("unchecked")
        Collection<Timer> timers = this.timerService.getTimers();
        for (Timer timer : timers) {
            this.log.finest("Timer will expire after " + timer.getTimeRemaining() + " milliseconds.\n");
        }
    }

    public void stopTimers() {
        @SuppressWarnings("unchecked")
        Collection<Timer> timers = this.timerService.getTimers();
        for (Timer timer : timers) {
            timer.cancel();
        }
        this.log.finest("DestroyVMOfExpiredReservationTimerSessionBean stopped");
    }
}
