/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: ResourcesReservationManagerBean.java 199 2011-07-26 13:12:48Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service.reservation.impl;

import java.util.Calendar;
import java.util.List;

import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;

import org.ow2.easybeans.osgi.annotation.OSGiResource;
import org.ow2.sirocco.cloudmanager.admin.exception.DuplicateNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InsufficientResourceException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidPowerStateException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidUsernameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVMImageException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVMSizeException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVirtualMachineIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.NoRelevantResourcesPoolException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.exception.ResourceQuotaExceededException;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineSpec;
import org.ow2.sirocco.cloudmanager.clustermanager.api.IClusterManager;
import org.ow2.sirocco.cloudmanager.common.DbParameters;
import org.ow2.sirocco.cloudmanager.service.ICloudAdmin;
import org.ow2.sirocco.cloudmanager.service.reservation.api.IRemoteResourcesReservation;
import org.ow2.sirocco.cloudmanager.service.reservation.api.IResourcesReservation;
import org.ow2.sirocco.cloudmanager.service.reservation.api.ReservationException;
import org.ow2.sirocco.cloudmanager.service.reservation.api.TimeUnitGranularityType;

@Stateless(name = "ResourcesReservationBean", mappedName = "ResourcesReservationBean")
@Remote(IRemoteResourcesReservation.class)
@Local(IResourcesReservation.class)
public class ResourcesReservationManagerBean implements IResourcesReservation {

    @OSGiResource
    private IClusterManager clusterManager;

    @EJB(mappedName = "CloudAdminBean")
    private ICloudAdmin cloudAdmin;

    @PersistenceContext(unitName = "persistence-unit/main", type = PersistenceContextType.TRANSACTION)
    private EntityManager em = null;

    final TimeUnitGranularityType temporalGranularityType = DbParameters.getInstance().TEMPORAL_GRANULARITY_TYPE;

    final int bookingPeriodExpressedInTimeUnit = DbParameters.getInstance().BOOKING_PERIOD_EXPRESSED_IN_TIME_UNIT;

    /*
     * (non-Javadoc)
     * @see org.ow2.sirocco.cloudmanager.service.reservation.api.
     * IResourcesReservation#getTimeUnitGranularityType()
     */
    public final TimeUnitGranularityType getTimeUnitGranularityType() throws InvalidArgumentException,
        NoRelevantResourcesPoolException {
        TimeUnitGranularityType result = null;
        IResourcesReservation resourcesReservationModuleImpl = ResourcesReservationModuleImpl.getInstance(
            this.temporalGranularityType, this.bookingPeriodExpressedInTimeUnit, this.clusterManager, this.cloudAdmin, this.em);
        result = resourcesReservationModuleImpl.getTimeUnitGranularityType();
        return result;
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.sirocco.cloudmanager.service.reservation.api.
     * IResourcesReservation#reserveResources(java.lang.String,
     * java.util.Calendar, java.util.Calendar, java.lang.Integer,
     * java.lang.Integer, java.lang.Integer, java.lang.String, java.lang.String)
     */
    public Integer reserveResources(final String projectId, final Calendar startTime, final Calendar endTime,
        final Integer numberOfCPUCore, final Integer memorySizeInMB, final Integer storageSizeInMB,
        final String hypervisorType, final String location) throws InvalidArgumentException, InsufficientResourceException,
        NoRelevantResourcesPoolException {
        Integer result = null;
        IResourcesReservation resourcesReservationModuleImpl = ResourcesReservationModuleImpl.getInstance(
            this.temporalGranularityType, this.bookingPeriodExpressedInTimeUnit, this.clusterManager, this.cloudAdmin, this.em);
        result = resourcesReservationModuleImpl.reserveResources(projectId, startTime, endTime, numberOfCPUCore,
            memorySizeInMB, storageSizeInMB, hypervisorType, location);
        return result;
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.sirocco.cloudmanager.service.reservation.api.
     * IResourcesReservation#listResourcesReservations()
     */
    public List<ResourcesReservationVO> listResourcesReservations() throws InvalidArgumentException,
        NoRelevantResourcesPoolException {
        List<ResourcesReservationVO> result = null;
        IResourcesReservation resourcesReservationModuleImpl = ResourcesReservationModuleImpl.getInstance(
            this.temporalGranularityType, this.bookingPeriodExpressedInTimeUnit, this.clusterManager, this.cloudAdmin, this.em);
        result = resourcesReservationModuleImpl.listResourcesReservations();
        return result;
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.sirocco.cloudmanager.service.reservation.api.
     * IResourcesReservation#createAVMUsingReservedResources(int,
     * java.lang.String,
     * org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineSpec)
     */
    public Boolean createAVMUsingReservedResources(final Integer reservationId, final VirtualMachineSpec vmSpec)
        throws InvalidUsernameException, InvalidNameException, InvalidVMImageException, ResourceQuotaExceededException,
        DuplicateNameException, InvalidVMSizeException, InvalidProjectNameException, InvalidArgumentException,
        InsufficientResourceException, NoRelevantResourcesPoolException, ReservationException {
        Boolean result = null;
        IResourcesReservation resourcesReservationModuleImpl = ResourcesReservationModuleImpl.getInstance(
            this.temporalGranularityType, this.bookingPeriodExpressedInTimeUnit, this.clusterManager, this.cloudAdmin, this.em);
        result = resourcesReservationModuleImpl.createAVMUsingReservedResources(reservationId, vmSpec);
        return result;
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.sirocco.cloudmanager.service.reservation.api.
     * IResourcesReservation
     * #destroyAVMThatUsesReservedResources(java.lang.String, java.lang.String)
     */
    public Boolean destroyAVMThatUsesReservedResources(final String vmId) throws InvalidVirtualMachineIdException,
        InvalidPowerStateException, PermissionDeniedException, InvalidArgumentException, NoRelevantResourcesPoolException,
        ReservationException {

        Boolean result = null;
        IResourcesReservation resourcesReservationModuleImpl = ResourcesReservationModuleImpl.getInstance(
            this.temporalGranularityType, this.bookingPeriodExpressedInTimeUnit, this.clusterManager, this.cloudAdmin, this.em);
        result = resourcesReservationModuleImpl.destroyAVMThatUsesReservedResources(vmId);
        return result;
    }

    public void cancelAResourcesReservation(final String fromUsername, final Integer reservationId)
        throws InvalidVirtualMachineIdException, InvalidPowerStateException, PermissionDeniedException,
        InvalidArgumentException, NoRelevantResourcesPoolException, ReservationException {
        IResourcesReservation resourcesReservationModuleImpl = ResourcesReservationModuleImpl.getInstance(
            this.temporalGranularityType, this.bookingPeriodExpressedInTimeUnit, this.clusterManager, this.cloudAdmin, this.em);
        resourcesReservationModuleImpl.cancelAResourcesReservation(fromUsername, reservationId);
    }

    public Integer describeResourcesReservationAvailability(final String projectId, final Calendar startTime,
        final Calendar endTime, final Integer numberOfCPUCore, final Integer memorySizeInMB, final Integer storageSizeInMB,
        final String hypervisorType, final String location) throws InvalidArgumentException, NoRelevantResourcesPoolException {
        Integer result = null;
        IResourcesReservation resourcesReservationModuleImpl = ResourcesReservationModuleImpl.getInstance(
            this.temporalGranularityType, this.bookingPeriodExpressedInTimeUnit, this.clusterManager, this.cloudAdmin, this.em);
        result = resourcesReservationModuleImpl.describeResourcesReservationAvailability(projectId, startTime, endTime,
            numberOfCPUCore, memorySizeInMB, storageSizeInMB, hypervisorType, location);
        return result;
    }

    public void configureBookingPeriod(final Integer bookingPeriodExpressedInTimeUnit) throws InvalidArgumentException,
        NoRelevantResourcesPoolException {
        IResourcesReservation resourcesReservationModuleImpl = ResourcesReservationModuleImpl.getInstance(
            this.temporalGranularityType, bookingPeriodExpressedInTimeUnit, this.clusterManager, this.cloudAdmin, this.em);
        resourcesReservationModuleImpl.configureBookingPeriod(bookingPeriodExpressedInTimeUnit);
    }

    public final Integer getBookingPeriodExpressedInTimeUnit() throws InvalidArgumentException,
        NoRelevantResourcesPoolException {
        Integer result = null;
        IResourcesReservation resourcesReservationModuleImpl = ResourcesReservationModuleImpl.getInstance(
            this.temporalGranularityType, this.bookingPeriodExpressedInTimeUnit, this.clusterManager, this.cloudAdmin, this.em);
        result = resourcesReservationModuleImpl.getBookingPeriodExpressedInTimeUnit();
        return result;
    }

    @Override
    public Boolean deleteAReservationFromDataBase(final Integer reservationId) throws InvalidArgumentException,
        NoRelevantResourcesPoolException {
        Boolean result = null;
        IResourcesReservation resourcesReservationModuleImpl = ResourcesReservationModuleImpl.getInstance(
            this.temporalGranularityType, this.bookingPeriodExpressedInTimeUnit, this.clusterManager, this.cloudAdmin, this.em);
        result = resourcesReservationModuleImpl.deleteAReservationFromDataBase(reservationId);
        return result;
    }

}
