/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: ResourcesReservationModuleImpl.java 318 2011-09-09 12:20:59Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service.reservation.impl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.logging.Logger;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.ow2.sirocco.cloudmanager.admin.exception.DuplicateNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InsufficientResourceException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidPowerStateException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidUsernameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVMImageException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVMSizeException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVirtualMachineIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.NoRelevantResourcesPoolException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.exception.ResourceQuotaExceededException;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineSpec;
import org.ow2.sirocco.cloudmanager.business.Project;
import org.ow2.sirocco.cloudmanager.business.VMImage;
import org.ow2.sirocco.cloudmanager.business.VirtualMachine;
import org.ow2.sirocco.cloudmanager.clustermanager.api.AllocationMode;
import org.ow2.sirocco.cloudmanager.clustermanager.api.IClusterManager;
import org.ow2.sirocco.cloudmanager.clustermanager.api.ResourcePartitionInfo;
import org.ow2.sirocco.cloudmanager.service.ICloudAdmin;
import org.ow2.sirocco.cloudmanager.service.reservation.api.IResourcesReservation;
import org.ow2.sirocco.cloudmanager.service.reservation.api.ReservationException;
import org.ow2.sirocco.cloudmanager.service.reservation.api.ResourcesReservationState;
import org.ow2.sirocco.cloudmanager.service.reservation.api.TimeUnitGranularityType;

public class ResourcesReservationModuleImpl implements IResourcesReservation {

    private static Logger log = Logger.getLogger(ResourcesReservationModuleImpl.class.getName());

    private static ResourcesReservationModuleImpl instance = null;

    private TimeUnitGranularityType timeUnitGranularityType = null;

    private Integer bookingPeriodExpressedInTimeUnit = null;

    private IClusterManager clusterManager;

    private ICloudAdmin cloudAdmin;

    private List<ResourcesPoolOverallPlanning> resourcesPoolOverallPlanning_List;

    private EntityManager em = null;

    // Note: When a reservation becomes out-of-date, all its (running) VM are
    // destroyed.

    // protected VMReservationModuleImpl() {
    // // Exists only to defeat instantiation.
    // }

    public static ResourcesReservationModuleImpl getInstance(final TimeUnitGranularityType temporalGranularityType,
        final Integer bookingPeriodExpressedInTimeUnit, final IClusterManager clusterManager, final ICloudAdmin cloudAdmin,
        final EntityManager em) throws InvalidArgumentException, NoRelevantResourcesPoolException {
        ResourcesReservationModuleImpl.log.finest("Enters in: ResourcesReservationModuleImpl.getInstance("
            + temporalGranularityType + ", " + bookingPeriodExpressedInTimeUnit + ", " + clusterManager + ", " + cloudAdmin
            + ", " + em + ".");
        if ((temporalGranularityType == null) || (bookingPeriodExpressedInTimeUnit == null) || (clusterManager == null)
            || (cloudAdmin == null) || (em == null)) {
            throw new InvalidArgumentException("At least one of the given parameters is null.");
        } else {
            if (ResourcesReservationModuleImpl.instance == null) {
                ResourcesReservationModuleImpl.instance = new ResourcesReservationModuleImpl(temporalGranularityType,
                    bookingPeriodExpressedInTimeUnit, clusterManager, cloudAdmin, em);
            } else {
                // nothing to do.
            }
        }
        ResourcesReservationModuleImpl.log.finest("Leaves: ResourcesReservationModuleImpl.getInstance(...).");
        return ResourcesReservationModuleImpl.instance;
    }

    /**
     * @param timeUnitGranularityType
     * @param bookingPeriodExpressedInTimeUnit
     * @param clusterManager
     * @param cloudAdmin
     * @param em
     * @throws InvalidArgumentException
     * @throws NoRelevantResourcesPoolException
     */
    @SuppressWarnings("unchecked")
    protected ResourcesReservationModuleImpl(final TimeUnitGranularityType timeUnitGranularityType,
        final Integer bookingPeriodExpressedInTimeUnit, final IClusterManager clusterManager, final ICloudAdmin cloudAdmin,
        final EntityManager em) throws InvalidArgumentException, NoRelevantResourcesPoolException {
        ResourcesReservationModuleImpl.log.finest("Enters in: ResourcesReservationModuleImpl("
            + bookingPeriodExpressedInTimeUnit + ", " + clusterManager + ", " + cloudAdmin + ", " + em + ").");
        if (timeUnitGranularityType == null) {
            throw new InvalidArgumentException("The given timeUnitGranularityType is: " + timeUnitGranularityType + ".");
        } else if (bookingPeriodExpressedInTimeUnit == null) {
            throw new InvalidArgumentException("The given bookingPeriodExpressedInTimeUnit is: "
                + bookingPeriodExpressedInTimeUnit + ".");
        } else if (bookingPeriodExpressedInTimeUnit < 0) {
            throw new InvalidArgumentException("The given bookingPeriodExpressedInTimeUnit ("
                + bookingPeriodExpressedInTimeUnit + ") is < 0.");
        } else if (clusterManager == null) {
            throw new InvalidArgumentException("The given clusterManager is: " + clusterManager + ".");
        } else if (cloudAdmin == null) {
            throw new InvalidArgumentException("The given cloudAdmin is: " + cloudAdmin + ".");
        } else {
            // Retrieving clusterManager (i.e., the resourcePoolIdentifier)
            this.clusterManager = clusterManager;

            //

            this.resourcesPoolOverallPlanning_List = new ArrayList<ResourcesPoolOverallPlanning>();

            // Initiliaze (all) the plannings with regard to the resourcesPools
            // retrieved via the clusterManager.
            List<ResourcePartitionInfo> resourcePartitionInfo_list = this.clusterManager.getResourcePartitions();
            Iterator<ResourcePartitionInfo> rPI_temp_ITERATOR = resourcePartitionInfo_list.iterator();
            while (rPI_temp_ITERATOR.hasNext()) {
                ResourcePartitionInfo rPI_temp = rPI_temp_ITERATOR.next();
                if (AllocationMode.ADVANCED_RESERVATION.equals(rPI_temp.getAllocationMode())) {
                    ResourcesPoolOverallPlanning rPOP_temp = new ResourcesPoolOverallPlanning(rPI_temp.getMbean(),
                        rPI_temp.getHypervisor(), rPI_temp.getLocation());
                    this.resourcesPoolOverallPlanning_List.add(rPOP_temp);
                } else {
                    // nothing to do.
                }
            }

            this.cloudAdmin = cloudAdmin;

            this.timeUnitGranularityType = timeUnitGranularityType;
            this.bookingPeriodExpressedInTimeUnit = bookingPeriodExpressedInTimeUnit;

            this.em = em;

            // Initialize all the ResourcesPoolOverallPlannings with regard
            // to the persisted reservations.

            List<ResourcesReservation> persistedResourcesReservationsList = null;

            Query result_temp = null;
            result_temp = this.em.createQuery("FROM " + ResourcesReservation.class.getSimpleName() + " r");
            if (result_temp == null) {
                // There is no persisted reservation.
                persistedResourcesReservationsList = new ArrayList<ResourcesReservation>();
            } else {
                persistedResourcesReservationsList = result_temp.getResultList();
            }

            Iterator<ResourcesReservation> persistedResourcesReservation_ITERATOR = persistedResourcesReservationsList
                .iterator();
            while (persistedResourcesReservation_ITERATOR.hasNext()) {
                ResourcesReservation persistedResourcesReservation_temp = persistedResourcesReservation_ITERATOR.next();
                // convert startTime, and endTime w.r.t. chosen
                // timeUnitGranularityType.
                Calendar startTime = persistedResourcesReservation_temp.getStartTime();
                Calendar endTime = persistedResourcesReservation_temp.getEndTime();
                if (TimeUnitGranularityType.HOUR.equals(this.timeUnitGranularityType)) {
                    long startTimeInHour = startTime.getTimeInMillis() / (60 * 60 * 1000);
                    long endTimeInHour = endTime.getTimeInMillis() / (60 * 60 * 1000);
                    this.initiateOrUpdateResourcesReservationPlanningWithRegardToTheResourcesReservation(startTimeInHour,
                        endTimeInHour, persistedResourcesReservation_temp);
                } else if (TimeUnitGranularityType.DAY.equals(this.timeUnitGranularityType)) {
                    long startTimeInDay = startTime.getTimeInMillis() / (24 * 60 * 60 * 1000);
                    long endTimeInDay = endTime.getTimeInMillis() / (24 * 60 * 60 * 1000);
                    // Same process than Hour's process.
                    this.initiateOrUpdateResourcesReservationPlanningWithRegardToTheResourcesReservation(startTimeInDay,
                        endTimeInDay, persistedResourcesReservation_temp);
                } else if (TimeUnitGranularityType.WEEK.equals(this.timeUnitGranularityType)) {
                    long startTimeInWeek = startTime.getTimeInMillis() / (7 * 24 * 60 * 60 * 1000);
                    long endTimeInWeek = endTime.getTimeInMillis() / (7 * 24 * 60 * 60 * 1000);
                    // Same process than Hour's process.
                    this.initiateOrUpdateResourcesReservationPlanningWithRegardToTheResourcesReservation(startTimeInWeek,
                        endTimeInWeek, persistedResourcesReservation_temp);
                } else {
                    throw new InvalidArgumentException("The " + this.getClass().getName() + " module supports only: "
                        + TimeUnitGranularityType.HOUR + ", " + TimeUnitGranularityType.DAY + ", or "
                        + TimeUnitGranularityType.WEEK + " time unit granularity types.");
                }
            }
        }
    }

    @Override
    public final TimeUnitGranularityType getTimeUnitGranularityType() {
        ResourcesReservationModuleImpl.log.finest("Enters in: getTimeUnitGranularityType().");
        ResourcesReservationModuleImpl.log.finest("Leaves: getTimeUnitGranularityType().");
        return this.timeUnitGranularityType;
    }

    /**
     * Initiates the relevant resourcesReservationPlanning using the
     * ResourcesReservation.
     * 
     * @return Boolean.TRUE if everything runs well, Boolean.FALSE otherwise.
     * @throws NoRelevantResourcesPoolException
     */
    private Boolean initiateOrUpdateResourcesReservationPlanningWithRegardToTheResourcesReservation(
        final long startTimeExpressedUsingTheDefinedTimeUnitGranularityType,
        final long endTimeExpressedUsingTheDefinedTimeUnitGranularityType,
        final ResourcesReservation persistedResourcesReservation_temp) throws NoRelevantResourcesPoolException {

        ResourcesReservationModuleImpl.log
            .finest("Enters in: initiateOrUpdateResourcesReservationPlanningWithRegardToTheResourcesReservation("
                + startTimeExpressedUsingTheDefinedTimeUnitGranularityType + ", "
                + endTimeExpressedUsingTheDefinedTimeUnitGranularityType + ", " + persistedResourcesReservation_temp + ").");

        Boolean result;

        long timeExpressedUsingTheDefinedTimeUnitGranularityType_temp = startTimeExpressedUsingTheDefinedTimeUnitGranularityType;

        // this.displayResourcesReservationPlanning();

        if (!(ResourcesReservationState.DELETED.equals(persistedResourcesReservation_temp.getResourcesReservationState()))) {

            // Let's select the resourcesPool that cooresponds to the
            // persisted reservation hypervisor type, and location.
            String persistedResourcesReservation_temp_hypervisorType = persistedResourcesReservation_temp.getHypervisorType();
            String persistedResourcesReservation_temp_location = persistedResourcesReservation_temp.getLocation();

            Iterator<ResourcesPoolOverallPlanning> rPOP_temp_ITERATOR = this.resourcesPoolOverallPlanning_List.iterator();
            Boolean hasAResourcesPoolBeenFoundForPersistedResourcesReservation_temp = Boolean.FALSE;
            while (rPOP_temp_ITERATOR.hasNext()) {
                ResourcesPoolOverallPlanning rPOP_temp = rPOP_temp_ITERATOR.next();
                if ((persistedResourcesReservation_temp_hypervisorType.equalsIgnoreCase(rPOP_temp
                    .getResourcePoolHypervisorType()))
                    && (persistedResourcesReservation_temp_location.equalsIgnoreCase(rPOP_temp.getResourcePoolLocation()))) {
                    hasAResourcesPoolBeenFoundForPersistedResourcesReservation_temp = Boolean.TRUE;

                    while (timeExpressedUsingTheDefinedTimeUnitGranularityType_temp <= endTimeExpressedUsingTheDefinedTimeUnitGranularityType) {

                        ResourcesPoolPlanningByTimeUnit resourcesPoolPlanningByTimeUnit_temp = rPOP_temp
                            .getResourcesReservationPlanning().get(timeExpressedUsingTheDefinedTimeUnitGranularityType_temp);
                        if (resourcesPoolPlanningByTimeUnit_temp == null) {
                            // There is no ResourcesPoolPlanningByTimeUnit for
                            // this timeUnit. Creation of a new
                            // ResourcesPoolPlanningByTimeUnit.

                            LinkedHashSet<Integer> reservationIdSet = new LinkedHashSet<Integer>();
                            reservationIdSet.add(persistedResourcesReservation_temp.getId());

                            // Retrieving the resourcesQuotaMax of the projet
                            // with the id:
                            // persistedResourcesReservation_temp_.getProjectId()
                            Integer resourceQuotaCPUForProject = null;
                            Integer resourceQuotaMemoryInMBForProject = null;
                            Integer resourceQuotaStorageInMBForProject = null;

                            Project project_temp = null;
                            Query project_temp_list_query = this.em.createQuery("FROM Project p WHERE p.projectId=:projectId")
                                .setParameter("projectId", persistedResourcesReservation_temp.getProjectId());
                            if (project_temp_list_query == null) {
                                // throw new ReservationException(
                                // "The given reservation ("
                                // + persistedResourcesReservation_temp_
                                // .getProjectId()
                                // +
                                // ") is linked to a project that is unknown to Sirocco.");
                                result = Boolean.FALSE;
                                return result;
                            } else {
                                if (project_temp_list_query.getResultList().isEmpty()) {
                                    // throw new ReservationException(
                                    // "The given reservation ("
                                    // + persistedResourcesReservation_temp_
                                    // .getProjectId()
                                    // +
                                    // ") is linked to a project that is unknown to Sirocco.");
                                    result = Boolean.FALSE;
                                    return result;
                                } else {
                                    project_temp = (Project) project_temp_list_query.getSingleResult();
                                }
                            }

                            // -1 means unbound (was: 0 means unbound).
                            resourceQuotaCPUForProject = project_temp.getResourceQuota().getCpuQuota();
                            Integer availableCPUResourcesForThisProjectInThisTimeUnit = null;
                            if (resourceQuotaCPUForProject == -1) {
                                availableCPUResourcesForThisProjectInThisTimeUnit = resourceQuotaCPUForProject;
                            } else {
                                availableCPUResourcesForThisProjectInThisTimeUnit = resourceQuotaCPUForProject
                                    - persistedResourcesReservation_temp.getReservedNumberOfCPUCore();
                            }

                            // -1 means unbound (was: 0 means unbound).
                            resourceQuotaMemoryInMBForProject = project_temp.getResourceQuota().getRamQuotaInMB();
                            Integer availableMemoryResourcesForThisProjectInThisTimeUnit = null;
                            if (resourceQuotaMemoryInMBForProject == -1) {
                                availableMemoryResourcesForThisProjectInThisTimeUnit = resourceQuotaMemoryInMBForProject;
                            } else {
                                availableMemoryResourcesForThisProjectInThisTimeUnit = resourceQuotaMemoryInMBForProject
                                    - persistedResourcesReservation_temp.getReservedMemorySizeInMB();
                            }

                            // -1 means unbound (was: 0 means unbound).
                            resourceQuotaStorageInMBForProject = project_temp.getResourceQuota().getDiskQuotaInMB();
                            Integer availableStorageResourcesForThisProjectInThisTimeUnit = null;
                            if (resourceQuotaStorageInMBForProject == -1) {
                                availableStorageResourcesForThisProjectInThisTimeUnit = resourceQuotaStorageInMBForProject;
                            } else {
                                availableStorageResourcesForThisProjectInThisTimeUnit = resourceQuotaStorageInMBForProject
                                    - persistedResourcesReservation_temp.getReservedStorageSizeInMB();
                            }

                            ProjectPlanningByTimeUnit projectPlanningByTimeUnit_temp = new ProjectPlanningByTimeUnit(
                                availableCPUResourcesForThisProjectInThisTimeUnit,
                                availableMemoryResourcesForThisProjectInThisTimeUnit,
                                availableStorageResourcesForThisProjectInThisTimeUnit, reservationIdSet);

                            Hashtable<String, ProjectPlanningByTimeUnit> projectHavingAVMReservationInThisTimeUnit_temp = new Hashtable<String, ProjectPlanningByTimeUnit>();
                            projectHavingAVMReservationInThisTimeUnit_temp.put(
                                persistedResourcesReservation_temp.getProjectId(), projectPlanningByTimeUnit_temp);

                            // -1 means unbound.
                            Long availableCPUResourcesForThisResourcePoolInThisTimeUnit = null;
                            if (rPOP_temp.getResourcesPoolTotalCPUCapacity() == -1) {
                                availableCPUResourcesForThisResourcePoolInThisTimeUnit = rPOP_temp
                                    .getResourcesPoolTotalCPUCapacity();
                            } else {
                                availableCPUResourcesForThisResourcePoolInThisTimeUnit = rPOP_temp
                                    .getResourcesPoolTotalCPUCapacity()
                                    - persistedResourcesReservation_temp.getReservedNumberOfCPUCore();
                            }
                            Long availableMemoryResourcesForThisResourcePoolInThisTimeUnit = null;
                            if (rPOP_temp.getResourcesPoolTotalMemoryCapacity() == -1) {
                                availableMemoryResourcesForThisResourcePoolInThisTimeUnit = rPOP_temp
                                    .getResourcesPoolTotalMemoryCapacity();
                            } else {
                                availableMemoryResourcesForThisResourcePoolInThisTimeUnit = rPOP_temp
                                    .getResourcesPoolTotalMemoryCapacity()
                                    - persistedResourcesReservation_temp.getReservedMemorySizeInMB();
                            }
                            Long availableStorageResourcesForThisResourcePoolInThisTimeUnit = null;
                            if (rPOP_temp.getResourcesPoolTotalStorageCapacity() == -1) {
                                availableStorageResourcesForThisResourcePoolInThisTimeUnit = rPOP_temp
                                    .getResourcesPoolTotalStorageCapacity();
                            } else {
                                availableStorageResourcesForThisResourcePoolInThisTimeUnit = rPOP_temp
                                    .getResourcesPoolTotalStorageCapacity()
                                    - persistedResourcesReservation_temp.getReservedStorageSizeInMB();
                            }

                            resourcesPoolPlanningByTimeUnit_temp = new ResourcesPoolPlanningByTimeUnit(
                                availableCPUResourcesForThisResourcePoolInThisTimeUnit,
                                availableMemoryResourcesForThisResourcePoolInThisTimeUnit,
                                availableStorageResourcesForThisResourcePoolInThisTimeUnit,
                                projectHavingAVMReservationInThisTimeUnit_temp);

                            // time-slot in hour, ProjectPlanningByTimeUnit.
                            rPOP_temp.getResourcesReservationPlanning().put(
                                timeExpressedUsingTheDefinedTimeUnitGranularityType_temp, resourcesPoolPlanningByTimeUnit_temp);

                        } else {
                            // A ResourcesPoolPlanningByTimeUnit is already
                            // defined for this timeUnit. Let's update it.

                            // Retrieving reservation information via
                            // persistedResourcesReservation_temp.
                            // Updating of the resources available by
                            // substracting the resources taken by the
                            // reservation.
                            // -1 means unbound.
                            if (resourcesPoolPlanningByTimeUnit_temp
                                .getAvailableCPUResourcesForThisResourcePoolInThisTimeUnit() == -1) {
                                // nothing to do.
                            } else {
                                resourcesPoolPlanningByTimeUnit_temp
                                    .setAvailableCPUResourcesForThisResourcePoolInThisTimeUnit(resourcesPoolPlanningByTimeUnit_temp
                                        .getAvailableCPUResourcesForThisResourcePoolInThisTimeUnit()
                                        - persistedResourcesReservation_temp.getReservedNumberOfCPUCore());
                            }

                            if (resourcesPoolPlanningByTimeUnit_temp
                                .getAvailableMemoryResourcesForThisResourcePoolInThisTimeUnit() == -1) {
                                // nothing to do.
                            } else {
                                resourcesPoolPlanningByTimeUnit_temp
                                    .setAvailableMemoryResourcesForThisResourcePoolInThisTimeUnit(resourcesPoolPlanningByTimeUnit_temp
                                        .getAvailableMemoryResourcesForThisResourcePoolInThisTimeUnit()
                                        - persistedResourcesReservation_temp.getReservedMemorySizeInMB());
                            }

                            if (resourcesPoolPlanningByTimeUnit_temp
                                .getAvailableStorageResourcesForThisResourcePoolInThisTimeUnit() == -1) {
                                // nothing to do.
                            } else {
                                resourcesPoolPlanningByTimeUnit_temp
                                    .setAvailableStorageResourcesForThisResourcePoolInThisTimeUnit(resourcesPoolPlanningByTimeUnit_temp
                                        .getAvailableStorageResourcesForThisResourcePoolInThisTimeUnit()
                                        - persistedResourcesReservation_temp.getReservedStorageSizeInMB());
                            }

                            // Updating
                            // ProjectHavingAResourcesReservationInThisTimeUnit
                            // for this resourcesPoolPlanningByTimeUnit_temp.
                            Hashtable<String, ProjectPlanningByTimeUnit> projectHavingAResourcesReservationInThisTimeUnit_temp = resourcesPoolPlanningByTimeUnit_temp
                                .getProjectHavingAResourcesReservationInThisTimeUnit();

                            String projectId_temp = persistedResourcesReservation_temp.getProjectId();

                            ProjectPlanningByTimeUnit projectPlanningByTimeUnit_temp = projectHavingAResourcesReservationInThisTimeUnit_temp
                                .get(projectId_temp);

                            if (projectPlanningByTimeUnit_temp == null) {
                                // Creation of a new
                                // projectPlanningByTimeUnit_temp for this
                                // projectId_temp (it doesn't exist right now).

                                // Retrieveing the resourcesQuotaMax of the
                                // project identified by
                                // persistedResourcesReservation_temp_.getProjectId()
                                Integer resourceQuotaCPUForProject = null;
                                Integer resourceQuotaMemoryInMBForProject = null;
                                Integer resourceQuotaStorageInMBForProject = null;

                                // This is the first reservation for this
                                // project, and for this timeUnit.
                                // As a consequence, project's resourceQuota max
                                // is used.
                                Project project_temp = null;
                                Query project_temp_list_query = this.em.createQuery(
                                    "FROM Project p WHERE p.projectId=:projectId").setParameter("projectId", projectId_temp);
                                if (project_temp_list_query == null) {
                                    // throw new ReservationException(
                                    // "The given reservation (" +
                                    // projectId_temp +
                                    // ") is linked to a project that is unknown to Sirocco.");
                                    result = Boolean.FALSE;
                                    return result;
                                } else {
                                    project_temp = (Project) project_temp_list_query.getSingleResult();
                                }

                                // -1 means unbound (was: 0 means unbound).
                                resourceQuotaCPUForProject = project_temp.getResourceQuota().getCpuQuota();
                                Integer availableCPUResourcesForThisProjectInThisTimeUnit = null;
                                if (resourceQuotaCPUForProject == -1) {
                                    availableCPUResourcesForThisProjectInThisTimeUnit = resourceQuotaCPUForProject;
                                } else {
                                    availableCPUResourcesForThisProjectInThisTimeUnit = resourceQuotaCPUForProject
                                        - persistedResourcesReservation_temp.getReservedNumberOfCPUCore();
                                }

                                // -1 means unbound (was: 0 means unbound).
                                resourceQuotaMemoryInMBForProject = project_temp.getResourceQuota().getRamQuotaInMB();
                                Integer availableMemoryResourcesForThisProjectInThisTimeUnit = null;
                                if (resourceQuotaMemoryInMBForProject == -1) {
                                    availableMemoryResourcesForThisProjectInThisTimeUnit = resourceQuotaMemoryInMBForProject;
                                } else {
                                    availableMemoryResourcesForThisProjectInThisTimeUnit = resourceQuotaMemoryInMBForProject
                                        - persistedResourcesReservation_temp.getReservedMemorySizeInMB();
                                }

                                // -1 means unbound (was: 0 means unbound).
                                resourceQuotaStorageInMBForProject = project_temp.getResourceQuota().getDiskQuotaInMB();
                                Integer availableStorageResourcesForThisProjectInThisTimeUnit = null;
                                if (resourceQuotaStorageInMBForProject == -1) {
                                    availableStorageResourcesForThisProjectInThisTimeUnit = resourceQuotaStorageInMBForProject;
                                } else {
                                    availableStorageResourcesForThisProjectInThisTimeUnit = resourceQuotaStorageInMBForProject
                                        - persistedResourcesReservation_temp.getReservedStorageSizeInMB();
                                }

                                // This is the first reservation for this
                                // project, and for this timeUnit.
                                LinkedHashSet<Integer> reservationIdSet = new LinkedHashSet<Integer>();
                                reservationIdSet.add(persistedResourcesReservation_temp.getId());

                                projectPlanningByTimeUnit_temp = new ProjectPlanningByTimeUnit(
                                    availableCPUResourcesForThisProjectInThisTimeUnit,
                                    availableMemoryResourcesForThisProjectInThisTimeUnit,
                                    availableStorageResourcesForThisProjectInThisTimeUnit, reservationIdSet);

                                projectHavingAResourcesReservationInThisTimeUnit_temp.put(projectId_temp,
                                    projectPlanningByTimeUnit_temp);

                                resourcesPoolPlanningByTimeUnit_temp
                                    .setProjectHavingAResourcesReservationInThisTimeUnit(projectHavingAResourcesReservationInThisTimeUnit_temp);

                            } else {

                                // Updating of projectPlanningByTimeUnit_temp of
                                // this projectId_temp (it exists already).

                                // Retrieving the resourcesQuotaMax of the
                                // project identified by id:
                                // persistedResourcesReservation_temp_.getProjectId()
                                Integer availableCPUResourcesForThisProjectInThisTimeUnit_PRE = null;
                                Integer availableMemoryResourcesForThisProjectInThisTimeUnit_PRE = null;
                                Integer availableStorageResourcesForThisProjectInThisTimeUnit_PRE = null;

                                // Retrieving the three
                                // availableXXXXXResourcesForThisProjectInThisTimeUnit_PRE,
                                // and updating them.

                                // -1 means unbound (was: 0 means unbound).
                                availableCPUResourcesForThisProjectInThisTimeUnit_PRE = projectPlanningByTimeUnit_temp
                                    .getAvailableCPUResourcesForThisProjectInThisTimeUnit();
                                Integer availableCPUResourcesForThisProjectInThisTimeUnit_POST = null;
                                if (availableCPUResourcesForThisProjectInThisTimeUnit_PRE == -1) {
                                    availableCPUResourcesForThisProjectInThisTimeUnit_POST = availableCPUResourcesForThisProjectInThisTimeUnit_PRE;
                                } else {
                                    availableCPUResourcesForThisProjectInThisTimeUnit_POST = availableCPUResourcesForThisProjectInThisTimeUnit_PRE
                                        - persistedResourcesReservation_temp.getReservedNumberOfCPUCore();
                                }

                                // -1 means unbound (was: 0 means unbound).
                                availableMemoryResourcesForThisProjectInThisTimeUnit_PRE = projectPlanningByTimeUnit_temp
                                    .getAvailableMemoryResourcesForThisProjectInThisTimeUnit();
                                Integer availableMemoryResourcesForThisProjectInThisTimeUnit_POST = null;
                                if (availableMemoryResourcesForThisProjectInThisTimeUnit_PRE == -1) {
                                    availableMemoryResourcesForThisProjectInThisTimeUnit_POST = availableMemoryResourcesForThisProjectInThisTimeUnit_PRE;
                                } else {
                                    availableMemoryResourcesForThisProjectInThisTimeUnit_POST = availableMemoryResourcesForThisProjectInThisTimeUnit_PRE
                                        - persistedResourcesReservation_temp.getReservedMemorySizeInMB();
                                }

                                // -1 means unbound (was: 0 means unbound).
                                availableStorageResourcesForThisProjectInThisTimeUnit_PRE = projectPlanningByTimeUnit_temp
                                    .getAvailableStorageResourcesForThisProjectInThisTimeUnit();
                                Integer availableStorageResourcesForThisProjectInThisTimeUnit_POST = null;
                                if (availableStorageResourcesForThisProjectInThisTimeUnit_PRE == -1) {
                                    availableStorageResourcesForThisProjectInThisTimeUnit_POST = availableStorageResourcesForThisProjectInThisTimeUnit_PRE;
                                } else {
                                    availableStorageResourcesForThisProjectInThisTimeUnit_POST = availableStorageResourcesForThisProjectInThisTimeUnit_PRE
                                        - persistedResourcesReservation_temp.getReservedStorageSizeInMB();
                                }

                                // Retrieving of the existing reservationIdSet,
                                // and updating it.
                                projectPlanningByTimeUnit_temp.getReservationIdSet().add(
                                    persistedResourcesReservation_temp.getId());

                                projectPlanningByTimeUnit_temp
                                    .setAvailableCPUResourcesForThisProjectInThisTimeUnit(availableCPUResourcesForThisProjectInThisTimeUnit_POST);
                                projectPlanningByTimeUnit_temp
                                    .setAvailableMemoryResourcesForThisProjectInThisTimeUnit(availableMemoryResourcesForThisProjectInThisTimeUnit_POST);
                                projectPlanningByTimeUnit_temp
                                    .setAvailableStorageResourcesForThisProjectInThisTimeUnit(availableStorageResourcesForThisProjectInThisTimeUnit_POST);
                                projectPlanningByTimeUnit_temp.setReservationIdSet(projectPlanningByTimeUnit_temp
                                    .getReservationIdSet());

                                projectHavingAResourcesReservationInThisTimeUnit_temp.put(projectId_temp,
                                    projectPlanningByTimeUnit_temp);

                                resourcesPoolPlanningByTimeUnit_temp
                                    .setProjectHavingAResourcesReservationInThisTimeUnit(projectHavingAResourcesReservationInThisTimeUnit_temp);

                            }
                        }
                        // this.displayResourcesReservationPlanning();
                        timeExpressedUsingTheDefinedTimeUnitGranularityType_temp = timeExpressedUsingTheDefinedTimeUnitGranularityType_temp + 1;
                    }

                } else {
                    // nothing to do.
                }
            }
            if (hasAResourcesPoolBeenFoundForPersistedResourcesReservation_temp) {
                // Everything is ok.
                result = Boolean.TRUE;
                return result;
            } else {
                throw new NoRelevantResourcesPoolException("No resourcesPool can be found for the persisted reservation: "
                    + persistedResourcesReservation_temp.getId() + ", " + persistedResourcesReservation_temp_hypervisorType
                    + ", " + persistedResourcesReservation_temp_location + ".");
            }
        } else {
            result = Boolean.TRUE;
            return result;
        }
    }

    // private void displayResourcesReservationPlanning() {
    // if (this.resourcesReservationPlanning == null) {
    // ResourcesReservationModuleImpl.log.finest("resourcesReservationPlanning is null");
    // } else {
    // Iterator<Long>
    // timeExpressedUsingTheDefinedTimeUnitGranularityType_temp_ITERATOR =
    // this.resourcesReservationPlanning
    // .keySet().iterator();
    // while
    // (timeExpressedUsingTheDefinedTimeUnitGranularityType_temp_ITERATOR.hasNext())
    // {
    // Long timeExpressedUsingTheDefinedTimeUnitGranularityType_temp =
    // timeExpressedUsingTheDefinedTimeUnitGranularityType_temp_ITERATOR
    // .next();
    // ResourcesPoolPlanningByTimeUnit resourcesPoolPlanningByTimeUnit_temp =
    // this.resourcesReservationPlanning
    // .get(timeExpressedUsingTheDefinedTimeUnitGranularityType_temp);
    // ResourcesReservationModuleImpl.log.finest("resourcesReservationPlanning for time: "
    // + timeExpressedUsingTheDefinedTimeUnitGranularityType_temp + " is " +
    // resourcesPoolPlanningByTimeUnit_temp
    // + ".");
    // }
    // }
    // }

    @Override
    public Integer reserveResources(final String projectId, final Calendar startTime, final Calendar endTime,
        final Integer numberOfCPUCore, final Integer memorySizeInMB, final Integer storageSizeInMB,
        final String hypervisorType, final String location) throws InvalidArgumentException, InsufficientResourceException,
        NoRelevantResourcesPoolException {

        ResourcesReservationModuleImpl.log.finest("Enters in: reserveResources(...).");

        // result will contain the newly created reservationId.
        Integer result = null;

        // Checking parameters: null, projectId known by Sirocco,
        // startTime>=date du jour, startTime>=endTime, numberOfCPUCore ...
        // >=1

        if ((projectId == null) || ("".equals(projectId))) {
            throw new InvalidArgumentException("The given projectId is either null or \"\".");
        } else {
            // Project project_temp = null;
            Query project_temp_list_query = this.em.createQuery("FROM Project p WHERE p.projectId=:projectId").setParameter(
                "projectId", projectId);
            if (project_temp_list_query == null) {
                throw new InvalidArgumentException("The given projectId (" + projectId + ") is unknown to Sirocco.");
            } else {
                if (project_temp_list_query.getResultList().size() >= 1) {
                    // project_temp = (Project) project_temp_list_query
                    // .getResultList().get(0);
                    project_temp_list_query.getResultList().get(0);
                } else {
                    throw new InvalidArgumentException("The given projectId (" + projectId + ") is unknown to Sirocco.");
                }
            }
        }

        if (startTime == null) {
            throw new InvalidArgumentException("The given startTime (" + startTime + ") is null.");
        } else {
            // nothing to do.
        }

        if (endTime == null) {
            throw new InvalidArgumentException("The given endTime (" + endTime + ") is null.");
        } else {
            // nothing to do.
        }

        if (startTime.getTimeInMillis() <= endTime.getTimeInMillis()) {
            // nothing to do.
        } else {
            throw new InvalidArgumentException("The given endTime (" + endTime.getTime()
                + ") is strictly prior (<) to startTime (" + startTime.getTime() + ").");
        }

        if ((numberOfCPUCore == null) || (numberOfCPUCore <= 0)) {
            throw new InvalidArgumentException("The given numberOfCPUCore (" + numberOfCPUCore + ") is null or <= 0.");
        } else {
            // nothing to do.
        }

        if ((memorySizeInMB == null) || (memorySizeInMB <= 0)) {
            throw new InvalidArgumentException("The given memorySizeInMB (" + memorySizeInMB + ") is null or <= 0.");
        } else {
            // nothing to do.
        }

        if ((storageSizeInMB == null) || (storageSizeInMB <= 0)) {
            throw new InvalidArgumentException("The given storageSizeInMB (" + storageSizeInMB + ") is null or <= 0.");
        } else {
            // nothing to do.
        }

        if ((hypervisorType == null) || ("".equals(hypervisorType))) {
            throw new InvalidArgumentException("The given hypervisorType (" + hypervisorType + ") is null or \"\".");
        } else {
            // nothing to do.
        }

        if (location == null) {
            throw new InvalidArgumentException("The given location (" + location + ") is null.");
        } else {
            // nothing to do.
        }

        // convert startTime, and endTime w.r.t. chosen
        // timeUnitGranularityType.
        long startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp;
        Calendar startTimeAsACalendar;
        long endTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp;
        Calendar endTimeAsACalendar;
        long currentTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp;
        if (TimeUnitGranularityType.HOUR.equals(this.timeUnitGranularityType)) {
            startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp = startTime.getTimeInMillis() / (60 * 60 * 1000);
            startTimeAsACalendar = new GregorianCalendar();
            startTimeAsACalendar
                .setTimeInMillis(startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp * 60 * 60 * 1000);
            endTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp = endTime.getTimeInMillis() / (60 * 60 * 1000);
            endTimeAsACalendar = new GregorianCalendar();
            endTimeAsACalendar.setTimeInMillis(endTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp * 60 * 60 * 1000);
            currentTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp = new Date().getTime() / (60 * 60 * 1000);
        } else if (TimeUnitGranularityType.DAY.equals(this.timeUnitGranularityType)) {
            startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp = startTime.getTimeInMillis() / (24 * 60 * 60 * 1000);
            startTimeAsACalendar = new GregorianCalendar();
            startTimeAsACalendar.setTimeInMillis(startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp * 24 * 60 * 60
                * 1000);
            endTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp = endTime.getTimeInMillis() / (24 * 60 * 60 * 1000);
            endTimeAsACalendar = new GregorianCalendar();
            endTimeAsACalendar.setTimeInMillis(endTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp * 24 * 60 * 60
                * 1000);
            currentTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp = new Date().getTime() / (24 * 60 * 60 * 1000);
        } else if (TimeUnitGranularityType.WEEK.equals(this.timeUnitGranularityType)) {
            // Let's find monday.
            Calendar startTime_temp = startTime;
            // If it is not monday, let's move in the past (a day backward) to
            // find the first previous monday.
            while (Calendar.MONDAY != startTime_temp.get(Calendar.DAY_OF_WEEK)) {
                startTime_temp.setTimeInMillis(startTime_temp.getTimeInMillis() - 24 * 60 * 60 * 1000);
            }
            startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp = startTime_temp.getTimeInMillis()
                / (7 * 24 * 60 * 60 * 1000);
            startTimeAsACalendar = new GregorianCalendar();
            startTimeAsACalendar.setTimeInMillis(startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp * 7 * 24 * 60
                * 60 * 1000);

            // Let's find sunday.
            Calendar endTime_temp = endTime;
            // If it is not sunday, let's move in the future (a day forward) to
            // find the first next sunday.
            while (Calendar.SUNDAY != endTime_temp.get(Calendar.DAY_OF_WEEK)) {
                endTime_temp.setTimeInMillis(endTime_temp.getTimeInMillis() + 24 * 60 * 60 * 1000);
            }
            endTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp = endTime_temp.getTimeInMillis()
                / (7 * 24 * 60 * 60 * 1000);
            endTimeAsACalendar = new GregorianCalendar();
            endTimeAsACalendar.setTimeInMillis(endTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp * 7 * 24 * 60 * 60
                * 1000);

            currentTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp = new Date().getTime() / (7 * 24 * 60 * 60 * 1000);
        } else {
            throw new InvalidArgumentException("The " + this.getClass().getName() + " module supports only: "
                + TimeUnitGranularityType.HOUR + ", " + TimeUnitGranularityType.DAY + ", or " + TimeUnitGranularityType.WEEK
                + " time unit granularity types.");
        }

        // Let's retrieve the relevant resourcesReservationPlanning.
        Iterator<ResourcesPoolOverallPlanning> rPOP_temp_ITERATOR = this.resourcesPoolOverallPlanning_List.iterator();
        ResourcesPoolOverallPlanning relevantResourcesPoolOverallPlanning = null;
        while (rPOP_temp_ITERATOR.hasNext()) {
            ResourcesPoolOverallPlanning rPOP_temp = rPOP_temp_ITERATOR.next();
            if ((hypervisorType.equalsIgnoreCase(rPOP_temp.getResourcePoolHypervisorType()))
                && (location.equalsIgnoreCase(rPOP_temp.getResourcePoolLocation()))) {
                relevantResourcesPoolOverallPlanning = rPOP_temp;
            } else {
                // nothing to do.
            }
        }

        if (relevantResourcesPoolOverallPlanning != null) {
            // nothing to do. Let's continue.
        } else {
            throw new NoRelevantResourcesPoolException("No resourcesPool can be found for the given hypervisorType: "
                + hypervisorType + ", and location: " + location + ".");
        }

        // Check the reservation feasability:
        // current date + bookingPeriodExpressedInTimeUnit <= startTime
        if ((currentTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp + this.bookingPeriodExpressedInTimeUnit) <= startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp) {

            // Let's run on/over the reservation's timeUnits.
            long timeExpressedUsingTheDefinedTimeUnitGranularityType_temp = startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp;

            while (timeExpressedUsingTheDefinedTimeUnitGranularityType_temp <= endTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp) {
                // ResourcesPoolPlanningByTimeUnit
                // resourcesPoolPlanningByTimeUnit_temp =
                // relevantResourcesPoolOverallPlanning
                // .getResourcesReservationPlanning()
                // .get(timeExpressedUsingTheDefinedTimeUnitGranularityType_temp);
                relevantResourcesPoolOverallPlanning.getResourcesReservationPlanning().get(
                    timeExpressedUsingTheDefinedTimeUnitGranularityType_temp);

                this.checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(
                    relevantResourcesPoolOverallPlanning, projectId, timeExpressedUsingTheDefinedTimeUnitGranularityType_temp,
                    numberOfCPUCore, memorySizeInMB, storageSizeInMB);

                timeExpressedUsingTheDefinedTimeUnitGranularityType_temp = timeExpressedUsingTheDefinedTimeUnitGranularityType_temp + 1;
            }

            // Here, the reservation is reservable.
            ResourcesReservation rREB = null;

            // Reservation creation, defining its state w.r.t. the current
            // timeUnit
            // (currentTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp),
            // and to the the first timeUnit of the reservation
            // (startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp).

            ResourcesReservationState resourcesReservationState = null;
            if (currentTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp < startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp) {
                resourcesReservationState = ResourcesReservationState.PENDING;
            } else {
                resourcesReservationState = ResourcesReservationState.ACTIVE;
            }

            // Here the reserved resources, and the free/remaining resources
            // available are the same (of course).
            rREB = new ResourcesReservation(resourcesReservationState, projectId, startTimeAsACalendar, endTimeAsACalendar,
                numberOfCPUCore, memorySizeInMB, storageSizeInMB, numberOfCPUCore, memorySizeInMB, storageSizeInMB,
                hypervisorType, location);

            // Persisting reservation.
            this.em.persist(rREB);
            this.em.flush();

            // Updating the the available resourceQuota for each timeUnit of the
            // reservation, and the available resourcePool for each timeUnit
            // of the reservation.

            this.initiateOrUpdateResourcesReservationPlanningWithRegardToTheResourcesReservation(
                startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp,
                endTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp, rREB);

            // Persisting the reservation.
            this.em.persist(rREB);
            this.em.flush();

            result = rREB.getId();
            return result;
        } else {
            throw new InvalidArgumentException("Current date ("
                + currentTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp + ") + bookingPeriodExpressedInTimeUnit ("
                + this.bookingPeriodExpressedInTimeUnit + ") is NOT less or equal to: "
                + startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp + ".");
        }
    }

    private void checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(
        final ResourcesPoolOverallPlanning resourcesPoolOverallPlanning, final String projectId, final long currentTime_temp,
        final Integer numberOfCPUCore, final Integer memorySizeInMB, final Integer storageSizeInMB)
        throws InsufficientResourceException, InvalidArgumentException {
        ResourcesReservationModuleImpl.log
            .finest("Enters in: checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject("
                + resourcesPoolOverallPlanning + ", " + projectId + ", " + currentTime_temp + ", " + numberOfCPUCore + ", "
                + memorySizeInMB + ", " + storageSizeInMB + ").");

        if (resourcesPoolOverallPlanning == null) {

        } else {

            ResourcesPoolPlanningByTimeUnit resourcesPoolPlanningByTimeUnit_temp = resourcesPoolOverallPlanning
                .getResourcesReservationPlanning().get(currentTime_temp);

            if (resourcesPoolPlanningByTimeUnit_temp == null) {
                // Checking the resources availability (CPU, memory, and
                // Storage) w.r.t. the resourcePool available in this timeUnit.
                if (resourcesPoolOverallPlanning.getResourcesPoolTotalCPUCapacity() == -1) {
                    // nothing to do.
                } else {
                    if (resourcesPoolOverallPlanning.getResourcesPoolTotalCPUCapacity() - numberOfCPUCore < 0) {
                        InsufficientResourceException e = new InsufficientResourceException(
                            "resourcesPoolTotalCPUCapacity - numberOfCPUCore < 0.");
                        ResourcesReservationModuleImpl.log
                            .finest("Leaves: checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(...): "
                                + e.getMessage());
                        throw e;
                    } else {
                        // nothing to do.
                    }
                }
                if (resourcesPoolOverallPlanning.getResourcesPoolTotalMemoryCapacity() == -1) {
                    // nothing to do.
                } else {
                    if (resourcesPoolOverallPlanning.getResourcesPoolTotalMemoryCapacity() - memorySizeInMB < 0) {
                        InsufficientResourceException e = new InsufficientResourceException(
                            "resourcesPoolTotalMemoryCapacity - memorySizeInMB < 0.");
                        ResourcesReservationModuleImpl.log
                            .finest("Leaves: checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(...): "
                                + e.getMessage());
                        throw e;
                    } else {
                        // nothing to do.
                    }
                }
                if (resourcesPoolOverallPlanning.getResourcesPoolTotalStorageCapacity() == -1) {
                    // nothing to do.
                } else {
                    if (resourcesPoolOverallPlanning.getResourcesPoolTotalStorageCapacity() - storageSizeInMB < 0) {
                        InsufficientResourceException e = new InsufficientResourceException(
                            "resourcesPoolTotalStorageCapacity - storageSizeInMB < 0.");
                        ResourcesReservationModuleImpl.log
                            .finest("Leaves: checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(...): "
                                + e.getMessage());
                        throw e;
                    } else {
                        // nothing to do.
                    }
                }

                // Checks resources availability in terms of CPU, memory, and
                // storage w.r.t. project's available resourceQuota for this
                // timeUnit.
                Project project_temp = null;
                Query project_temp_list_query = this.em.createQuery("FROM Project p WHERE p.projectId=:projectId")
                    .setParameter("projectId", projectId);
                if (project_temp_list_query == null) {
                    throw new InvalidArgumentException("The given projectId (" + projectId + ") is unknown to Sirocco.");
                } else {
                    project_temp = (Project) project_temp_list_query.getSingleResult();
                }

                // -1 means unbound (was: 0 means unbound).
                Integer resourceQuotaCPUForProject = project_temp.getResourceQuota().getCpuQuota();
                if (resourceQuotaCPUForProject == -1) {
                    // nothing to do.
                } else {
                    if (resourceQuotaCPUForProject - numberOfCPUCore < 0) {
                        InsufficientResourceException e = new InsufficientResourceException(
                            "resourceQuotaCPUForProject - numberOfCPUCore < 0.");
                        ResourcesReservationModuleImpl.log
                            .finest("Leaves: checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(...): "
                                + e.getMessage());
                        throw e;
                    } else {
                        // nothing to do.
                    }
                }

                // -1 means unbound (was: 0 means unbound).
                Integer resourceQuotaMemoryInMBForProject = project_temp.getResourceQuota().getRamQuotaInMB();
                if (resourceQuotaMemoryInMBForProject == -1) {
                    // nothing to do.
                } else {
                    if (resourceQuotaMemoryInMBForProject - memorySizeInMB < 0) {
                        InsufficientResourceException e = new InsufficientResourceException(
                            "resourceQuotaMemoryInMBForProject - memorySizeInMB < 0.");
                        ResourcesReservationModuleImpl.log
                            .finest("Leaves: checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(...): "
                                + e.getMessage());
                        throw e;
                    } else {
                        // nothing to do.
                    }
                }

                // -1 means unbound (was: 0 means unbound).
                Integer resourceQuotaStorageInMBForProject = project_temp.getResourceQuota().getDiskQuotaInMB();
                if (resourceQuotaStorageInMBForProject == -1) {
                    // nothing to do.
                } else {
                    if (resourceQuotaStorageInMBForProject - storageSizeInMB < 0) {
                        InsufficientResourceException e = new InsufficientResourceException(
                            "resourceQuotaStorageInMBForProject - storageSizeInMB < 0.");
                        ResourcesReservationModuleImpl.log
                            .finest("Leaves: checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(...): "
                                + e.getMessage());
                        throw e;
                    } else {
                        // nothing to do.
                    }
                }
            } else {
                // Checking CPU, memory, and storage w.r.t. the available
                // resourcePool for this timeUnit reservation.
                if (resourcesPoolPlanningByTimeUnit_temp.getAvailableCPUResourcesForThisResourcePoolInThisTimeUnit() == -1) {
                    // nothing to do.
                } else {
                    if (resourcesPoolPlanningByTimeUnit_temp.getAvailableCPUResourcesForThisResourcePoolInThisTimeUnit()
                        - numberOfCPUCore < 0) {
                        InsufficientResourceException e = new InsufficientResourceException(
                            "resourcesPoolPlanningByTimeUnit_temp.getAvailableCPUResourcesForThisResourcePoolInThisTimeUnit() - numberOfCPUCore < 0.");
                        ResourcesReservationModuleImpl.log
                            .finest("Leaves: checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(...): "
                                + e.getMessage());
                        throw e;
                    } else {
                        // nothing to do.
                    }
                }
                if (resourcesPoolPlanningByTimeUnit_temp.getAvailableMemoryResourcesForThisResourcePoolInThisTimeUnit() == -1) {
                    // nothing to do.
                } else {
                    if (resourcesPoolPlanningByTimeUnit_temp.getAvailableMemoryResourcesForThisResourcePoolInThisTimeUnit()
                        - memorySizeInMB < 0) {
                        InsufficientResourceException e = new InsufficientResourceException(
                            "resourcesPoolPlanningByTimeUnit_temp.getAvailableMemoryResourcesForThisResourcePoolInThisTimeUnit() - memorySizeInMB < 0.");
                        ResourcesReservationModuleImpl.log
                            .finest("Leaves: checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(...): "
                                + e.getMessage());
                        throw e;
                    } else {
                        // nothing to do.
                    }
                }
                if (resourcesPoolPlanningByTimeUnit_temp.getAvailableStorageResourcesForThisResourcePoolInThisTimeUnit() == -1) {
                    // nothing to do.
                } else {
                    if (resourcesPoolPlanningByTimeUnit_temp.getAvailableStorageResourcesForThisResourcePoolInThisTimeUnit()
                        - storageSizeInMB < 0) {
                        InsufficientResourceException e = new InsufficientResourceException(
                            "resourcesPoolPlanningByTimeUnit_temp.getAvailableStorageResourcesForThisResourcePoolInThisTimeUnit() - storageSizeInMB < 0.");
                        ResourcesReservationModuleImpl.log
                            .finest("Leaves: checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(...): "
                                + e.getMessage());
                        throw e;
                    } else {
                        // nothing to do.
                    }
                }

                // Checking availability of CPU, memory, and storage w.r.t. the
                // project's available resourceQuota for this reservation
                // timeUnit.
                Hashtable<String, ProjectPlanningByTimeUnit> projectHavingAResourcesReservationInThisTimeUnit_temp = resourcesPoolPlanningByTimeUnit_temp
                    .getProjectHavingAResourcesReservationInThisTimeUnit();
                ProjectPlanningByTimeUnit projectPlanningByTimeUnit_temp = projectHavingAResourcesReservationInThisTimeUnit_temp
                    .get(projectId);

                if (projectPlanningByTimeUnit_temp == null) {
                    // Retrieving resourcesQuotaMax of the project identified by
                    // id: persistedResourcesReservation_temp_.getProjectId()
                    Integer resourceQuotaCPUForProject = null;
                    Integer resourceQuotaMemoryForProject = null;
                    Integer resourceQuotaStorageForProject = null;

                    // This is the first reservation for this project, and this
                    // timeUnit. So, let's use the project's resourceQuota max.
                    Project project_temp = null;
                    Query project_temp_list_query = this.em.createQuery("FROM Project p WHERE p.projectId=:projectId")
                        .setParameter("projectId", projectId);
                    if (project_temp_list_query == null) {
                        throw new InvalidArgumentException("The given projectId (" + projectId + ") is unknown to Sirocco.");
                    } else {
                        project_temp = (Project) project_temp_list_query.getSingleResult();
                    }

                    // -1 means unbound (was: 0 means unbound).
                    resourceQuotaCPUForProject = project_temp.getResourceQuota().getCpuQuota();
                    if (resourceQuotaCPUForProject == -1) {
                        // nothing to do.
                    } else {
                        if (resourceQuotaCPUForProject - numberOfCPUCore < 0) {
                            InsufficientResourceException e = new InsufficientResourceException(
                                "resourceQuotaCPUForProject - numberOfCPUCore < 0.");
                            ResourcesReservationModuleImpl.log
                                .finest("Leaves: checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(...): "
                                    + e.getMessage());
                            throw e;
                        } else {
                            // nothing to do.
                        }
                    }

                    // -1 means unbound (was: 0 means unbound).
                    resourceQuotaMemoryForProject = project_temp.getResourceQuota().getRamQuotaInMB();
                    if (resourceQuotaMemoryForProject == -1) {
                        // nothing to do.
                    } else {
                        if (resourceQuotaMemoryForProject - memorySizeInMB < 0) {
                            InsufficientResourceException e = new InsufficientResourceException(
                                "resourceQuotaMemoryForProject - memorySizeInMB < 0.");
                            ResourcesReservationModuleImpl.log
                                .finest("Leaves: checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(...): "
                                    + e.getMessage());
                            throw e;
                        } else {
                            // nothing to do.
                        }
                    }

                    // -1 means unbound (was: 0 means unbound).
                    resourceQuotaStorageForProject = project_temp.getResourceQuota().getDiskQuotaInMB();
                    if (resourceQuotaStorageForProject == -1) {
                        // nothing to do.
                    } else {
                        if (resourceQuotaStorageForProject - storageSizeInMB < 0) {
                            InsufficientResourceException e = new InsufficientResourceException(
                                "resourceQuotaStorageForProject - storageSizeInMB < 0.");
                            ResourcesReservationModuleImpl.log
                                .finest("Leaves: checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(...): "
                                    + e.getMessage());
                            throw e;
                        } else {
                            // nothing to do.
                        }
                    }
                } else {
                    Integer availableResourceQuotaCPUForProject = null;
                    Integer availableResourceQuotaMemoryForProject = null;
                    Integer availableResourceQuotaStorageForProject = null;

                    // -1 means unbound (was: 0 means unbound).
                    availableResourceQuotaCPUForProject = projectPlanningByTimeUnit_temp
                        .getAvailableCPUResourcesForThisProjectInThisTimeUnit();
                    if (availableResourceQuotaCPUForProject == -1) {
                        // nothing to do.
                    } else {
                        if (availableResourceQuotaCPUForProject - numberOfCPUCore < 0) {
                            InsufficientResourceException e = new InsufficientResourceException(
                                "availableResourceQuotaCPUForProject - numberOfCPUCore < 0.");
                            ResourcesReservationModuleImpl.log
                                .finest("Leaves: checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(...): "
                                    + e.getMessage());
                            throw e;
                        } else {
                            // nothing to do.
                        }
                    }

                    // -1 means unbound (was: 0 means unbound).
                    availableResourceQuotaMemoryForProject = projectPlanningByTimeUnit_temp
                        .getAvailableCPUResourcesForThisProjectInThisTimeUnit();
                    if (availableResourceQuotaMemoryForProject == -1) {
                        // nothing to do.
                    } else {
                        if (availableResourceQuotaMemoryForProject - memorySizeInMB < 0) {
                            InsufficientResourceException e = new InsufficientResourceException(
                                "availableResourceQuotaMemoryForProject - memorySizeInMB < 0.");
                            ResourcesReservationModuleImpl.log
                                .finest("Leaves: checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(...): "
                                    + e.getMessage());
                            throw e;
                        } else {
                            // nothing to do.
                        }
                    }

                    // -1 means unbound (was: 0 means unbound).
                    availableResourceQuotaStorageForProject = projectPlanningByTimeUnit_temp
                        .getAvailableStorageResourcesForThisProjectInThisTimeUnit();
                    if (availableResourceQuotaStorageForProject == -1) {
                        // nothing to do.
                    } else {
                        if (availableResourceQuotaStorageForProject - storageSizeInMB < 0) {
                            InsufficientResourceException e = new InsufficientResourceException(
                                "availableResourceQuotaStorageForProject - storageSizeInMB < 0.");
                            ResourcesReservationModuleImpl.log
                                .finest("Leaves: checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(...): "
                                    + e.getMessage());
                            throw e;
                        } else {
                            // nothing to do.
                        }
                    }
                }
            }

            // Here, the reservation is reservable.

        }
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<ResourcesReservationVO> listResourcesReservations() throws InvalidArgumentException,
        NoRelevantResourcesPoolException {
        ResourcesReservationModuleImpl.log.finest("Enters in: listResourcesReservations().");

        List<ResourcesReservationVO> result = null;
        Query result_temp = null;
        result_temp = this.em.createQuery("FROM " + ResourcesReservation.class.getSimpleName() + " p");
        if (result_temp == null) {
            result = new ArrayList<ResourcesReservationVO>();
            return result;
        } else {
            // Convert result_temp.getResultList() from
            // List<ResourcesReservation> to
            // List<ResourcesReservationValueObject>
            Iterator<ResourcesReservation> pRR_temp_ITERATOR = result_temp.getResultList().iterator();
            result = new ArrayList<ResourcesReservationVO>();
            while (pRR_temp_ITERATOR.hasNext()) {
                ResourcesReservation pRR_temp = pRR_temp_ITERATOR.next();
                if (ResourcesReservationState.DELETED.equals(pRR_temp.getResourcesReservationState())) {
                    // We ignore the DELETED reservations.
                } else {
                    ResourcesReservationVO pRRVO_temp = new ResourcesReservationVO(pRR_temp.getId(),
                        pRR_temp.getResourcesReservationState(), pRR_temp.getProjectId(), pRR_temp.getStartTime(),
                        pRR_temp.getEndTime(), pRR_temp.getReservedNumberOfCPUCore(), pRR_temp.getReservedMemorySizeInMB(),
                        pRR_temp.getReservedStorageSizeInMB(), pRR_temp.getFreeNumberOfCpuCores(),
                        pRR_temp.getFreeMemorySizeInMB(), pRR_temp.getFreeStorageSizeInMB(), pRR_temp.getHypervisorType(),
                        pRR_temp.getLocation());
                    result.add(pRRVO_temp);
                }
            }
            return result;
        }
    }

    @Override
    public Boolean createAVMUsingReservedResources(final Integer reservationId, final VirtualMachineSpec vmSpec)
        throws InvalidUsernameException, InvalidNameException, InvalidVMImageException, ResourceQuotaExceededException,
        DuplicateNameException, InvalidVMSizeException, InvalidProjectNameException, InvalidArgumentException,
        InsufficientResourceException, ReservationException {

        ResourcesReservationModuleImpl.log.finest("Enters in: createAVMUsingReservedResources(" + reservationId + ", " + vmSpec
            + ").");

        Boolean result = null;

        ResourcesReservation rREB = null;

        // Checking parameters: known by Sirocco.
        Query persistedResourcesReservation_list_query = this.em.createQuery(
            "FROM " + ResourcesReservation.class.getSimpleName() + " p WHERE p.id=:reservationId").setParameter(
            "reservationId", reservationId);
        if (persistedResourcesReservation_list_query == null) {
            throw new InvalidArgumentException("The given reservationId (" + reservationId + ") is unknown to Sirocco.");
        } else {
            @SuppressWarnings("unchecked")
            List<ResourcesReservation> persistedResourcesReservation_list = persistedResourcesReservation_list_query
                .getResultList();
            if (persistedResourcesReservation_list.size() == 0) {
                throw new InvalidArgumentException("The given reservationId (" + reservationId + ") is unknown to Sirocco.");
            } else {
                rREB = persistedResourcesReservation_list.get(0);
            }
        }

        // Checking the reservation's state : it should be ACTIVE.
        if (ResourcesReservationState.ACTIVE.equals(rREB.getResourcesReservationState())) {
            // nothing to do.
        } else {
            throw new InvalidArgumentException("The ResourcesReservationState of the given reservationId (" + reservationId
                + ") is NOT: " + ResourcesReservationState.ACTIVE + " (it is: " + rREB.getResourcesReservationState() + ").");
        }

        // Checking the hypervisor type (amongst the one devlared in rREB, and
        // the one specified by vmSpec (via imageId)).

        Integer givenImageId = null;
        givenImageId = vmSpec.getImageId();

        Query vMImage_list_query = this.em.createQuery("FROM VMImage v WHERE v.id=:givenImageId AND v.status<>'DESTROYED'")
            .setParameter("givenImageId", givenImageId);
        if (vMImage_list_query == null) {
            throw new InvalidArgumentException("The given vmSpec specifies an imageId (" + givenImageId
                + ") unknown to Sirocco.");
        } else {
            VMImage vMImage = (VMImage) vMImage_list_query.getSingleResult();
            if (rREB.getHypervisorType().equals(vMImage.getHypervisor())) {
                // nothing to do (everything is ok).
            } else {
                throw new ReservationException("The reservation's hypervisor type (" + rREB.getHypervisorType()
                    + "), and the given VM's one (" + vMImage.getHypervisor()
                    + ") are different. It is not possible to create this VM on these reserved resources.");
            }
        }

        // Checking the available resources by comparing the reserved resources
        // taken by already created VM, (via entitymanager.VirtualMachine), and
        // the resources required for this VM creation.

        // Retrieving the created VM.
        @SuppressWarnings("unchecked")
        List<VirtualMachine> virtualMachine_list = this.em.createQuery("FROM VirtualMachine v WHERE v.status<>'DESTROYED'")
            .getResultList();

        // Retrieving the VM created on reserved resources coming from the
        // reservation identified by the id: reservationId_
        int numCPUAlreadyTaken = 0;
        int memorySizeInMBAlreadyTaken = 0;
        int diskSizeInMBAlreadyTaken = 0;
        Iterator<VirtualMachine> vM_temp_ITERATOR = virtualMachine_list.iterator();
        while (vM_temp_ITERATOR.hasNext()) {
            VirtualMachine vM_temp = vM_temp_ITERATOR.next();
            Integer reservationId_temp = vM_temp.getReservationId();
            if (reservationId_temp == null) {
                // nothing to do. This VM hasn't been created using reserved
                // resources.
            } else {
                if (reservationId_temp.intValue() == reservationId.intValue()) {
                    if (vM_temp.getName().equals(vmSpec.getVmName())) {
                        // nothing to do, this is VM we are about to create.
                    } else {
                        numCPUAlreadyTaken = numCPUAlreadyTaken + vM_temp.getNbCpus();
                        memorySizeInMBAlreadyTaken = memorySizeInMBAlreadyTaken + vM_temp.getRamSizeMB();
                        diskSizeInMBAlreadyTaken = diskSizeInMBAlreadyTaken + vM_temp.getDiskSizeMB();
                    }
                } else {
                    // nothing to do. This VM has been created using reserved
                    // resources, but not the ones of reservation with id:
                    // reservationId_.
                }
            }
        }

        // Checking if there is enough resources to create vmSpec_.
        int numCPUStillAvailableInResa = rREB.getReservedNumberOfCPUCore() - numCPUAlreadyTaken;
        int memorySizeInMBStillAvailableInResa = rREB.getReservedMemorySizeInMB() - memorySizeInMBAlreadyTaken;
        int diskSizeInMBStillAvailableInResa = rREB.getReservedStorageSizeInMB() - diskSizeInMBAlreadyTaken;

        if (numCPUStillAvailableInResa < vmSpec.getNumCPUs()) {
            throw new InsufficientResourceException("There is not enough CPU cores available (" + numCPUStillAvailableInResa
                + ") in the reservation to create the given VM that requires (" + vmSpec.getNumCPUs() + ") CPU core(s).");
        } else {
            if (memorySizeInMBStillAvailableInResa < vmSpec.getMemorySizeMB()) {
                throw new InsufficientResourceException("There is not enough memory (in MB) available ("
                    + memorySizeInMBStillAvailableInResa + ") in the reservation to create the given VM that requires ("
                    + vmSpec.getMemorySizeMB() + ") MB of memory.");
            } else {
                if (diskSizeInMBStillAvailableInResa < vmSpec.getDiskSizeMB()) {
                    throw new InsufficientResourceException("There is not enough storage (in MB) available ("
                        + diskSizeInMBStillAvailableInResa + ") in the reservation to create the given VM that requires ("
                        + vmSpec.getDiskSizeMB() + ") MB of storage.");
                } else {
                    // There is enough resources in resa.
                    rREB.setFreeNumberOfCpuCores(numCPUStillAvailableInResa - vmSpec.getNumCPUs());
                    rREB.setFreeMemorySizeInMB(memorySizeInMBStillAvailableInResa - vmSpec.getMemorySizeMB());
                    rREB.setFreeStorageSizeInMB(diskSizeInMBStillAvailableInResa - vmSpec.getDiskSizeMB());
                }
            }
        }

        // Using the VM creation command provided by resourcePoolIdentifier via
        // cloudAdmin.
        // this.cloudAdmin.createVirtualMachine(rREB.getProjectId(), username,
        // vmSpec);
        // Note that CloudAdminBean.createVM is in charge of calling
        // createAVMUsingReservedResources !

        // Persisting reservation.
        this.em.merge(rREB);
        this.em.flush();

        result = Boolean.TRUE;
        return result;
    }

    @Override
    public Boolean destroyAVMThatUsesReservedResources(final String vmId) throws InvalidVirtualMachineIdException,
        InvalidPowerStateException, PermissionDeniedException, InvalidArgumentException, ReservationException {

        ResourcesReservationModuleImpl.log.finest("Enters in: destroyAVMThatUsesReservedResources(" + vmId + ").");

        Boolean result = null;

        // Checking parameters: null, "", known by Sirocco
        // (via entityManager), id with a defined reservationId.
        if ((vmId == null) || ("".equals(vmId))) {
            throw new InvalidArgumentException("The given vMId (" + vmId + ") is null or \"\".");
        } else {
            // nothing to do.
        }

        VirtualMachine virtualMachine = null;

        // Checking parameters: known by Sirocco.
        // (via entityManager)
        Query virtualMachine_list_query = this.em.createQuery("FROM VirtualMachine v WHERE v.id=:vmId").setParameter("vmId",
            Integer.parseInt(vmId));
        if (virtualMachine_list_query == null) {
            throw new InvalidArgumentException("The given vmId (" + vmId + ") is unknown to Sirocco.");
        } else {
            @SuppressWarnings("unchecked")
            List<VirtualMachine> virtualMachine_list = virtualMachine_list_query.getResultList();
            if (virtualMachine_list.size() == 0) {
                throw new InvalidArgumentException("The given vmId (" + vmId + ") is unknown to Sirocco.");
            } else {
                virtualMachine = virtualMachine_list.get(0);
            }
        }
        Integer reservationId = null;
        reservationId = virtualMachine.getReservationId();
        if (reservationId == null) {
            // "The given vMId (" + vMId +
            // ") has not been created using reserved resources (its reservationId is null).";
            result = false;
            return result;
        } else {
            ResourcesReservation rREB = null;
            // Checking parameters: known by Sirocco.
            // (via entityManager)
            Query persistedResourcesReservation_list_query = this.em.createQuery(
                "FROM " + ResourcesReservation.class.getSimpleName() + " p WHERE p.id=:reservationId").setParameter(
                "reservationId", reservationId);
            if (persistedResourcesReservation_list_query == null) {
                throw new ReservationException("The reservationId (" + reservationId + ") found via the given vmId (" + vmId
                    + ") is unknown to Sirocco.");
            } else {
                @SuppressWarnings("unchecked")
                List<ResourcesReservation> persistedResourcesReservation_list = persistedResourcesReservation_list_query
                    .getResultList();
                if (persistedResourcesReservation_list.size() == 0) {
                    throw new ReservationException("The reservationId (" + reservationId + ") found via the given vmId ("
                        + vmId + ") is unknown to Sirocco.");
                } else {
                    // Here, a reservation associated to the given parameter
                    // exists.
                    rREB = persistedResourcesReservation_list.get(0);
                    // The VM will be destroyed, so we update rREB.

                    rREB.setFreeNumberOfCpuCores(rREB.getFreeNumberOfCpuCores() + virtualMachine.getNbCpus());
                    rREB.setFreeMemorySizeInMB(rREB.getFreeMemorySizeInMB() + virtualMachine.getRamSizeMB());
                    rREB.setFreeStorageSizeInMB(rREB.getFreeStorageSizeInMB() + virtualMachine.getDiskSizeMB());

                    this.em.merge(rREB);
                    this.em.flush();
                }
            }
            // // Checking the reservation' state; it should be ACTIVE.
            // if
            // (ResourcesReservationState.ACTIVE.equals(rREB.getResourcesReservationState()))
            // {
            // // nothing to do.
            // } else {
            // throw new
            // ReservationException("The ResourcesReservationState of the reservationId ("
            // + reservationId
            // + ") found via the given vMId (" + vMId + ") is NOT:" +
            // ResourcesReservationState.ACTIVE + " (it is: "
            // + rREB.getResourcesReservationState() + ").");
            // }
        }
        result = Boolean.TRUE;
        return result;
    }

    @Override
    public void cancelAResourcesReservation(final String fromUsername, final Integer reservationId)
        throws InvalidVirtualMachineIdException, InvalidPowerStateException, PermissionDeniedException,
        InvalidArgumentException, NoRelevantResourcesPoolException, ReservationException {

        ResourcesReservationModuleImpl.log.finest("Enters in: cancelAResourcesReservation(" + fromUsername + ", "
            + reservationId + ").");

        // Checking parameters: == null, corresponding to a persisted
        // reservation.
        // Checking parameters: null, "" and known by Sirocco
        // (via entityManager) id with a defined reservationId.
        if (reservationId == null) {
            throw new InvalidArgumentException("The given reservationId (" + reservationId + ") is null.");
        } else {
            // nothing to do.
        }

        ResourcesReservation rREB = null;

        // Checking parameters: known by Sirocco
        // (via entityManager)

        Query rREB_list_query = this.em.createQuery(
            "FROM " + ResourcesReservation.class.getSimpleName() + " p WHERE p.id=:reservationId").setParameter(
            "reservationId", reservationId);
        if (rREB_list_query == null) {
            throw new InvalidArgumentException("The given reservationId (" + reservationId + ") is unknown to Sirocco.");
        } else {
            @SuppressWarnings("unchecked")
            List<ResourcesReservation> rREB_list = rREB_list_query.getResultList();
            if (rREB_list.size() == 0) {
                throw new InvalidArgumentException("The given reservationId (" + reservationId + ") is unknown to Sirocco.");
            } else {
                // Only one reservation is allowed by given reservation id.
                rREB = rREB_list.get(0);
            }
        }

        if (ResourcesReservationState.PENDING.equals(rREB.getResourcesReservationState())) {
            // The reservation is in the PENDING state.
            // Updating the avaible resources.
            this.removesTheGivenResourcesReservationAndUpdatesTheResourcesReservationPlanning(rREB);

            // Change the reservation's state to DELETED.
            rREB.setResourcesReservationState(ResourcesReservationState.DELETED);
        } else if (ResourcesReservationState.ACTIVE.equals(rREB.getResourcesReservationState())) {
            // The reservation is ACTIVE.
            // Destroying the associated VM via
            // destroyAVMThatUsesReservedResources.

            // Retrieving VM created on resources reserved by reservationId_.
            Query vM_list_query = this.em.createQuery("FROM VirtualMachine v WHERE v.reservationId=:reservationId")
                .setParameter("reservationId", reservationId);
            if (vM_list_query == null) {
                throw new ReservationException("Query failed.");
            } else {
                @SuppressWarnings("unchecked")
                List<VirtualMachine> vM_list = vM_list_query.getResultList();
                if (vM_list.size() == 0) {
                    // nothing to do.
                } else {
                    // Destructing all the VM associated to th reservation
                    // identifier: reservationId.
                    Iterator<VirtualMachine> vM_temp_ITERATOR = vM_list.iterator();
                    while (vM_temp_ITERATOR.hasNext()) {
                        VirtualMachine vM_temp = vM_temp_ITERATOR.next();
                        this.cloudAdmin.destroyVirtualMachine(vM_temp.getId().toString(), fromUsername);
                    }
                }
            }

            // Updating the available resources.
            this.removesTheGivenResourcesReservationAndUpdatesTheResourcesReservationPlanning(rREB);

            // update rREB freeNumberOfCpuCores, freeMemeory, and freeStorage
            rREB.setFreeNumberOfCpuCores(rREB.getReservedNumberOfCPUCore());
            rREB.setFreeMemorySizeInMB(rREB.getReservedMemorySizeInMB());
            rREB.setFreeStorageSizeInMB(rREB.getReservedStorageSizeInMB());

            // Move reservation to the DELETED state.
            rREB.setResourcesReservationState(ResourcesReservationState.DELETED);
        } else if (ResourcesReservationState.DELETED.equals(rREB.getResourcesReservationState())) {
            // nothing to do. Reservation is already in the DELETED state.
        }

        // Let's persist the reservation.
        this.em.merge(rREB);
        this.em.flush();
    }

    /**
     * Removes the given persistedResourcesReservation_temp from
     * this.resourcesReservationPlanning, and updates
     * this.resourcesReservationPlanning
     * 
     * @throws InvalidArgumentException
     * @throws NoRelevantResourcesPoolException
     */
    private void removesTheGivenResourcesReservationAndUpdatesTheResourcesReservationPlanning(
        final ResourcesReservation persistedResourcesReservation_temp) throws InvalidArgumentException,
        NoRelevantResourcesPoolException {

        ResourcesReservationModuleImpl.log
            .finest("Enters in: removesTheGivenResourcesReservationAndUpdatesTheResourcesReservationPlanning("
                + persistedResourcesReservation_temp + ").");

        Calendar startTime = persistedResourcesReservation_temp.getStartTime();
        Calendar endTime = persistedResourcesReservation_temp.getEndTime();

        // convert startTime, and endTime w.r.t. chosen
        // timeUnitGranularityType.
        long startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp;
        Calendar startTimeAsACalendar;
        long endTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp;
        Calendar endTimeAsACalendar;
        if (TimeUnitGranularityType.HOUR.equals(this.timeUnitGranularityType)) {
            startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp = startTime.getTimeInMillis() / (60 * 60 * 1000);
            startTimeAsACalendar = new GregorianCalendar();
            startTimeAsACalendar
                .setTimeInMillis(startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp * 60 * 60 * 1000);
            endTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp = endTime.getTimeInMillis() / (60 * 60 * 1000);
            endTimeAsACalendar = new GregorianCalendar();
            endTimeAsACalendar.setTimeInMillis(endTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp * 60 * 60 * 1000);
        } else if (TimeUnitGranularityType.DAY.equals(this.timeUnitGranularityType)) {
            startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp = startTime.getTimeInMillis() / (24 * 60 * 60 * 1000);
            startTimeAsACalendar = new GregorianCalendar();
            startTimeAsACalendar.setTimeInMillis(startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp * 24 * 60 * 60
                * 1000);
            endTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp = endTime.getTimeInMillis() / (24 * 60 * 60 * 1000);
            endTimeAsACalendar = new GregorianCalendar();
            endTimeAsACalendar.setTimeInMillis(endTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp * 24 * 60 * 60
                * 1000);
        } else if (TimeUnitGranularityType.WEEK.equals(this.timeUnitGranularityType)) {
            startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp = startTime.getTimeInMillis()
                / (7 * 24 * 60 * 60 * 1000);
            startTimeAsACalendar = new GregorianCalendar();
            startTimeAsACalendar.setTimeInMillis(startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp * 7 * 24 * 60
                * 60 * 1000);
            endTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp = endTime.getTimeInMillis() / (7 * 24 * 60 * 60 * 1000);
            endTimeAsACalendar = new GregorianCalendar();
            endTimeAsACalendar.setTimeInMillis(endTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp * 7 * 24 * 60 * 60
                * 1000);
        } else {
            throw new InvalidArgumentException("The " + this.getClass().getName() + " module supports only: "
                + TimeUnitGranularityType.HOUR + ", " + TimeUnitGranularityType.DAY + ", or " + TimeUnitGranularityType.WEEK
                + " time unit granularity types.");
        }

        // Let's retrieve the relevant resourcesReservationPlanning.
        String hypervisorType = persistedResourcesReservation_temp.getHypervisorType();
        String location = persistedResourcesReservation_temp.getLocation();

        Iterator<ResourcesPoolOverallPlanning> rPOP_temp_ITERATOR = this.resourcesPoolOverallPlanning_List.iterator();
        ResourcesPoolOverallPlanning relevantResourcesPoolOverallPlanning = null;
        while (rPOP_temp_ITERATOR.hasNext()) {
            ResourcesPoolOverallPlanning rPOP_temp = rPOP_temp_ITERATOR.next();
            if ((hypervisorType.equalsIgnoreCase(rPOP_temp.getResourcePoolHypervisorType()))
                && (location.equalsIgnoreCase(rPOP_temp.getResourcePoolLocation()))) {
                relevantResourcesPoolOverallPlanning = rPOP_temp;
            } else {
                // nothing to do.
            }
        }

        if (relevantResourcesPoolOverallPlanning != null) {
            // nothing to do. Let's continue.
        } else {
            throw new NoRelevantResourcesPoolException("No resourcesPool can be found for the given resources reservation: "
                + persistedResourcesReservation_temp.getId() + ", " + hypervisorType + ", and location: " + location + ".");
        }

        long timeExpressedUsingTheDefinedTimeUnitGranularityType_temp = startTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp;

        while (timeExpressedUsingTheDefinedTimeUnitGranularityType_temp <= endTimeExpressedUsingTheDefinedTimeUnitGranularityType_temp) {
            ResourcesPoolPlanningByTimeUnit resourcesPoolPlanningByTimeUnit_temp = relevantResourcesPoolOverallPlanning
                .getResourcesReservationPlanning().get(timeExpressedUsingTheDefinedTimeUnitGranularityType_temp);
            if (resourcesPoolPlanningByTimeUnit_temp == null) {
                // XXX this case should be handled in order to enhance the
                // robustness.
                // nothing to do.
            } else {
                // A ResourcesPoolPlanningByTimeUnit
                // exists for this timeUnit. Let's delete things that should be
                // deleted.

                // Retrieving reservation information via
                // persistedResourcesReservation_temp_.

                // Updating the resources availability by adding the
                // reservation's amount.
                // -1 means unbound.
                if (resourcesPoolPlanningByTimeUnit_temp.getAvailableCPUResourcesForThisResourcePoolInThisTimeUnit() == -1) {
                    // nothing to do.
                } else {
                    resourcesPoolPlanningByTimeUnit_temp
                        .setAvailableCPUResourcesForThisResourcePoolInThisTimeUnit(resourcesPoolPlanningByTimeUnit_temp
                            .getAvailableCPUResourcesForThisResourcePoolInThisTimeUnit()
                            + persistedResourcesReservation_temp.getReservedNumberOfCPUCore());
                }

                if (resourcesPoolPlanningByTimeUnit_temp.getAvailableMemoryResourcesForThisResourcePoolInThisTimeUnit() == -1) {
                    // nothing to do.
                } else {
                    resourcesPoolPlanningByTimeUnit_temp
                        .setAvailableMemoryResourcesForThisResourcePoolInThisTimeUnit(resourcesPoolPlanningByTimeUnit_temp
                            .getAvailableMemoryResourcesForThisResourcePoolInThisTimeUnit()
                            + persistedResourcesReservation_temp.getReservedMemorySizeInMB());
                }

                if (resourcesPoolPlanningByTimeUnit_temp.getAvailableStorageResourcesForThisResourcePoolInThisTimeUnit() == -1) {
                    // nothing to do.
                } else {
                    resourcesPoolPlanningByTimeUnit_temp
                        .setAvailableStorageResourcesForThisResourcePoolInThisTimeUnit(resourcesPoolPlanningByTimeUnit_temp
                            .getAvailableStorageResourcesForThisResourcePoolInThisTimeUnit()
                            + persistedResourcesReservation_temp.getReservedStorageSizeInMB());
                }

                // Updating
                // ProjectHavingAResourcesReservationInThisTimeUnit
                // for this resourcesPoolPlanningByTimeUnit_temp.
                Hashtable<String, ProjectPlanningByTimeUnit> projectHavingAResourcesReservationInThisTimeUnit_temp = resourcesPoolPlanningByTimeUnit_temp
                    .getProjectHavingAResourcesReservationInThisTimeUnit();

                String projectId_temp = persistedResourcesReservation_temp.getProjectId();

                ProjectPlanningByTimeUnit projectPlanningByTimeUnit_temp = projectHavingAResourcesReservationInThisTimeUnit_temp
                    .get(projectId_temp);

                if (projectPlanningByTimeUnit_temp == null) {
                    // XXX this case should be handled in order to enhance the
                    // robustness.
                    // nothing to do.
                } else {
                    // Updating projectPlanningByTimeUnit_temp associated to
                    // this projectId_temp (it exists already).

                    // This reservation is removed from
                    // projectPlanningByTimeUnit_temp.
                    projectPlanningByTimeUnit_temp.getReservationIdSet().remove(persistedResourcesReservation_temp.getId());

                    // Retrieving the resourcesQuotaMax of the project
                    // identified by
                    // persistedResourcesReservation_temp_.getProjectId()
                    Integer availableCPUResourcesForThisProjectInThisTimeUnit_PRE = null;
                    Integer availableMemoryResourcesForThisProjectInThisTimeUnit_PRE = null;
                    Integer availableStorageResourcesForThisProjectInThisTimeUnit_PRE = null;

                    // Retrieving
                    // availableXXXXXResourcesForThisProjectInThisTimeUnit_PRE,
                    // and updating them.

                    // -1 means unbound (was: 0 means unbound).
                    availableCPUResourcesForThisProjectInThisTimeUnit_PRE = projectPlanningByTimeUnit_temp
                        .getAvailableCPUResourcesForThisProjectInThisTimeUnit();
                    Integer availableCPUResourcesForThisProjectInThisTimeUnit_POST = null;
                    if (availableCPUResourcesForThisProjectInThisTimeUnit_PRE == -1) {
                        availableCPUResourcesForThisProjectInThisTimeUnit_POST = availableCPUResourcesForThisProjectInThisTimeUnit_PRE;
                    } else {
                        availableCPUResourcesForThisProjectInThisTimeUnit_POST = availableCPUResourcesForThisProjectInThisTimeUnit_PRE
                            + persistedResourcesReservation_temp.getReservedNumberOfCPUCore();
                    }

                    // -1 means unbound (was: 0 means unbound).
                    availableMemoryResourcesForThisProjectInThisTimeUnit_PRE = projectPlanningByTimeUnit_temp
                        .getAvailableMemoryResourcesForThisProjectInThisTimeUnit();
                    Integer availableMemoryResourcesForThisProjectInThisTimeUnit_POST = null;
                    if (availableMemoryResourcesForThisProjectInThisTimeUnit_PRE == -1) {
                        availableMemoryResourcesForThisProjectInThisTimeUnit_POST = availableMemoryResourcesForThisProjectInThisTimeUnit_PRE;
                    } else {
                        availableMemoryResourcesForThisProjectInThisTimeUnit_POST = availableMemoryResourcesForThisProjectInThisTimeUnit_PRE
                            + persistedResourcesReservation_temp.getReservedMemorySizeInMB();
                    }

                    // -1 means unbound (was: 0 means unbound).
                    availableStorageResourcesForThisProjectInThisTimeUnit_PRE = projectPlanningByTimeUnit_temp
                        .getAvailableStorageResourcesForThisProjectInThisTimeUnit();
                    Integer availableStorageResourcesForThisProjectInThisTimeUnit_POST = null;
                    if (availableStorageResourcesForThisProjectInThisTimeUnit_PRE == -1) {
                        availableStorageResourcesForThisProjectInThisTimeUnit_POST = availableStorageResourcesForThisProjectInThisTimeUnit_PRE;
                    } else {
                        availableStorageResourcesForThisProjectInThisTimeUnit_POST = availableStorageResourcesForThisProjectInThisTimeUnit_PRE
                            + persistedResourcesReservation_temp.getReservedStorageSizeInMB();
                    }

                    projectPlanningByTimeUnit_temp
                        .setAvailableCPUResourcesForThisProjectInThisTimeUnit(availableCPUResourcesForThisProjectInThisTimeUnit_POST);
                    projectPlanningByTimeUnit_temp
                        .setAvailableMemoryResourcesForThisProjectInThisTimeUnit(availableMemoryResourcesForThisProjectInThisTimeUnit_POST);
                    projectPlanningByTimeUnit_temp
                        .setAvailableStorageResourcesForThisProjectInThisTimeUnit(availableStorageResourcesForThisProjectInThisTimeUnit_POST);

                    projectPlanningByTimeUnit_temp.setReservationIdSet(projectPlanningByTimeUnit_temp.getReservationIdSet());

                    if (projectPlanningByTimeUnit_temp.getReservationIdSet().isEmpty()) {
                        // This project doesn't have anymore reservation, as a
                        // consequence, it must be deleted from
                        // projectHavingAResourcesReservationInThisTimeUnit_temp.
                        projectHavingAResourcesReservationInThisTimeUnit_temp.remove(projectId_temp);
                    } else {
                        projectHavingAResourcesReservationInThisTimeUnit_temp.put(projectId_temp,
                            projectPlanningByTimeUnit_temp);
                    }
                    resourcesPoolPlanningByTimeUnit_temp
                        .setProjectHavingAResourcesReservationInThisTimeUnit(projectHavingAResourcesReservationInThisTimeUnit_temp);
                }
            }
            timeExpressedUsingTheDefinedTimeUnitGranularityType_temp = timeExpressedUsingTheDefinedTimeUnitGranularityType_temp + 1;
        }
    }

    @Override
    public Integer describeResourcesReservationAvailability(final String projectId, final Calendar startTime,
        final Calendar endTime, final Integer numberOfCPUCore, final Integer memorySizeInMB, final Integer storageSizeInMB,
        final String hypervisorType, final String location) throws InvalidArgumentException, NoRelevantResourcesPoolException {

        ResourcesReservationModuleImpl.log.finest("Enters in: describeResourcesReservationAvailability(" + projectId + ", "
            + startTime + ", " + endTime + ", " + numberOfCPUCore + ", " + memorySizeInMB + ", " + storageSizeInMB + ", "
            + hypervisorType + ", " + location + ").");

        Integer result = 0;

        // Let's retrieve the relevant resourcesReservationPlanning.
        Iterator<ResourcesPoolOverallPlanning> rPOP_temp_ITERATOR = this.resourcesPoolOverallPlanning_List.iterator();
        ResourcesPoolOverallPlanning relevantResourcesPoolOverallPlanning = null;
        while (rPOP_temp_ITERATOR.hasNext()) {
            ResourcesPoolOverallPlanning rPOP_temp = rPOP_temp_ITERATOR.next();
            if ((hypervisorType.equalsIgnoreCase(rPOP_temp.getResourcePoolHypervisorType()))
                && (location.equalsIgnoreCase(rPOP_temp.getResourcePoolLocation()))) {
                relevantResourcesPoolOverallPlanning = rPOP_temp;
            } else {
                // nothing to do.
            }
        }

        if (relevantResourcesPoolOverallPlanning != null) {
            // nothing to do. Let's continue.
        } else {
            throw new NoRelevantResourcesPoolException("No resourcesPool can be found for the given hypervisorType: "
                + hypervisorType + ", and location: " + location + ".");
        }

        Integer numberOfCPUCore_temp = numberOfCPUCore;
        Integer memorySizeInMB_temp = memorySizeInMB;
        Integer storageSizeInMB_temp = storageSizeInMB;

        long startTimeExpressedUsingTheDefinedTimeUnitGranularityType;
        long endTimeExpressedUsingTheDefinedTimeUnitGranularityType;

        if (TimeUnitGranularityType.HOUR.equals(this.timeUnitGranularityType)) {
            startTimeExpressedUsingTheDefinedTimeUnitGranularityType = startTime.getTimeInMillis() / (60 * 60 * 1000);
            endTimeExpressedUsingTheDefinedTimeUnitGranularityType = endTime.getTimeInMillis() / (60 * 60 * 1000);
        } else if (TimeUnitGranularityType.DAY.equals(this.timeUnitGranularityType)) {
            startTimeExpressedUsingTheDefinedTimeUnitGranularityType = startTime.getTimeInMillis() / (24 * 60 * 60 * 1000);
            endTimeExpressedUsingTheDefinedTimeUnitGranularityType = endTime.getTimeInMillis() / (24 * 60 * 60 * 1000);
        } else if (TimeUnitGranularityType.WEEK.equals(this.timeUnitGranularityType)) {
            // Let's find monday.
            Calendar startTime_temp = startTime;
            // If it is not monday, let's move in the past (a day backward) to
            // find the first previous monday.
            while (Calendar.MONDAY != startTime_temp.get(Calendar.DAY_OF_WEEK)) {
                startTime_temp.setTimeInMillis(startTime_temp.getTimeInMillis() - 24 * 60 * 60 * 1000);
            }
            startTimeExpressedUsingTheDefinedTimeUnitGranularityType = startTime_temp.getTimeInMillis()
                / (7 * 24 * 60 * 60 * 1000);

            // Let's find sunday.
            Calendar endTime_temp = endTime;
            // If it is not sunday, let's move in the future (a day forward) to
            // find the first next sunday.
            while (Calendar.SUNDAY != endTime_temp.get(Calendar.DAY_OF_WEEK)) {
                endTime_temp.setTimeInMillis(endTime_temp.getTimeInMillis() + 24 * 60 * 60 * 1000);
            }
            endTimeExpressedUsingTheDefinedTimeUnitGranularityType = endTime_temp.getTimeInMillis() / (7 * 24 * 60 * 60 * 1000);
        } else {
            throw new InvalidArgumentException("The " + this.getClass().getName() + " module supports only: "
                + TimeUnitGranularityType.HOUR + ", " + TimeUnitGranularityType.DAY + ", or " + TimeUnitGranularityType.WEEK
                + " time unit granularity types.");
        }

        Boolean trueIfTheReservationWithTheGivenAmoutOfResourcesIsReservable = true;

        // The idea is to find the maximal number of reservations that are
        // reservable. The given parameters represent a reservation.
        while (trueIfTheReservationWithTheGivenAmoutOfResourcesIsReservable) {

            // Check of all the reservation's timeUnits.
            long timeExpressedUsingTheDefinedTimeUnitGranularityType_temp = startTimeExpressedUsingTheDefinedTimeUnitGranularityType;

            try {

                while (timeExpressedUsingTheDefinedTimeUnitGranularityType_temp <= endTimeExpressedUsingTheDefinedTimeUnitGranularityType) {
                    this.checkTheReservationFeasabilityWithRegardToResourcesQuotaComingFromResourcesPoolAndProject(
                        relevantResourcesPoolOverallPlanning, projectId,
                        timeExpressedUsingTheDefinedTimeUnitGranularityType_temp, numberOfCPUCore_temp, memorySizeInMB_temp,
                        storageSizeInMB_temp);

                    timeExpressedUsingTheDefinedTimeUnitGranularityType_temp = timeExpressedUsingTheDefinedTimeUnitGranularityType_temp + 1;
                }
                result = result + 1;
                numberOfCPUCore_temp = numberOfCPUCore_temp * 2;
                memorySizeInMB_temp = memorySizeInMB_temp * 2;
                storageSizeInMB_temp = storageSizeInMB_temp * 2;
            } catch (InsufficientResourceException e) {
                trueIfTheReservationWithTheGivenAmoutOfResourcesIsReservable = false;
            } catch (InvalidArgumentException e) {
                trueIfTheReservationWithTheGivenAmoutOfResourcesIsReservable = false;
            }
        }
        return result;
    }

    @Override
    public void configureBookingPeriod(final Integer bookingPeriodExpressedInTimeUnit) throws InvalidArgumentException {
        ResourcesReservationModuleImpl.log.finest("Enters in: configureBookingPeriod(" + bookingPeriodExpressedInTimeUnit
            + ").");
        if (bookingPeriodExpressedInTimeUnit == null) {
            ResourcesReservationModuleImpl.log.finest("Leaves: configureBookingPeriod(...).");
            throw new InvalidArgumentException("The given bookingPeriodExpressedInTimeUnit is ("
                + bookingPeriodExpressedInTimeUnit + ").");
        } else if (bookingPeriodExpressedInTimeUnit < 0) {
            ResourcesReservationModuleImpl.log.finest("Leaves: configureBookingPeriod(...).");
            throw new InvalidArgumentException("The given bookingPeriodExpressedInTimeUnit is strictly less than 0: ("
                + bookingPeriodExpressedInTimeUnit + ").");
        } else {
            this.bookingPeriodExpressedInTimeUnit = bookingPeriodExpressedInTimeUnit;
            ResourcesReservationModuleImpl.log.finest("Leaves: configureBookingPeriod(...).");
        }
    }

    @Override
    public final Integer getBookingPeriodExpressedInTimeUnit() throws InvalidArgumentException {
        return this.bookingPeriodExpressedInTimeUnit;
    }

    @Override
    public Boolean deleteAReservationFromDataBase(final Integer reservationId) throws InvalidArgumentException {
        ResourcesReservationModuleImpl.log.finest("Enters in: deleteAReservationFromDataBase(" + reservationId + ").");

        Boolean result;

        ResourcesReservation rR;
        Query rREB_list_query = this.em.createQuery(
            "FROM " + ResourcesReservation.class.getSimpleName() + " p WHERE p.id=:reservationId").setParameter(
            "reservationId", reservationId);
        if (rREB_list_query == null) {
            throw new InvalidArgumentException("The given reservationId (" + reservationId + ") is unknown to Sirocco.");
        } else {
            @SuppressWarnings("unchecked")
            List<ResourcesReservation> rREB_list = rREB_list_query.getResultList();
            if (rREB_list.size() == 0) {
                throw new InvalidArgumentException("The given reservationId (" + reservationId + ") is unknown to Sirocco.");
            } else {
                rR = rREB_list.get(0);
                ResourcesReservationModuleImpl.log.finest("Delete reservation: " + rR.getId() + ".");
                this.em.remove(rR);
                this.em.flush();
                result = Boolean.TRUE;
            }
        }

        ResourcesReservationModuleImpl.log.finest("Leaves: deleteAReservationFromDataBase(" + reservationId + ").");
        return result;
    }
}
